Package: socialmixr
Title: Social Mixing Matrices for Infectious Disease Modelling
Version: 0.6.0
Authors@R: c(
    person("Sebastian", "Funk", , "sebastian.funk@lshtm.ac.uk", role = c("aut", "cre")),
    person("Lander", "Willem", role = "aut"),
    person("Hugo", "Gruson", role = "aut"),
    person("Nicholas", "Tierney", , "nicholas.tierney@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-1460-8722")),
    person("Maria", "Bekker-Nielsen Dunbar", role = "ctb"),
    person("Carl A. B.", "Pearson", role = "ctb"),
    person("Sam", "Clifford", , "sj.clifford@gmail.com", role = "ctb"),
    person("Christopher", "Jarvis", role = "ctb"),
    person("Alexis", "Robert", , "alexis.robert@lshtm.ac.uk", role = "ctb"),
    person("Niel", "Hens", role = "ctb"),
    person("Pietro", "Coletti", role = c("col", "dtm")),
    person("Lloyd", "Chapman", , "l.chapman4@lancaster.ac.uk", role = "ctb")
  )
Description: Methods for sampling contact matrices from diary data for
    use in infectious disease modelling, as discussed in Mossong et al.
    (2008) <doi:10.1371/journal.pmed.0050074>.
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Imports: checkmate, countrycode, curl, data.table, grDevices, httr,
        jsonlite, lifecycle, lubridate, memoise, purrr, oai, wpp2017,
        xml2, cli, rlang, methods
Suggests: contactsurveys, ggplot2, here, knitr, quarto, reshape2,
        rmarkdown, roxyglobals (>= 1.0.0), testthat, withr
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
RoxygenNote: 7.3.3
URL: https://github.com/epiforecasts/socialmixr,
        https://epiforecasts.io/socialmixr/
BugReports: https://github.com/epiforecasts/socialmixr/issues
Config/testthat/edition: 3
Packaged: 2026-04-28 17:43:28 UTC; sebfunk
Author: Sebastian Funk [aut, cre],
  Lander Willem [aut],
  Hugo Gruson [aut],
  Nicholas Tierney [aut] (ORCID: <https://orcid.org/0000-0003-1460-8722>),
  Maria Bekker-Nielsen Dunbar [ctb],
  Carl A. B. Pearson [ctb],
  Sam Clifford [ctb],
  Christopher Jarvis [ctb],
  Alexis Robert [ctb],
  Niel Hens [ctb],
  Pietro Coletti [col, dtm],
  Lloyd Chapman [ctb]
Maintainer: Sebastian Funk <sebastian.funk@lshtm.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-29 06:40:18 UTC
Built: R 4.5.2; ; 2026-05-01 18:05:45 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",contact_survey)
S3method(as.matrix,contact_matrix)
S3method(check,contact_survey)
S3method(clean,contact_survey)
S3method(plot,contact_matrix)
S3method(print,contact_matrix)
export(agegroups_to_limits)
export(as_contact_survey)
export(assign_age_groups)
export(check)
export(clean)
export(compute_matrix)
export(contact_age_distribution)
export(contact_matrix)
export(download_survey)
export(get_citation)
export(get_survey)
export(limits_to_agegroups)
export(list_surveys)
export(load_survey)
export(matrix_plot)
export(per_capita)
export(pop_age)
export(reduce_agegroups)
export(split_matrix)
export(survey)
export(survey_countries)
export(survey_country_population)
export(symmetrise)
export(weigh)
export(wpp_age)
export(wpp_countries)
import(data.table)
import(wpp2017)
importFrom(checkmate,assert_character)
importFrom(checkmate,assert_data_frame)
importFrom(checkmate,assert_list)
importFrom(checkmate,assert_names)
importFrom(countrycode,countrycode)
importFrom(curl,curl_download)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fcase)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,uniqueN)
importFrom(grDevices,heat.colors)
importFrom(graphics,axis)
importFrom(graphics,image)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(httr,GET)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(httr,status_code)
importFrom(httr,user_agent)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecated)
importFrom(lubridate,period)
importFrom(lubridate,period_to_seconds)
importFrom(lubridate,years)
importFrom(memoise,memoise)
importFrom(methods,formalArgs)
importFrom(oai,list_records)
importFrom(purrr,walk)
importFrom(rlang,"%||%")
importFrom(stats,median)
importFrom(stats,runif)
importFrom(stats,xtabs)
importFrom(utils,bibentry)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,hasName)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_text)

# Social mixing matrices for infectious disease modelling in R

<!-- badges: start -->

[![CRAN
version](https://www.r-pkg.org/badges/version-ago/socialmixr)](https://cran.r-project.org/package=socialmixr)
[![R-CMD-check](https://github.com/epiforecasts/socialmixr/workflows/R-CMD-check/badge.svg)](https://github.com/epiforecasts/socialmixr/actions)
[![Codecov test
coverage](https://codecov.io/gh/epiforecasts/socialmixr/graph/badge.svg)](https://app.codecov.io/gh/epiforecasts/socialmixr)
<!-- badges: end -->

[socialmixr](https://github.com/epiforecasts/socialmixr) is an `R`
package to derive social mixing matrices from survey data.

# Installation

The package can be installed using

``` r
install.packages("socialmixr")
```

The current development version can be installed using the `remotes`
package

``` r
remotes::install_github("epiforecasts/socialmixr")
```

# Usage

Contact matrices are computed through a small pipeline of composable
functions: subsetting the survey, assigning age groups, optionally
weighing participants, and computing the matrix. A minimal example using
the included POLYMOD data:

``` r
library(socialmixr)
data(polymod)

polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix()
```

Post-processing functions `symmetrise()`, `split_matrix()` and
`per_capita()` can be piped after `compute_matrix()` to enforce
symmetry, decompose the matrix, or convert to per-capita contact rates.

# Documentation

For more on how to use the socialmixr package, see the [introduction
vignette](https://epiforecasts.io/socialmixr/articles/socialmixr.html).

## Contributors

<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->
<!-- prettier-ignore-start -->
<!-- markdownlint-disable -->

All contributions to this project are gratefully acknowledged using the
[`allcontributors` package](https://github.com/ropensci/allcontributors)
following the [allcontributors](https://allcontributors.org)
specification. Contributions of any kind are welcome!

### Code

<a href="https://github.com/epiforecasts/socialmixr/commits?author=sbfnk">sbfnk</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=Bisaloo">Bisaloo</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=lwillem">lwillem</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=njtierney">njtierney</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=alxsrobert">alxsrobert</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=Degoot-AM">Degoot-AM</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=pearsonca">pearsonca</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=jarvisc1">jarvisc1</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=jamesmbaazam">jamesmbaazam</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=LloydChapman">LloydChapman</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=mariabnd">mariabnd</a>,
<a href="https://github.com/epiforecasts/socialmixr/commits?author=ukhsa-tt">ukhsa-tt</a>

### Issue Authors

<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Abastistician">bastistician</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3ABlackEdder">BlackEdder</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3APinzo1">Pinzo1</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Aflorpi">florpi</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Acchauve">cchauve</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Athutran">thutran</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Adlaydon">dlaydon</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Adeusthindwa">deusthindwa</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Akrivit">krivit</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Alinyang17">linyang17</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Acliu822">cliu822</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3ATimTaylor">TimTaylor</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3ANaomiWaterlow">NaomiWaterlow</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Aaakhmetz">aakhmetz</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Aadamkucharski">adamkucharski</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Achitrams">chitrams</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3AIsaacStopard">IsaacStopard</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Aavallecam">avallecam</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Amaishaoshao">maishaoshao</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3AFrancescoBonacina">FrancescoBonacina</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+author%3Alucy-gf">lucy-gf</a>

### Issue Contributors

<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+commenter%3Avikkytom">vikkytom</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+commenter%3Abahadzie">bahadzie</a>,
<a href="https://github.com/epiforecasts/socialmixr/issues?q=is%3Aissue+commenter%3Ajoshwlambert">joshwlambert</a>

<!-- markdownlint-enable -->
<!-- prettier-ignore-end -->
<!-- ALL-CONTRIBUTORS-LIST:END -->

library(socialmixr)
library(here)
library(cli)

cli_h1("Starting survey download...")

## list all surveys
survey_list <- list_surveys()
cli_h1("Found {nrow(survey_list)} surveys to download")

dir.create(here("surveys"), showWarnings = FALSE)

## download all surveys using the `url` column in the survey list and
## save them in the `surveys` folder (which is created if it does not exist)
survey_files <- purrr::map(survey_list$url, function(x) {
  tryCatch(
    {
      f <- download_survey(x, here("surveys"))
      Sys.sleep(2) # Be nice to the server
      f
    },
    error = function(e) {
      cli::cli_alert_warning(
        "Download failed for {.url {x}}: {conditionMessage(e)}"
      )
      NULL
    }
  )
})

## name list elements according to url
names(survey_files) <- glue::glue("{survey_list$title} ({survey_list$url})")

## save list of survey files
saveRDS(survey_files, here("surveys", "survey_files.rds"))

cli_h1("Download phase completed")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
data.table::setDTthreads(1)

## -----------------------------------------------------------------------------
library(socialmixr)
library(ggplot2)
data(polymod)

## -----------------------------------------------------------------------------
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix()

## -----------------------------------------------------------------------------
uk_grouped <- polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15))

head(uk_grouped$participants[, c("part_id", "part_age", "age.group")])
head(uk_grouped$contacts[, c("part_id", "cnt_age", "contact.age.group")])

## -----------------------------------------------------------------------------
unique(polymod$participants$country)

## -----------------------------------------------------------------------------
polymod[country %in% c("United Kingdom", "Germany")]

## -----------------------------------------------------------------------------
bootstrap <- function(survey) {
  sampled_ids <- sample(
    unique(survey$participants$part_id),
    replace = TRUE
  )
  survey$participants <- survey$participants[
    list(sampled_ids), on = "part_id"
  ]
  survey$contacts <- survey$contacts[
    list(sampled_ids),
    on = "part_id",
    nomatch = NULL,
    allow.cartesian = TRUE
  ]
  survey
}

uk <- polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15))

m <- suppressWarnings(
  replicate(n = 5, uk |> bootstrap() |> compute_matrix())
)
mr <- Reduce("+", lapply(m["matrix", ], function(x) x / ncol(m)))
mr

## ----eval=requireNamespace("wpp2024", quietly = TRUE), message=FALSE, warning=FALSE----
# data("popAge1dt", package = "wpp2024")
# uk_pop <- popAge1dt[name == "United Kingdom" & year == 2020,
#   .(lower.age.limit = age, population = pop * 1000)
# ]
# head(uk_pop)

## -----------------------------------------------------------------------------
custom_pop <- data.frame(
  lower.age.limit = c(0, 18, 60),
  population = c(12000000, 35000000, 20000000)
)

## -----------------------------------------------------------------------------
survey_country_population(polymod, countries = "United Kingdom")

## ----message=FALSE, warning=FALSE---------------------------------------------
uk_pop <- survey_country_population(polymod, countries = "United Kingdom")

polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix() |>
  symmetrise(survey_pop = uk_pop)

## ----message=FALSE, warning=FALSE---------------------------------------------
de_pop <- survey_country_population(polymod, countries = "Germany")

polymod[country == "Germany"] |>
  assign_age_groups(age_limits = c(0, 60)) |>
  compute_matrix() |>
  symmetrise(survey_pop = de_pop) |>
  per_capita(survey_pop = de_pop)

## -----------------------------------------------------------------------------
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix() |>
  split_matrix(survey_pop = uk_pop)

## ----warning=FALSE, message=FALSE---------------------------------------------
# contact matrix for school-related contacts
polymod[cnt_school == 1] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

# contact matrix for work-related contacts involving physical contact
polymod[cnt_work == 1][phys_contact == 1] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

# contact matrix for daily contacts at home with males
polymod[cnt_home == 1][cnt_gender == "M"][duration_multi == 5] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

## ----message=FALSE, warning=FALSE---------------------------------------------
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
  weigh("part_age", target = uk_pop) |>
  compute_matrix()

## ----message=FALSE, warning=FALSE---------------------------------------------
polymod |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("hh_size") |>
  compute_matrix()

## ----message=FALSE, warning=FALSE---------------------------------------------
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
  weigh("part_age", target = uk_pop) |>
  compute_matrix(weight_threshold = 3)

## ----echo=FALSE---------------------------------------------------------------
survey_data <- data.frame(
  age = c(1, 1, 2, 2, 2, 3),
  day.of.week = as.factor(c(
    "weekend", "weekend", "weekend",
    "week", "week", "week"
  )),
  age.group = NA,
  m_i = c(3, 2, 9, 10, 8, 15)
)

# age groups 1-2 and 3
survey_data$age.group <- 1 - (survey_data$age < 3) + 1
survey_data$age.group <- as.factor(c("A", "B"))[survey_data$age.group]

knitr::kable(survey_data)

## -----------------------------------------------------------------------------
N <- 6
N_age <- c(2, 3, 1)
N_age.group <- c(5, 1)
N_day.of.week <- c(3, 3)

P <- 3000
P_age <- c(1000, 1000, 1000)
P_age.group <- c(2000, 1000)

P_day.of.week <- c(5 / 7, 2 / 7) * 3000

## ----echo=FALSE---------------------------------------------------------------
cat(paste(
  "unweighted average number of contacts:",
  round(mean(survey_data$m_i), digits = 2)
))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(aggregate(
  m_i ~ age + age.group, data = survey_data, mean
))

## ----echo=FALSE---------------------------------------------------------------
# including population constants
survey_data$w <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w[i] <- (P_day.of.week[day_i] / P) / (N_day.of.week[day_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# without population constants
survey_data$w_dot <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w_dot[i] <- (P_day.of.week[day_i]) / (N_day.of.week[day_i])
}
survey_data$w_dot_tilde <- survey_data$w_dot / sum(survey_data$w_dot) * N

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)

# print
knitr::kable(survey_data)

# remove the 'dot' weights
survey_data$w_dot <- NULL
survey_data$w_dot_tilde <- NULL

## ----echo=FALSE---------------------------------------------------------------
# add weighted number of contacts
survey_data["m_i * w_tilde"] <- survey_data$m_i * survey_data$w_tilde

# show table
knitr::kable(survey_data)

# remove the weighted number of contacts
survey_data$`m_i * w_tilde` <- NULL

cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))

## ----echo=FALSE---------------------------------------------------------------
survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] /
    sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)

knitr::kable(survey_data)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(aggregate(
  m_i * w_PS ~ age.group,
  data = survey_data, mean
))

## ----echo=FALSE---------------------------------------------------------------
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] /
    sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)

# print
knitr::kable(survey_data)

cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(aggregate(m_i * w_PS ~ age.group, data = survey_data, mean))

## ----echo=FALSE---------------------------------------------------------------
survey_data <- survey_data[c(rep(1:2, 5), 3:6), ]
survey_data$m_i[survey_data$age == 3] <- 30
rownames(survey_data) <- NULL

survey_data <- survey_data[order(survey_data$age), ]
survey_data$w <- NULL
survey_data$w_tilde <- NULL
survey_data$w_PS <- NULL

## ----echo=FALSE---------------------------------------------------------------
N <- nrow(survey_data)
N_age <- table(survey_data$age)
N_age.group <- table(survey_data$age.group)
N_day.of.week <- table(survey_data$day.of.week)

## ----echo=FALSE---------------------------------------------------------------
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)
knitr::kable(survey_data)

cat(paste(
  "unweighted average number of contacts:",
  round(mean(survey_data$m_i), digits = 2)
))
cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))

## ----echo=FALSE---------------------------------------------------------------
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3

## ----echo=FALSE---------------------------------------------------------------
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3

cat(paste(
  "weighted average number of contacts after truncation:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))

## ----fig.width=4, fig.height=4------------------------------------------------
df <- reshape2::melt(
  mr,
  varnames = c("age.group", "age.group.contact"),
  value.name = "contacts"
)
ggplot(df, aes(x = age.group, y = age.group.contact, fill = contacts)) +
  theme(legend.position = "bottom") +
  geom_tile()

## ----fig.width=4, fig.height=4------------------------------------------------
matrix_plot(mr)
matrix_plot(mr, color.palette = gray.colors)

library(socialmixr)
library(purrr)
library(here)
library(cli)

cli_h1("Starting survey file checks...")

## load list of survey files
survey_files <- readRDS(here("surveys", "survey_files.rds"))

cli_h1("Loaded {length(survey_files)} survey files")

## define safe checking function
safe_check <- safely(\(files) socialmixr::check(load_survey(files)))

## check all surveys
cli_h1("Checking all surveys...")
checks <- map(survey_files, safe_check)

errors <- map(checks, "error")
no_error <- map_lgl(errors, is.null)
error_messages <- map(errors[!no_error], "message")

cli_h1("Survey check results")
cli_li("Total surveys: {length(survey_files)}")
cli_li("Successful checks: {sum(no_error)}")
cli_li("Failed checks: {sum(!no_error)}")

if (sum(!no_error) > 0) {
  iwalk(
    error_messages,
    ~ {
      cli_li("Survey: {.y}")
      cli_li("Error: {.x}")
    }
  )
}

# Capture warnings
if (length(warnings()) > 0) {
  cli_h1("Warnings")
  print(warnings())
}

# Create markdown summary (useful for CLI, GitHub step summary, and issues)
summary_md <- c(
  "# Survey check summary",
  sprintf("- Total: %d", length(survey_files)),
  sprintf("- Passed: %d", sum(no_error)),
  sprintf("- Failed: %d", sum(!no_error))
)

if (sum(!no_error) > 0) {
  summary_md <- c(
    summary_md,
    "",
    "## Failed surveys",
    paste0("- ", names(error_messages))
  )
}

# Save summary to file (for CLI use and workflow to read)
writeLines(summary_md, here("surveys", "check_summary.md"))

# Also write to GitHub Actions step summary if available
sum_path <- Sys.getenv("GITHUB_STEP_SUMMARY")
if (nzchar(sum_path)) {
  writeLines(summary_md, sum_path, useBytes = TRUE)
}

# Report results but don't fail the workflow
# (Some surveys may always fail, so we log the issues but don't block CI)
if (sum(!no_error) > 0) {
  cli_h1("Some surveys failed checks. Results saved for review.")
} else {
  cli_h1("All surveys passed checks successfully!")
}

#' Download a survey from its Zenodo repository
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `download_survey()` has been deprecated in favour of
#'   `contactsurveys::download_survey()`.
#'
#' `download_survey()` downloads survey data from Zenodo.
#'
#' @param survey a URL (see `contactsurveys::list_surveys()`)
#' @param dir a directory to save the files to; if not given, will save to a
#'   temporary directory
#' @param sleep time to sleep between requests to avoid overloading the server
#'   (passed on to \code{\link[base]{Sys.sleep}})
#' @importFrom httr GET content status_code http_error config user_agent
#' @importFrom jsonlite fromJSON toJSON
#' @importFrom curl curl_download
#' @importFrom utils read.csv packageVersion
#' @importFrom xml2 xml_text xml_find_first xml_find_all xml_attr
#' @autoglobal
#' @examples
#' # we recommend using the contactsurveys package for download_survey()
#' \dontrun{
#' # if needed, discover surveys with:
#' contactsurveys::list_surveys()
#' peru_survey <- download_survey("https://doi.org/10.5281/zenodo.1095664")
#' # -->
#' peru_survey <- contactsurveys::download_survey(
#'   "https://doi.org/10.5281/zenodo.1095664"
#' )
#' }
#' @return a vector of filenames that can be used with [load_survey]
#  @seealso load_survey
#' @export
download_survey <- function(survey, dir = NULL, sleep = 1) {
  lifecycle::deprecate_soft(
    when = "0.5.0",
    what = "download_survey()",
    with = "contactsurveys::download_survey()"
  )
  if (!is.character(survey) || length(survey) != 1 || is.na(survey)) {
    cli::cli_abort("{.arg survey} must be a single, non-NA character string.")
  }

  survey <- sub("^(https?:\\/\\/(dx\\.)?doi\\.org\\/|doi:)", "", survey)
  survey <- sub("#.*$", "", survey)
  is.doi <- is_doi(survey)
  is.url <- is.doi || grepl("^https?:\\/\\/", survey)

  if (!is.url) {
    cli::cli_abort("{.arg survey} is not a DOI or URL.")
  }

  if (is.doi) {
    survey_url <- paste0("https://doi.org/", survey)
  } else {
    survey_url <- survey
  }

  temp_body <- GET(
    survey_url,
    config = config(
      followlocation = 1
    ),
    user_agent(paste0(
      "http://github.com/epiforecasts/socialmixr R package socialmixr/v.",
      packageVersion("socialmixr")
    ))
  )
  if (status_code(temp_body) == 404) {
    cli::cli_abort("DOI {.val {survey}} not found.")
  }
  if (http_error(temp_body)) {
    cli::cli_abort(
      c(
        "Failed to fetch the requested resource",
        "x" = "The website server returned an HTTP error", # nolint
        "i" = "Check your connection or the server status" # nolint
      )
    )
  }

  parsed_body <- content(temp_body, encoding = "UTF-8")
  parsed_cite <- fromJSON(
    xml_text(
      xml_find_first(parsed_body, '//script[@type="application/ld+json"]')
    )
  )

  reference <- list(
    title = parsed_cite$name,
    bibtype = "Misc",
    author = parsed_cite$author$name,
    year = data.table::year(parsed_cite$datePublished)
  )
  if ("version" %in% names(parsed_cite)) {
    reference[["note"]] <- paste("Version", parsed_cite$version)
  }
  reference[[ifelse(is.doi, "doi", "url")]] <- survey

  links <- xml_attr(
    xml_find_all(
      parsed_body,
      "//link[@type=\"text/csv\" and not(@rel=\"alternate\")]"
    ),
    "href"
  )

  zenodo_links <- data.table(url = links)
  ## only download csv files
  zenodo_links[, file_name := tolower(basename(url))]

  if (anyDuplicated(zenodo_links$file_name) > 0) {
    cli::cli_warn(
      c(
        "Zenodo repository contains files with names that only differ by case.",
        "!" = "This will cause unpredictable behaviour on case-insensitive \\
        file systems.",
        "i" = "Please contact the authors to get this fixed." # nolint
      )
    )
    zenodo_links <- zenodo_links[!duplicated(file_name)]
  }

  if (is.null(dir)) {
    dir <- tempdir()
  }

  cli::cli_inform("Getting {parsed_cite$name}.")

  lcs <- find_common_prefix(zenodo_links$file_name)
  reference_file_path <- file.path(dir, paste0(lcs, "reference.json"))
  reference_json <- toJSON(reference)
  write(reference_json, reference_file_path)

  files <- c(
    reference_file_path,
    vapply(
      seq_len(nrow(zenodo_links)),
      function(i) {
        zenodo_url <- zenodo_links[i, ]$url
        temp <- file.path(dir, zenodo_links[i, ]$file_name)
        message("Downloading ", zenodo_url)
        Sys.sleep(sleep)
        dl <- curl_download(zenodo_url, temp)
        temp
      },
      ""
    )
  )

  return(files)
}

find_common_prefix <- function(vec) {
  # find initial longest common prefix of file names
  i <- 1
  finish <- FALSE
  lcs <- ""
  while (!finish) {
    initial_bits <- vapply(vec, substr, start = 1, stop = i, "x")
    if (length(unique(initial_bits)) > 1) {
      finish <- TRUE
    } else {
      lcs <- unique(initial_bits)
      i <- i + 1
    }
  }

  lcs
}

##' Checks if a character string is a DOI
##'
##' @param x Character vector; the string or strings to check
##' @return Logical; \code{TRUE} if \code{x} is a DOI, \code{FALSE} otherwise
##' @author Sebastian Funk
is_doi <- function(x) {
  is.character(x) && grepl("^10.[0-9.]{4,}/[-._;()/:A-z0-9]+$", x)
}

test_that("download_survey() gives deprecation warning", {
  skip_if_offline("zenodo.org")
  skip_on_cran()
  skip_on_ci()
  expect_snapshot_warning(
    download_survey("https://zenodo.org/records/3874805")
  )
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
data.table::setDTthreads(1)

## -----------------------------------------------------------------------------
library(socialmixr)
library(ggplot2)
data(polymod)

## ----eval=FALSE---------------------------------------------------------------
# ?contact_matrix

## -----------------------------------------------------------------------------
contact_matrix(polymod, countries = "United Kingdom", age_limits = c(0, 1, 5, 15))

## -----------------------------------------------------------------------------
unique(polymod$participants$country)

## -----------------------------------------------------------------------------
m <- replicate(
  n = 5,
  contact_matrix(
    polymod,
    countries = "United Kingdom", age_limits = c(0, 1, 5, 15),
    sample_participants = TRUE
  )
)
mr <- Reduce("+", lapply(m["matrix", ], function(x) x / ncol(m)))
mr

## ----warning=FALSE, message=FALSE---------------------------------------------
contact_matrix(polymod,
  countries = "United Kingdom", age_limits = c(0, 20),
  return_demography = TRUE
)$demography

## -----------------------------------------------------------------------------
contact_matrix(polymod, countries = "United Kingdom", age_limits = c(0, 1, 5, 15), symmetric = TRUE)

## ----message=FALSE, warning=FALSE---------------------------------------------
contact_matrix(survey = polymod, countries = "Germany", age_limits = c(0, 60), symmetric = TRUE, per_capita = TRUE)

## -----------------------------------------------------------------------------
contact_matrix(polymod, countries = "United Kingdom", age_limits = c(0, 1, 5, 15), split = TRUE)

## ----warning=FALSE, message=FALSE---------------------------------------------
# contact matrix for school-related contacts
contact_matrix(polymod, age_limits = c(0, 20, 60), filter = list(cnt_school = 1))$matrix

# contact matrix for work-related contacts involving physical contact
contact_matrix(polymod, age_limits = c(0, 20, 60), filter = list(cnt_work = 1, phys_contact = 1))$matrix

# contact matrix for daily contacts at home with males
contact_matrix(polymod, age_limits = c(0, 20, 60), filter = list(cnt_home = 1, cnt_gender = "M", duration_multi = 5))$matrix

## ----message=FALSE, warning=FALSE---------------------------------------------
contact_matrix(
  survey = polymod, age_limits = c(0, 18, 60), weigh_dayofweek = TRUE,
  weigh_age = TRUE, return_part_weights = TRUE
)

## ----message=FALSE, warning=FALSE---------------------------------------------
# e.g. use household size as (dummy) weight to provide more importance to participant data from large households
contact_matrix(survey = polymod, age_limits = c(0, 18, 60), weights = "hh_size")

## ----message=FALSE, warning=FALSE---------------------------------------------
contact_matrix(
  survey = polymod, age_limits = c(0, 18, 60), weigh_dayofweek = TRUE,
  weigh_age = TRUE, return_part_weights = TRUE, weight_threshold = 3
)

## ----echo=FALSE---------------------------------------------------------------
survey_data <- data.frame(
  age = c(1, 1, 2, 2, 2, 3),
  day.of.week = as.factor(c("weekend", "weekend", "weekend", "week", "week", "week")),
  age.group = NA,
  m_i = c(3, 2, 9, 10, 8, 15)
)

# age groups 1-2 and 3
survey_data$age.group <- 1 - (survey_data$age < 3) + 1
survey_data$age.group <- as.factor(c("A", "B"))[survey_data$age.group]

knitr::kable(survey_data)

## -----------------------------------------------------------------------------
N <- 6
N_age <- c(2, 3, 1)
N_age.group <- c(5, 1)
N_day.of.week <- c(3, 3)

P <- 3000
P_age <- c(1000, 1000, 1000)
P_age.group <- c(2000, 1000)

P_day.of.week <- c(5 / 7, 2 / 7) * 3000

## ----echo=FALSE---------------------------------------------------------------
cat(paste("unweighted average number of contacts:", round(mean(survey_data$m_i), digits = 2)))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(aggregate(m_i ~ age + age.group, data = survey_data, mean))

## ----echo=FALSE---------------------------------------------------------------
# including population constants
survey_data$w <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w[i] <- (P_day.of.week[day_i] / P) / (N_day.of.week[day_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# without population constants
survey_data$w_dot <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w_dot[i] <- (P_day.of.week[day_i]) / (N_day.of.week[day_i])
}
survey_data$w_dot_tilde <- survey_data$w_dot / sum(survey_data$w_dot) * N

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)

# print
knitr::kable(survey_data)

# remove the 'dot' weights
survey_data$w_dot <- NULL
survey_data$w_dot_tilde <- NULL

## ----echo=FALSE---------------------------------------------------------------
# add weighted number of contacts
survey_data["m_i * w_tilde"] <- survey_data$m_i * survey_data$w_tilde

# show table
knitr::kable(survey_data)

# remove the weighted number of contacts
survey_data$`m_i * w_tilde` <- NULL

cat(paste("weighted average number of contacts:", round(mean(survey_data$m_i * survey_data$w_tilde), digits = 2)))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))

## ----echo=FALSE---------------------------------------------------------------
survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] / sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)

knitr::kable(survey_data)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(aggregate(m_i * w_PS ~ age.group, data = survey_data, mean))

## ----echo=FALSE---------------------------------------------------------------
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] / sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)

# print
knitr::kable(survey_data)

cat(paste("weighted average number of contacts:", round(mean(survey_data$m_i * survey_data$w_tilde), digits = 2)))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(aggregate(m_i * w_PS ~ age.group, data = survey_data, mean))

## ----echo=FALSE---------------------------------------------------------------
survey_data <- survey_data[c(rep(1:2, 5), 3:6), ]
survey_data$m_i[survey_data$age == 3] <- 30
rownames(survey_data) <- NULL

survey_data <- survey_data[order(survey_data$age), ]
survey_data$w <- NULL
survey_data$w_tilde <- NULL
survey_data$w_PS <- NULL

## ----echo=FALSE---------------------------------------------------------------
N <- nrow(survey_data)
N_age <- table(survey_data$age)
N_age.group <- table(survey_data$age.group)
N_day.of.week <- table(survey_data$day.of.week)

## ----echo=FALSE---------------------------------------------------------------
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)
knitr::kable(survey_data)

cat(paste("unweighted average number of contacts:", round(mean(survey_data$m_i), digits = 2)))
cat(paste("weighted average number of contacts:", round(mean(survey_data$m_i * survey_data$w_tilde), digits = 2)))

## ----echo=FALSE---------------------------------------------------------------
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3

## ----echo=FALSE---------------------------------------------------------------
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3

cat(paste("weighted average number of contacts after truncation:", round(mean(survey_data$m_i * survey_data$w_tilde), digits = 2)))

## ----fig.width=4, fig.height=4------------------------------------------------
df <- reshape2::melt(mr, varnames = c("age.group", "age.group.contact"), value.name = "contacts")
ggplot(df, aes(x = age.group, y = age.group.contact, fill = contacts)) +
  theme(legend.position = "bottom") +
  geom_tile()

## ----fig.width=4, fig.height=4------------------------------------------------
matrix_plot(mr)
matrix_plot(mr, color.palette = gray.colors)

#' List all countries and regions for which socialmixr has population data
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' This function is deprecated in favour of passing population data directly
#' to [contact_matrix()] via the `survey_pop` argument, which removes the need
#' for a country list. Additionally, the underlying `wpp2017` data is outdated.
#' For countries available in more recent WPP editions, use the `wpp2024`
#' package from GitHub.
#'
#' @details
#' Uses the World Population Prospects data from the `wpp2017` package.
#'
#' @return list of countries
#' @importFrom data.table fread setkey
#' @importFrom utils data
#' @importFrom countrycode countrycode
#' @autoglobal
#' @examples
#' if (requireNamespace("wpp2017", quietly = TRUE)) {
#'   wpp_countries()
#' }
#' @export
wpp_countries <- function() {
  lifecycle::deprecate_soft(
    "0.6.0",
    "wpp_countries()",
    details = c(
      "Pass population data directly via the \\
      {.arg survey_pop} argument instead.",
      i = "The underlying {.pkg wpp2017} data is also outdated; \\
           use {.pkg wpp2024} from GitHub for more recent data."
    )
  )

  if (!requireNamespace("wpp2017", quietly = TRUE)) {
    cli::cli_abort(
      c(
        "The {.pkg wpp2017} package is required but not installed.",
        i = "Install it with: {.code install.packages(\"wpp2017\")}, \\
             or use more recent data from {.pkg wpp2024} (GitHub only) \\
             and pass it directly via the {.arg survey_pop} argument."
      )
    )
  }

  popF <- fread(system.file("data", "popF.txt", package = "wpp2017"))
  popM <- fread(system.file("data", "popM.txt", package = "wpp2017"))

  countries <- unique(c(popF$country_code, popM$country_code))
  found_countries <-
    suppressWarnings(countrycode::countrycode(
      countries,
      "un",
      "country.name"
    ))
  found_countries[!is.na(found_countries)]
}

test_that("list of countries is not empty", {
  expect_gt(length(wpp_countries()), 0)
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_gt(length(survey_countries(polymod)), 0)
})

test_that("survey_countries() sends deprecation warning", {
  lifecycle::expect_deprecated(
    survey_countries(polymod)
  )
})

test_that("population data for 2015-2020 can be loaded", {
  suppressWarnings({
    expect_identical(wpp_age("Belgium", 2010)$year[1], 2010L)
    expect_identical(wpp_age("Belgium", 2011)$year[1], 2010L)
    expect_identical(wpp_age("Belgium", 2012)$year[1], 2010L)
    expect_identical(wpp_age("Belgium", 2013)$year[1], 2015L)
    expect_identical(wpp_age("Belgium", 2014)$year[1], 2015L)
    expect_identical(wpp_age("Belgium", 2019)$year[1], 2020L)
    expect_identical(wpp_age("Belgium", 2021)$year[1], 2020L)
  })
})

#' @export
clean <- function(x, ...) UseMethod("clean")

#' Normalise country names
#'
#' Uses the countrycode package to standardise country names. This handles
#' 2-letter ISO codes, 3-letter ISO codes, and full country names, converting
#' them all to standardised country names.
#'
#' @param countries A vector of country names or codes
#' @return A character vector of normalised country names
#' @importFrom data.table fcase
#' @importFrom countrycode countrycode
#' @keywords internal
#' @autoglobal
normalise_country_names <- function(countries) {
  origin_code <- fcase(
    all(nchar(as.character(countries)) == 2),
    "iso2c",
    all(nchar(as.character(countries)) == 3),
    "iso3c",
    default = "country.name"
  )
  converted <- suppressWarnings(countrycode(
    countries,
    origin_code,
    "country.name"
  ))
  converted[is.na(converted)] <- as.character(countries[is.na(converted)])
  converted
}

#' @name clean
#' @rdname clean
#' @title Clean contact survey data
#'
#' @description Cleans survey data to work with the 'contact_matrix' function
#'
#' @param x A [survey()] object
#' @param participant_age_column the column in `x$participants`
#'   containing participants' age
#' @param ... ignored
#' @param participant.age.column `r lifecycle::badge("deprecated")`
#'   Use `participant_age_column` instead.
#' @importFrom data.table fcase
#' @importFrom countrycode countrycode
#' @importFrom lubridate period_to_seconds period years
#' @return a cleaned survey in the correct format
#' @examples
#' data(polymod)
#' cleaned <- clean(polymod) # not really necessary, polymod is clean
#' @autoglobal
#' @export
clean.contact_survey <- function(
  x,
  participant_age_column = "part_age",
  ...,
  participant.age.column = deprecated()
) {
  chkDots(...)

  ## Handle deprecated arguments
  participant_age_column <- deprecate_arg(
    participant.age.column,
    participant_age_column,
    "participant.age.column",
    "participant_age_column",
    "clean"
  )

  ## update country names
  if ("country" %in% colnames(x$participants)) {
    x$participants[, country := factor(normalise_country_names(country))]
  }

  if (
    nrow(x$participants) > 0 &&
      participant_age_column %in% colnames(x$participants) &&
      (!is.numeric(x$participants[, get(participant_age_column)]) ||
        anyNA(x$participants[, get(participant_age_column)]))
  ) {
    ## set any entries not containing numbers to NA
    x$participants <- x$participants[,
      paste(participant_age_column) := fifelse(
        grepl("[0-9]", get(participant_age_column)),
        as.character(get(participant_age_column)),
        NA_character_
      )
    ]
    ## fix "under 1"
    x$participants <- x$participants[,
      paste(participant_age_column) := sub(
        "Under ",
        "0-",
        get(participant_age_column),
        fixed = TRUE
      )
    ]
    ## split off units
    if (
      any(grepl(
        " ",
        x$participants[, get(participant_age_column)],
        fixed = TRUE
      ))
    ) {
      ## extract numeric part and unit (e.g., "6 months" -> "6" and "months")
      x$participants <- x$participants[,
        c("..age.value", "..age.unit") := tstrsplit(
          as.character(get(participant_age_column)),
          " ",
          fixed = TRUE
        )
      ]
      ## update age column to just the numeric part
      x$participants <- x$participants[,
        paste(participant_age_column) := ..age.value
      ]
      x$participants[, ..age.value := NULL]
      x$participants <- x$participants[,
        ..age.unit := fifelse(
          !is.na(get(participant_age_column)) & is.na(..age.unit),
          "years",
          ..age.unit
        )
      ]
    } else {
      x$participants <- x$participants[,
        ..age.unit := "years"
      ]
    }

    limits <- c("..low", "..high")
    x$participants <- x$participants[,
      paste(limits) := tstrsplit(
        as.character(get(participant_age_column)),
        "-",
        fixed = TRUE
      )
    ]
    x$participants <- x$participants[
      is.na(..high),
      ..high := fifelse(is.na(..high), ..low, ..high)
    ]
    seconds_in_year <- period_to_seconds(years(1))
    for (limit in limits) {
      x$participants <- x$participants[, paste(limit) := as.numeric(get(limit))]
      ## period() is not vectorized, so we need to apply it row by row
      x$participants <-
        x$participants[
          ..age.unit != "years" & !is.na(get(limit)),
          paste(limit) := mapply(
            function(val, unit) {
              period_to_seconds(period(val, unit)) / seconds_in_year
            },
            get(limit),
            ..age.unit
          ),
        ]
    }

    # include included min and max age
    x$participants <- x$participants[,
      paste0(participant_age_column, "_est_min") := ..low
    ]
    x$participants <- x$participants[,
      paste0(participant_age_column, "_est_max") := ..high
    ]
    x$participants <- x$participants[,
      paste(participant_age_column, "exact", sep = "_") := suppressWarnings(
        as.integer(get(participant_age_column))
      )
    ]
    x$participants[, paste(participant_age_column) := NULL]

    x$participants[, ..high := NULL]
    x$participants[, ..low := NULL]
    x$participants[, ..age.unit := NULL]
  }

  x
}

# Helper to create minimal survey for testing
create_test_survey <- function(participants, contacts = NULL) {
  if (is.null(contacts)) {
    contacts <- data.frame(
      part_id = participants$part_id[1],
      cnt_age_exact = 25
    )
  }
  new_contact_survey(participants, contacts)
}

# Test 1: Country name normalisation
test_that("clean() normalises 2-letter ISO country codes", {
  participants <- data.frame(
    part_id = 1:3,
    part_age = c(20, 30, 40),
    country = c("GB", "DE", "FR"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_identical(
    as.character(cleaned$participants$country),
    c("United Kingdom", "Germany", "France")
  )
})

test_that("clean() normalises 3-letter ISO country codes", {
  participants <- data.frame(
    part_id = 1:3,
    part_age = c(20, 30, 40),
    country = c("GBR", "DEU", "FRA"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_identical(
    as.character(cleaned$participants$country),
    c("United Kingdom", "Germany", "France")
  )
})

test_that("clean() normalises full country names", {
  participants <- data.frame(
    part_id = 1:3,
    part_age = c(20, 30, 40),
    country = c("United Kingdom", "Germany", "France"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_identical(
    as.character(cleaned$participants$country),
    c("United Kingdom", "Germany", "France")
  )
})

test_that("clean() preserves completely unrecognised country names", {
  # When countrycode returns NA, the original value is preserved
  participants <- data.frame(
    part_id = 1:2,
    part_age = c(20, 30),
    country = c("Germany", "XYZABC123"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_identical(
    as.character(cleaned$participants$country),
    c("Germany", "XYZABC123")
  )
})

# Test 2: Non-numeric age entries become NA
test_that("clean() sets non-numeric age entries to NA", {
  participants <- data.frame(
    part_id = 1:4,
    part_age = c("25", "unknown", "N/A", "thirty"),
    stringsAsFactors = FALSE
  )

  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_identical(cleaned$participants$part_age_exact[1], 25L)
  expect_true(is.na(cleaned$participants$part_age_exact[2]))
  expect_true(is.na(cleaned$participants$part_age_exact[3]))
  expect_true(is.na(cleaned$participants$part_age_exact[4]))
})

# Test 3: "Under X" format conversion
test_that("clean() converts 'Under X' format to '0-X'", {
  participants <- data.frame(
    part_id = 1:3,
    part_age = c("Under 1", "Under 5", "Under 18"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_identical(cleaned$participants$part_age_est_min, c(0, 0, 0))
  expect_identical(cleaned$participants$part_age_est_max, c(1, 5, 18))
})

# Test 4: Age range parsing
test_that("clean() creates est_min and est_max for age ranges", {
  survey <- polymod
  # Avoid mutating polymod by reference (participants is a data.table)
  survey$participants <- data.table::copy(survey$participants)

  # Replace exact ages with ranges
  survey$participants[, part_age := "20-30"]
  survey$participants[, part_age_exact := NULL]

  cleaned <- clean(survey)

  expect_true("part_age_est_min" %in% names(cleaned$participants))
  expect_true("part_age_est_max" %in% names(cleaned$participants))
  expect_equal(cleaned$participants$part_age_est_min[1], 20, tolerance = 0)
  expect_equal(cleaned$participants$part_age_est_max[1], 30, tolerance = 0)
})

test_that("clean() handles single age values (no range)", {
  participants <- data.frame(
    part_id = 1:2,
    part_age = c("25", "30"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  # For single values, est_min and est_max should be the same
  expect_identical(cleaned$participants$part_age_est_min, c(25, 30))
  expect_identical(cleaned$participants$part_age_est_max, c(25, 30))
})

# Test 5: Exact age column creation
test_that("clean() creates part_age_exact from numeric ages", {
  participants <- data.frame(
    part_id = 1:3,
    part_age = c("25", "30", "45"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_true("part_age_exact" %in% names(cleaned$participants))
  expect_identical(cleaned$participants$part_age_exact, c(25L, 30L, 45L))
})

test_that("clean() sets part_age_exact to NA for age ranges", {
  participants <- data.frame(
    part_id = 1:2,
    part_age = c("25-30", "40-50"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  # Age ranges should result in NA for exact age
  expect_true(is.na(cleaned$participants$part_age_exact[1]))
  expect_true(is.na(cleaned$participants$part_age_exact[2]))
})

# Test 6: Survey with no cleaning needed
test_that("clean() handles survey that doesn't need cleaning", {
  # polymod is already clean
  cleaned <- clean(polymod)

  # Should return without errors and maintain structure
  expect_s3_class(cleaned, "contact_survey")
  expect_true("participants" %in% names(cleaned))
  expect_true("contacts" %in% names(cleaned))
})

# Test 7: Empty survey
test_that("clean() handles empty participants", {
  participants <- data.frame(
    part_id = integer(0),
    part_age = character(0),
    stringsAsFactors = FALSE
  )
  contacts <- data.frame(
    part_id = integer(0),
    cnt_age_exact = integer(0)
  )
  survey <- new_contact_survey(participants, contacts)

  cleaned <- clean(survey)
  expect_identical(nrow(cleaned$participants), 0L)
})

# Test 8: Survey without country column
test_that("clean() handles survey without country column", {
  participants <- data.frame(
    part_id = 1:2,
    part_age = c("25", "30"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)

  cleaned <- clean(survey)
  expect_false("country" %in% names(cleaned$participants))
})

# Test 9: Custom participant_age_column
test_that("clean() works with custom participant_age_column", {
  participants <- data.frame(
    part_id = 1:2,
    age = c("20-30", "40-50"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey, participant_age_column = "age")

  expect_true("age_est_min" %in% names(cleaned$participants))
  expect_true("age_est_max" %in% names(cleaned$participants))
  expect_identical(cleaned$participants$age_est_min, c(20, 40))
  expect_identical(cleaned$participants$age_est_max, c(30, 50))
})

# Test 10: Deprecated argument warning
test_that("clean() deprecated argument produces warning", {
  lifecycle::expect_deprecated(
    clean(polymod, participant.age.column = "part_age")
  )
})

# Test 11: Survey with numeric ages (no conversion needed)
test_that("clean() handles already numeric age column", {
  participants <- data.frame(
    part_id = 1:3,
    part_age = c(25, 30, 45)
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  # Numeric ages without NAs shouldn't trigger age cleaning
  expect_identical(cleaned$participants$part_age, c(25, 30, 45))
})

# Test 12: Age unit conversion (months to years)
test_that("clean() converts months to years", {
  participants <- data.frame(
    part_id = 1:2,
    part_age = c("6 months", "18 months"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_equal(cleaned$participants$part_age_est_min[1], 0.5, tolerance = 0.01)
  expect_equal(cleaned$participants$part_age_est_min[2], 1.5, tolerance = 0.01)
})

test_that("clean() converts weeks to years", {
  participants <- data.frame(
    part_id = 1:2,
    part_age = c("52 weeks", "26 weeks"),
    stringsAsFactors = FALSE
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  expect_equal(cleaned$participants$part_age_est_min[1], 1, tolerance = 0.02)
  expect_equal(cleaned$participants$part_age_est_min[2], 0.5, tolerance = 0.02)
})

# Test 13: Mixed NA and valid ages
test_that("clean() handles mix of valid and NA ages", {
  participants <- data.frame(
    part_id = 1:4,
    part_age = c(25, NA, 30, NA)
  )
  survey <- create_test_survey(participants)
  cleaned <- clean(survey)

  # NA values should trigger age cleaning but numeric values are preserved
  expect_true("part_age_est_min" %in% names(cleaned$participants))
  expect_identical(cleaned$participants$part_age_est_min[1], 25)
  expect_true(is.na(cleaned$participants$part_age_est_min[2]))
  expect_identical(cleaned$participants$part_age_est_min[3], 30)
  expect_true(is.na(cleaned$participants$part_age_est_min[4]))
})

#' Extract the empirical age distribution of contacts from a survey
#'
#' Returns a data.frame of (age, proportion) pairs representing how
#' contact ages are distributed in the survey. This can be passed to
#' [assign_age_groups()] as `estimated_contact_age` to impute ages
#' from ranges using this distribution rather than uniform sampling.
#'
#' @param survey a [survey()] object
#' @returns a data.frame with columns `age` (integer) and `proportion` (numeric,
#'   summing to 1)
#' @examples
#' data(polymod)
#' dist <- contact_age_distribution(polymod)
#' head(dist)
#' plot(dist$age, dist$proportion, type = "h",
#'      xlab = "Age", ylab = "Proportion")
#'
#' @export
#' @autoglobal
contact_age_distribution <- function(survey) {
  check_if_contact_survey(survey)
  contacts <- data.table::copy(survey$contacts)

  # Use exact ages only to avoid including previously imputed values
  age_col <- if ("cnt_age_exact" %in% colnames(contacts)) {
    "cnt_age_exact"
  } else if ("cnt_age" %in% colnames(contacts)) {
    "cnt_age"
  } else {
    cli::cli_abort("No contact age column found in survey.")
  }

  # Convert factor levels to their numeric values (not factor codes)
  contacts <- convert_factor_to_integer(contacts, age_col)
  ages <- suppressWarnings(as.numeric(contacts[[age_col]]))
  ages <- ages[!is.na(ages)]

  if (length(ages) == 0) {
    cli::cli_abort("No non-missing contact ages found in survey.")
  }
  if (!all(is.finite(ages))) {
    cli::cli_abort("Contact ages must be finite.")
  }
  if (any(ages < 0)) {
    cli::cli_abort("Contact ages must be non-negative.")
  }
  if (any(ages %% 1 != 0)) {
    cli::cli_abort("Contact ages must be whole numbers.")
  }
  ages <- as.integer(ages)

  counts <- data.table::data.table(age = ages)[, .N, by = age]
  counts[, proportion := N / sum(N)]
  counts <- counts[order(age)]
  data.frame(age = counts$age, proportion = counts$proportion)
}

#' Validate an age distribution data.frame
#' @param x object to validate
#' @returns `x` invisibly, or errors
#' @keywords internal
validate_age_distribution <- function(x) {
  if (!is.data.frame(x)) {
    cli::cli_abort(
      "{.arg estimated_contact_age} must be a string or a data.frame."
    )
  }
  required <- c("age", "proportion")
  missing_cols <- setdiff(required, colnames(x))
  if (length(missing_cols) > 0) {
    cli::cli_abort(
      "Age distribution must have column{?s} {.val {missing_cols}}."
    )
  }
  if (!is.numeric(x$age) || !is.numeric(x$proportion)) {
    cli::cli_abort(
      "Columns {.val age} and {.val proportion} must be numeric."
    )
  }
  if (!all(is.finite(x$age)) || !all(is.finite(x$proportion))) {
    cli::cli_abort(
      "Columns {.val age} and {.val proportion} must not contain missing or \\
       non-finite values."
    )
  }
  if (any(x$age < 0) || any(x$age %% 1 != 0)) {
    cli::cli_abort("Column {.val age} must contain non-negative integer ages.")
  }
  if (any(x$proportion < 0)) {
    cli::cli_abort("Column {.val proportion} must not contain negative values.")
  }
  # Normalise proportions to sum to 1 if they don't already
  total <- sum(x$proportion)
  if (total <= 0) {
    cli::cli_abort("Column {.val proportion} must have a positive sum.")
  }
  if (abs(total - 1) > 1e-8) {
    cli::cli_warn("Proportions do not sum to 1; normalising.")
    x$proportion <- x$proportion / total
  }
  invisible(x)
}

polymod_age_grouped <- assign_age_groups(polymod)

test_that("assign_age_groups() adds correct columns", {
  expect_true(all(
    c("lower.age.limit", "part_age", "age.group", "upper.age.limit") %in%
      names(polymod_age_grouped$participants)
  ))
  expect_true(
    "cnt_age" %in% names(polymod_age_grouped$contacts)
  )
})

test_that("assign_age_groups() appropriately changes dimensions", {
  expect_snapshot(
    dim(polymod_age_grouped$participants)
  )

  expect_snapshot(
    dim(polymod_age_grouped$contacts)
  )
})

# test that the levels are [0, 5), [5, 10], "10+"
polymod_age_grouped_0_5_10 <- polymod |>
  assign_age_groups(age_limits = c(0, 5, 10))

# test the same levels
polymod_age_grouped_5_10_15 <- polymod |>
  assign_age_groups(age_limits = c(5, 10, 15))

test_that("assign_age_groups() appropriately changes age.group factor", {
  expect_snapshot(
    levels(polymod_age_grouped$participants$age.group)
  )
  expect_snapshot(
    levels(polymod_age_grouped_0_5_10$participants$age.group)
  )
  expect_snapshot(
    levels(polymod_age_grouped_5_10_15$participants$age.group)
  )
  expect_snapshot(
    range(polymod_age_grouped_0_5_10$contacts$cnt_age, na.rm = TRUE)
  )
  expect_snapshot(
    range(polymod_age_grouped_5_10_15$contacts$cnt_age, na.rm = TRUE)
  )
})

## contact_age_distribution() -------------------------------------------------

test_that("contact_age_distribution() returns a valid distribution", {
  dist <- contact_age_distribution(polymod)
  expect_s3_class(dist, "data.frame")
  expect_true(all(c("age", "proportion") %in% names(dist)))
  expect_type(dist$age, "integer")
  expect_type(dist$proportion, "double")
  expect_equal(sum(dist$proportion), 1, tolerance = 1e-10)
  expect_true(all(dist$proportion >= 0))
})

test_that("contact_age_distribution() rejects fractional ages", {
  bad <- polymod
  bad$contacts <- data.table::copy(bad$contacts)
  bad$contacts[, cnt_age_exact := as.numeric(cnt_age_exact)]
  bad$contacts[1, cnt_age_exact := 3.5]
  expect_error(contact_age_distribution(bad), "whole numbers")
})

test_that("contact_age_distribution() rejects non-finite ages", {
  bad <- polymod
  bad$contacts <- data.table::copy(bad$contacts)
  bad$contacts[, cnt_age_exact := as.numeric(cnt_age_exact)]
  bad$contacts[1, cnt_age_exact := Inf]
  expect_error(contact_age_distribution(bad), "finite")
})

## Distribution-based imputation -----------------------------------------------

test_that("assign_age_groups() accepts a distribution for contact age", {
  dist <- contact_age_distribution(polymod)
  result <- assign_age_groups(
    polymod,
    estimated_contact_age = dist,
    age_limits = c(0, 5, 15)
  )
  expect_true("age.group" %in% names(result$participants))
  expect_true("contact.age.group" %in% names(result$contacts))
})

test_that("distribution-based imputation reduces missing contact ages", {
  dist <- contact_age_distribution(polymod)
  no_impute <- assign_age_groups(
    polymod,
    estimated_contact_age = "missing"
  )
  with_dist <- assign_age_groups(
    polymod,
    estimated_contact_age = dist
  )
  expect_lte(
    sum(is.na(with_dist$contacts$cnt_age)),
    sum(is.na(no_impute$contacts$cnt_age))
  )
})

test_that("validate_age_distribution() errors on bad input", {
  expect_error(
    validate_age_distribution(data.frame(x = 1)),
    "age"
  )
  expect_error(
    validate_age_distribution(
      data.frame(age = "a", proportion = 0.5, stringsAsFactors = FALSE)
    ),
    "numeric"
  )
  expect_error(
    validate_age_distribution(data.frame(age = 1, proportion = -0.5)),
    "negative"
  )
  expect_error(
    validate_age_distribution(data.frame(age = 1, proportion = 0)),
    "positive sum"
  )
})

test_that("validate_age_distribution() normalises proportions with warning", {
  x <- data.frame(age = c(1, 2), proportion = c(2, 3))
  expect_warning(validate_age_distribution(x), "normalising")
  dist <- suppressWarnings(validate_age_distribution(x))
  expect_equal(sum(dist$proportion), 1, tolerance = 1e-12)
  expect_equal(dist$proportion, c(0.4, 0.6), tolerance = 1e-12)
})

## String-based imputation -----------------------------------------------------

test_that("assign_age_groups() imputes ages from ranges", {
  polymod_no_impute <- assign_age_groups(
    polymod,
    estimated_participant_age = "missing",
    estimated_contact_age = "missing"
  )

  polymod_impute_mean <- assign_age_groups(
    polymod,
    estimated_participant_age = "mean",
    estimated_contact_age = "mean"
  )

  # When imputing, fewer values should be missing
  expect_lte(
    sum(is.na(polymod_impute_mean$contacts$cnt_age)),
    sum(is.na(polymod_no_impute$contacts$cnt_age))
  )

  expect_lte(
    sum(is.na(polymod_impute_mean$participants$part_age)),
    sum(is.na(polymod_no_impute$participants$part_age))
  )
})

test_that("'sample' age imputation can hit the upper bound", {
  # Regression test: previously runif(n, min, max) followed by as.integer()
  # truncated to [min, max - 1], so the maximum could never be drawn.
  fake_survey <- as_contact_survey(list(
    participants = data.table::data.table(
      part_id = seq_len(2000),
      part_age_exact = NA_integer_,
      part_age_est_min = 17L,
      part_age_est_max = 18L
    ),
    contacts = data.table::data.table(
      part_id = integer(),
      cnt_age_exact = integer()
    )
  ))

  set.seed(1)
  imputed <- assign_age_groups(
    fake_survey,
    estimated_participant_age = "sample",
    missing_participant_age = "keep",
    missing_contact_age = "keep"
  )

  expect_true(18L %in% imputed$participants$part_age)
  expect_true(17L %in% imputed$participants$part_age)
  expect_true(all(imputed$participants$part_age %in% c(17L, 18L)))
})

#' @title Contact survey
#' @description Deprecated. A `survey` object contains the results of a contact survey. In particular, it contains two data frames called `participants` and `contacts` that are linked by a column specified as `id.column`
#' @param participants a `data.frame` containing information on participants
#' @param contacts a `data.frame` containing information on contacts
#' @param reference a `list` containing information needed to reference the survey, in particular it can contain$a "title", "bibtype", "author", "doi", "publisher", "note", "year"
#' @return a new survey object
#' @author Sebastian Funk
#' @keywords internal
new_contact_survey <- function(participants, contacts, reference = NULL) {
  new_obj <-
    structure(
      list(
        participants = data.table(participants),
        contacts = data.table(contacts),
        reference = reference
      ),
      class = "contact_survey"
    )

  return(new_obj)
}

library(data.table)

erroneous_survey <-
  new_contact_survey(polymod$participants, polymod$contacts, polymod$reference)

erroneous_type1 <- copy(erroneous_survey)
erroneous_type1$participants <- "test"
erroneous_type2 <- copy(erroneous_survey)
erroneous_type2$participants <- 17

test_that("error is thrown if survey contains false data types", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    as_contact_survey(erroneous_type1)
  )
  expect_error(
    as_contact_survey(erroneous_type1),
    "Must be of type 'data.frame'"
  )
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    as_contact_survey(erroneous_type2)
  )
  expect_error(
    as_contact_survey(erroneous_type2),
    "Must be of type 'data.frame'"
  )
})

erroneous_structure1 <- copy(erroneous_survey)
erroneous_structure1$participants$part_id <- NULL

test_that("incorrect structure of data frames is correctly identified", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    as_contact_survey(erroneous_structure1)
  )
  expect_error(
    as_contact_survey(erroneous_structure1),
    "Names must include the elements \\{'part_id'\\}" ## nolint: nonportable_path_linter
  )
})

no_country_survey <- copy(erroneous_survey)
no_country_survey$participants$country <- NULL

no_year_survey <- copy(erroneous_survey)
no_year_survey$participants$year <- NULL

no_country_year_survey <- copy(erroneous_survey)
no_country_year_survey$participants$country <- NULL
no_country_year_survey$participants$year <- NULL

## nolint start: nonportable_path_linter
test_that("surveys without country/year columns are accepted", {
  expect_no_error(as_contact_survey(no_country_survey))
  expect_no_error(as_contact_survey(no_year_survey))
  expect_no_error(as_contact_survey(no_country_year_survey))
})
## nolint end: nonportable_path_linter

test_that("explicitly specified missing columns still error", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    as_contact_survey(no_country_survey, country.column = "country")
  )
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    as_contact_survey(no_year_survey, year.column = "year")
  )
})

#' Create a contact_matrix object
#'
#' @param matrix a numeric matrix with age group dimnames
#' @param participants a data.frame with columns `age.group`, `participants`,
#'   `proportion`
#' @param ... additional named elements (e.g. `mean.contacts`, `normalisation`,
#'   `contacts` from [split_matrix()])
#' @returns a `contact_matrix` object (an S3 class inheriting from `list`)
#' @keywords internal
new_contact_matrix <- function(matrix, participants, ...) {
  checkmate::assert(
    checkmate::check_matrix(matrix),
    checkmate::check_array(matrix)
  )
  checkmate::assert_data_frame(participants)
  result <- list(matrix = matrix, participants = participants, ...)
  structure(result, class = c("contact_matrix", "list"))
}

#' Test whether an object is a contact_matrix
#'
#' @param x object to test
#' @returns logical
#' @keywords internal
is_contact_matrix <- function(x) {
  inherits(x, "contact_matrix")
}

#' @export
print.contact_matrix <- function(x, ...) {
  groups <- rownames(x$matrix)

  cli::cli_h2("Contact matrix ({nrow(x$matrix)} age group{?s})")

  if (!is.null(groups)) {
    cli::cli_text("Ages: {.val {groups}}")
  }

  cli::cli_text("Participants: {sum(x$participants$participants)}")

  if (!is.null(x[["mean.contacts"]])) {
    cli::cli_text(
      "Mean contacts: {round(x[[\"mean.contacts\"]], 2)}"
    )
  }

  cli::cli_text("")
  print(x$matrix, ...)
  invisible(x)
}

#' @export
plot.contact_matrix <- function(x, ...) {
  matrix_plot(x$matrix, ...)
}

#' @export
as.matrix.contact_matrix <- function(x, ...) {
  x$matrix
}

#' Get age-specific population data according to the World
#' Population Prospects 2017 edition
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' This function is deprecated in favour of passing population data directly
#' to [contact_matrix()] via the `survey_pop` argument. Additionally, the
#' underlying `wpp2017` data is outdated. For more recent population data,
#' use the `wpp2024` package from GitHub.
#'
#' @details
#' This uses data from the `wpp2017` package but combines male and female,
#' and converts age groups to lower age limits. If the requested
#' year is not present in the historical data, WPP projections
#' are used.
#'
#' @return data frame of age-specific population data
#' @importFrom data.table dcast melt fread
#' @importFrom countrycode countrycode
#' @import wpp2017
#' @param countries countries, will return all if not given
#' @param years years, will return all if not given
#' @autoglobal
#' @examples
#' wpp_age("Italy", c(1990, 2000))
#'
#' # For more recent data, use wpp2024 from GitHub:
#' # remotes::install_github("PPgp/wpp2024")
#' # library(wpp2024)
#' # data(popAge1dt)
#' # uk_pop <- popAge1dt[name == "United Kingdom" & year == 2020,
#' #                     .(lower.age.limit = age, population = pop * 1000)]
#' # contact_matrix(polymod, countries = "United Kingdom", survey_pop = uk_pop)
#' @export
wpp_age <- function(countries, years) {
  lifecycle::deprecate_soft(
    "0.6.0",
    "wpp_age()",
    details = c(
      "Pass population data directly via the \\
      {.arg survey_pop} argument instead.",
      i = "The underlying {.pkg wpp2017} data is also outdated; \\
           use {.pkg wpp2024} from GitHub for more recent data."
    )
  )

  popM <- fread(system.file("data", "popM.txt", package = "wpp2017"))
  popF <- fread(system.file("data", "popF.txt", package = "wpp2017"))

  # wpp2017 is limited to 2015, so add wpp projections for later years
  years_included <- max(as.numeric(names(popM)[-(1:3)]))
  if (!missing(years) && any(years > years_included)) {
    popMprojMed <- fread(system.file(
      "data",
      "popMprojMed.txt",
      package = "wpp2017"
    ))
    popFprojMed <- fread(system.file(
      "data",
      "popFprojMed.txt",
      package = "wpp2017"
    ))

    popM <- merge(popM, popMprojMed)
    popF <- merge(popF, popFprojMed)

    cli::cli_warn(
      message = c(
        "Don't have historical population data available after: \\
        {.val {years_included}}.",
        # nolint start
        "i" = "Will make use of the median projection of population counts \\
        from the {.pkg WPP2017} package."
        # nolint end
      )
    )
  }

  popM <- popM[, sex := "male"]
  popF <- popF[, sex := "female"]

  pop <- rbind(popM, popF)

  # change column names to remain compatible with the 2015 package
  pop[, country := name]
  pop[, name := NULL]

  if (!missing(countries)) {
    ## match by UN country code
    pop <- suppressWarnings(pop[
      country_code %in% countrycode(countries, "country.name", "un")
    ])
  }

  if (nrow(pop) > 0) {
    pop <- melt(
      pop,
      id.vars = c("country", "country_code", "age", "sex"),
      variable.name = "year"
    )
    pop <- dcast(
      pop,
      country + country_code + age + year ~ sex,
      value.var = "value"
    )

    pop[, year := as.integer(as.character(year))]

    if (!missing(years)) {
      if (any(pop$year %in% years)) {
        pop <- pop[year %in% years]
      } else {
        available.years <- unique(pop$year)
        nearest.year <- available.years[which.min(abs(available.years - years))]
        cli::cli_warn(
          message = c(
            "Don't have population data available for year: {.val {years}}",
            "i" = "Will return nearest year: {.val {nearest.year}}" # nolint
          )
        )
        pop <- pop[year %in% nearest.year]
      }
    }

    pop <- pop[, lower.age.limit := as.integer(sub("[-+].*$", "", age))]
    pop <- pop[, list(
      country,
      lower.age.limit,
      year,
      population = (female + male) * 1000
    )] # reorder columns
    pop <- pop[order(lower.age.limit), ] # sort by lower.age.limit
  }

  as.data.frame(pop)
}

set.seed(123)

withr::local_options(
  lifecycle_verbosity = "quiet",
  .local_envir = teardown_env()
)
polymod2 <- get_survey(polymod)
polymod3 <- get_survey(polymod)
polymod4 <- get_survey(polymod)
polymod5 <- get_survey(polymod)
polymod6 <- get_survey(polymod)
polymod7 <- get_survey(polymod)
polymod8 <- get_survey(polymod)
polymod9 <- get_survey(polymod)
polymod10 <- get_survey(polymod)
polymod11 <- get_survey(polymod)

polymod2$participants$added_weight <- 0.5
polymod2$contacts$cnt_age_exact <- factor(polymod2$contacts$cnt_age_exact)
polymod2$participants$part_age_exact[1] <- NA_real_
polymod3$participants$dayofweek <- NULL
polymod3$participants$year <- NULL
polymod4$participants$country <- NULL
polymod5$participants$country <- factor("Zamonia")
polymod6$contacts$cnt_age_est_min <- NULL
polymod6$contacts$cnt_age_est_max <- NULL
polymod6$contacts$cnt_age_exact <- NULL
polymod7$participants$country <- NULL
polymod8$contacts$cnt_age_exact <- NA_real_
polymod8$contacts$cnt_age_est_min <- NA_real_
polymod8$contacts$cnt_age_est_max <- NA_real_
polymod8$contacts[polymod$contacts$part_id == 10, "cnt_age_exact"] <- 10
polymod8$contacts[polymod$contacts$part_id == 20, "cnt_age_exact"] <- 20
polymod9$participants$part_age_est_min <- 1
polymod9$participants$part_age_est_max <- 14
polymod9$participants$part_age_exact <- NULL
polymod9$participants$part_age_est_min <- 1
polymod9$participants$part_age_est_max <- 14
nn <- nrow(polymod9$participants)
polymod9$participants$part_age_exact <- ifelse(runif(nn) > 0.7, 20, NA)
polymod10$participants$added_weight <-
  ifelse(polymod10$participants$hh_size > 1, 2, 1)
polymod10$participants$added_weight2 <- 0.3

# to test weights (age and day.of.week)
part_selection <- (polymod11$participants$part_age %in%
  (1:2) &
  polymod11$participants$dayofweek %in% 1:6) |
  (polymod11$participants$part_age %in%
    (3) &
    polymod11$participants$dayofweek %in% 2:3)
polymod11$participants <- polymod11$participants[part_selection, ]

empty_pop <- data.frame(lower.age.limit = c(0, 5), population = NA_real_)

options <-
  list(
    test1 = list(
      survey = polymod,
      countries = "United Kingdom",
      counts = TRUE,
      weigh_dayofweek = TRUE,
      age_limits = seq(0, 80, by = 5),
      missing_contact_age = "remove"
    ),
    test2 = list(
      survey = polymod2,
      age_limits = c(0, 5),
      weights = "added_weight",
      symmetric = TRUE,
      sample_participants = TRUE
    ),
    test3 = list(
      survey = polymod,
      survey_pop = "Australia",
      countries = "GB",
      split = TRUE,
      filter = c(cnt_home = 1),
      age_limits = c(0, 5, 10),
      estimated_contact_age = "sample",
      symmetric = TRUE,
      missing_contact_age = "remove"
    ),
    test4 = list(
      survey = polymod8,
      missing_contact_age = "sample",
      symmetric = TRUE,
      age_limits = c(0, 5, 15),
      symmetric_norm_threshold = 4
    )
  )

contacts <- suppressMessages(lapply(options, function(x) {
  do.call(contact_matrix, x)
}))

test_that("contact matrix exists and is square", {
  expect_true(all(sapply(contacts, function(x) {
    length(unique(dim(x[["matrix"]]))) == 1
  })))
  expect_true(all(sapply(contacts, function(x) {
    prod(dim(x[["matrix"]])) > 0
  })))
})

test_that("contact matrix is numeric", {
  expect_true(all(sapply(contacts, function(x) {
    is.numeric(x[["matrix"]])
  })))
  expect_false(anyNA(sapply(contacts, function(x) {
    is.numeric(x[["matrix"]])
  })))
  expect_type(contacts[[3]]$contacts, "double")
  expect_type(contacts[[3]]$normalisation, "double")
})

test_that("demography has been returned", {
  expect_gt(nrow(contacts[[3]]$demography), 0)
})

test_that("demography is numeric", {
  expect_type(contacts[[3]]$demography$population, "double")
})

test_that("contact matrices look as expected", {
  expect_snapshot(contacts)
})

test_that("survey argument is validated", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(survey = "bogus")
  )
  expect_error(contact_matrix(survey = "bogus"), "survey")
})

test_that("error is thrown if age limits are non-numeric", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(
      survey = polymod,
      age_limits = c(0, 5, "fifteen")
    )
  )
  expect_error(
    suppressWarnings(contact_matrix(
      survey = polymod,
      age_limits = c(0, 5, "fifteen")
    )),
    "age.limits"
  )
})

test_that("error is thrown if country is not found", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(survey = polymod, countries = c("Italy", "Zamonia"))
  )
  expect_error(
    contact_matrix(survey = polymod, countries = c("Italy", "Zamonia")),
    "data not found"
  )
})

test_that("warning is thrown if filter column is not found", {
  expect_snapshot_warning(
    cran = FALSE,
    contact_matrix(survey = polymod, filter = c(test = 0))
  )
  expect_warning(
    contact_matrix(survey = polymod, filter = c(test = 0)),
    "column.* not found"
  )
})

test_that("warning is thrown if missing data exist", {
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod,
      missing_contact_age = "keep",
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod,
      missing_contact_age = "keep",
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    ),
    "missing.contact.age"
  )
  warning <- capture_warnings(contact_matrix(
    survey = polymod,
    missing_participant_age = "keep",
    split = TRUE
  ))
  expect_true(any(grepl(
    "missing.participant.age|missing_participant_age",
    warning
  )))
})

test_that("error is thrown if an unknown argument is passed", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(dummy = "test")
  )
  expect_error(contact_matrix(dummy = "test"), "Unknown argument")
})

test_that("error is thrown if invalid age limits are passed", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(survey = polymod, age_limits = c(13, 11))
  )
  expect_error(
    contact_matrix(survey = polymod, age_limits = c(13, 11)),
    "increasing"
  )
})

test_that("error if no participants after selecting the country", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(survey = polymod, countries = "Romania")
  )
  expect_error(
    contact_matrix(survey = polymod, countries = "Romania"),
    "No participants left"
  )
})

test_that("warning if population needed but no 'year' column", {
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod3,
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod3,
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    ),
    "No information on \"year\" found"
  )
})

test_that("warning if day of week weigh requested but not present", {
  expect_snapshot_warning(
    contact_matrix(survey = polymod3, weigh_dayofweek = TRUE)
  )
})

test_that("warning is thrown if country has no survey population", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    contact_matrix(survey = polymod5, symmetric = TRUE)
  )
  expect_error(
    contact_matrix(survey = polymod5, symmetric = TRUE),
    "not find population data"
  )
})

test_that("warning is thrown if contact survey has no age information", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_snapshot_warning(
    cran = FALSE,
    check(x = polymod6)
  )
  expect_warning(check(x = polymod6), "do not exist")
})

test_that("warning is thrown if participant data has no country", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_snapshot_warning(
    cran = FALSE,
    check(x = polymod4)
  )
  expect_warning(check(x = polymod4), "does not exist")
})

test_that("check result is reported back", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_snapshot(check(x = polymod2))
  expect_message(check(x = polymod2), "Check")
})

test_that("good suggestions are made", {
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod8,
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod8,
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    ),
    "adjusting the age limits"
  )
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod,
      symmetric = TRUE,
      age_limits = c(0, 5, 15),
      missing_participant_age = "keep"
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod,
      symmetric = TRUE,
      age_limits = c(0, 5, 15),
      missing_participant_age = "keep"
    ),
    "setting 'missing.participant.age"
  )
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod,
      symmetric = TRUE,
      age_limits = c(0, 5, 15),
      missing_participant_age = "keep",
      missing_contact_age = "keep"
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod,
      symmetric = TRUE,
      age_limits = c(0, 5, 15),
      missing_participant_age = "keep",
      missing_contact_age = "keep"
    ),
    "and 'missing.contact.age"
  )
})

test_that("nonsensical operations are warned about", {
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod,
      counts = TRUE,
      split = TRUE,
      age_limits = c(0, 5)
    )
  )
  expect_snapshot(
    cran = FALSE,
    contact_matrix(
      survey = polymod,
      counts = TRUE,
      symmetric = TRUE,
      age_limits = c(0, 5)
    )
  )
  expect_snapshot_warning(
    cran = FALSE,
    contact_matrix(
      survey = polymod,
      split = TRUE,
      age_limits = c(0, 5, 15),
      missing_participant_age = "keep"
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod,
      split = TRUE,
      age_limits = c(0, 5, 15),
      missing_participant_age = "keep"
    ),
    "does not work with missing data"
  )
})

test_that("warning if survey is assumed to be representative", {
  expect_snapshot_warning(
    contact_matrix(
      survey = polymod4,
      symmetric = TRUE,
      age_limits = c(0, 5, 15)
    )
  )
})


test_that("mean of estimated contact age gives NA outside limits", {
  cm <- contact_matrix(survey = polymod9, age_limits = c(0, 5, 10, 15, 20))
  expect_true(is.na(rowSums(cm$matrix)[1]))
  expect_false(is.na(rowSums(cm$matrix)[2]))
  expect_true(is.na(rowSums(cm$matrix)[3]))
  expect_true(is.na(rowSums(cm$matrix)[4]))
  expect_false(is.na(rowSums(cm$matrix)[5]))
})

test_that("sample of estimated participant age gives NA if no overlap", {
  cm <- contact_matrix(
    survey = polymod9,
    age_limits = c(0, 5, 10, 15, 20),
    estimated_participant_age = "sample"
  )
  expect_false(is.na(rowSums(cm$matrix)[1]))
  expect_false(is.na(rowSums(cm$matrix)[2]))
  expect_false(is.na(rowSums(cm$matrix)[3]))
  expect_true(is.na(rowSums(cm$matrix)[4]))
  expect_false(is.na(rowSums(cm$matrix)[5]))
})

test_that("If weights = added_weight, the results are not identical", {
  cm_orig <- suppressMessages(
    contact_matrix(
      survey = polymod10,
      countries = "United Kingdom",
      age_limits = c(0, 18, 60),
      return_part_weights = TRUE
    )
  )
  cm_weight <- suppressMessages(contact_matrix(
    survey = polymod10,
    countries = "United Kingdom",
    age_limits = c(0, 18, 60),
    weights = "added_weight",
    return_part_weights = TRUE
  ))

  expect_identical(cm_orig$participants, cm_weight$participants)
  expect_false(
    nrow(cm_orig$participants.weights) == nrow(cm_weight$participants.weights)
  )
  expect_false(all(cm_orig$matrix == cm_weight$matrix))
})


test_that("The order in which weights are applied do not change the results", {
  expect_identical(
    suppressMessages(
      contact_matrix(
        survey = polymod10,
        countries = "United Kingdom",
        weights = c("added_weight2", "added_weight")
      )
    ),
    suppressMessages(
      contact_matrix(
        survey = polymod10,
        countries = "United Kingdom",
        weights = c("added_weight", "added_weight2")
      )
    )
  )
})

test_that("day.of.week weight does not affect weekday-only groups", {
  matrix_unweighted <- suppressMessages(suppressWarnings(
    contact_matrix(
      polymod11,
      age_limits = 1:3,
      weigh_dayofweek = FALSE,
      symmetric = FALSE
    )
  ))
  matrix_weighted <- suppressMessages(suppressWarnings(
    contact_matrix(
      polymod11,
      age_limits = 1:3,
      weigh_dayofweek = TRUE,
      symmetric = FALSE
    )
  ))

  num_contacts_unweighted <- rowSums(matrix_unweighted$matrix)
  num_contacts_weighted <- rowSums(matrix_weighted$matrix)

  # ages 1 and 2 => impaced by weights
  expect_true(
    rowSums(matrix_unweighted$matrix)[1] != rowSums(matrix_weighted$matrix)[1]
  )
  expect_true(
    rowSums(matrix_unweighted$matrix)[2] != rowSums(matrix_weighted$matrix)[2]
  )

  # age 3 => contains only data on weekdays => should not be impacted by weights
  expect_equal(
    num_contacts_unweighted[3],
    num_contacts_weighted[3],
    tolerance = 1e-8
  )
})

test_that("day.of.week weight does not affect weekday-only group", {
  matrix_unweighted <- contact_matrix(
    polymod11,
    age_limits = c(0, 3),
    weigh_dayofweek = FALSE,
    symmetric = FALSE
  )
  matrix_weighted <- contact_matrix(
    polymod11,
    age_limits = c(0, 3),
    weigh_dayofweek = TRUE,
    symmetric = FALSE
  )

  num_contacts_unweighted <- rowSums(matrix_unweighted$matrix)
  num_contacts_weighted <- rowSums(matrix_weighted$matrix)

  # age group 1 => adjusted by weights
  expect_true(
    rowSums(matrix_unweighted$matrix)[1] != rowSums(matrix_weighted$matrix)[1]
  )

  # age group 2 => only weekday data => not impacted by weights
  expect_equal(
    num_contacts_unweighted[2],
    num_contacts_weighted[2],
    tolerance = 1e-8
  )
})

test_that("day.of.week weight changes result with one age group", {
  matrix_unweighted <- contact_matrix(
    polymod11,
    age_limits = 0,
    weigh_dayofweek = FALSE,
    symmetric = FALSE
  )
  matrix_weighted <- contact_matrix(
    polymod11,
    age_limits = 0,
    weigh_dayofweek = TRUE,
    symmetric = FALSE
  )

  expect_false(matrix_unweighted$matrix == matrix_weighted$matrix)
})

test_that("age-specific weight changes multi-year age groups", {
  suppressWarnings({
    matrix_unweighted <- contact_matrix(
      polymod11,
      age_limits = c(0, 3),
      weigh_age = FALSE,
      symmetric = FALSE
    )
    matrix_weighted <- contact_matrix(
      polymod11,
      age_limits = c(0, 3),
      weigh_age = TRUE,
      symmetric = FALSE
    )
  })

  expect_gt(
    sum(matrix_unweighted$matrix[1, ]),
    sum(matrix_weighted$matrix[1, ])
  ) # manual calculation
  expect_identical(matrix_unweighted$matrix[2, ], matrix_weighted$matrix[2, ])
})

test_that("age-specific weight unchanged for single year groups", {
  expect_equal(
    suppressWarnings(
      contact_matrix(
        survey = polymod,
        countries = "Poland",
        age_limits = 1:110,
        weigh_age = FALSE
      )$matrix
    ),
    suppressWarnings(
      contact_matrix(
        survey = polymod,
        countries = "Poland",
        age_limits = 1:110,
        weigh_age = TRUE
      )$matrix
    ),
    tolerance = 1e-15
  )
})

test_that("all age groups included even if no participants", {
  # Age groups beyond max participant age should still be included
  cm <- suppressWarnings(contact_matrix(
    polymod,
    age_limits = c(0, 18, 50, 100),
    symmetric = FALSE
  ))

  # All 4 age groups should be present in participants
  expect_identical(nrow(cm$participants), 4L)
  expect_true("[100,Inf)" %in% cm$participants$age.group)
  # The [100,Inf) group should have 0 participants
  expect_identical(cm$participants[age.group == "[100,Inf)", participants], 0L)
})

test_that("Demography age groups are subset of participant age groups", {
  cm <- suppressWarnings(contact_matrix(
    polymod,
    age_limits = c(0, 18, 50, 100),
    symmetric = TRUE
  ))

  # All demography age groups should be present in participant age groups
  # (demography might have fewer due to population data limits)
  expect_true(all(cm$demography$age.group %in% cm$participants$age.group))
})

test_that("The return.demography overrules other parameters", {
  suppressWarnings({
    # no demography data
    expect_null(
      contact_matrix(survey = polymod, age_limits = c(0, 18))$demography
    )
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        symmetric = FALSE
      )$demography
    )
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        weigh_age = FALSE
      )$demography
    )
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        split = FALSE
      )$demography
    )

    # default behaviour
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        symmetric = TRUE
      )$demography,
      "list"
    )
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        weigh_age = TRUE
      )$demography,
      "list"
    )
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        split = TRUE
      )$demography,
      "list"
    )

    # always return demography, irrespectively of other function paramters
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        symmetric = FALSE,
        return_demography = TRUE
      )$demography,
      "list"
    )
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        weigh_age = FALSE,
        return_demography = TRUE
      )$demography,
      "list"
    )
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        split = FALSE,
        return_demography = TRUE
      )$demography,
      "list"
    )

    # never return demography data, irrespectively of other function paramters
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        symmetric = TRUE,
        return_demography = FALSE
      )$demography
    )
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        weigh_age = TRUE,
        return_demography = FALSE
      )$demography
    )
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        split = TRUE,
        return_demography = FALSE
      )$demography
    )
  })
})

test_that("The return.part.weights option", {
  suppressWarnings({
    # no participant weights data
    expect_null(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18)
      )$participants.weights
    )

    # with participant weights data
    expect_type(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        return_part_weights = TRUE
      )$participants.weights,
      "list"
    )

    # without any weight method activated, weights should be 1
    expect_identical(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        return_part_weights = TRUE
      )$participants.weights$weight,
      c(1, 1)
    )
    expect_true(all(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        return_part_weights = TRUE
      )$participants.weights$weight ==
        1
    ))

    # with dayofweek weights activated, we should receive 6 weights
    # (2 age groups × 3 day types: weekday, weekend, NA dayofweek)
    expect_length(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        weigh_dayofweek = TRUE,
        return_part_weights = TRUE
      )$participants.weights$weight,
      6
    )
    expect_false(all(
      contact_matrix(
        survey = polymod,
        age_limits = c(0, 18),
        weigh_dayofweek = TRUE,
        return_part_weights = TRUE
      )$participants.weights$weight ==
        1
    ))
  })
})

test_that("The participant weights add up to the sample size", {
  suppressWarnings({
    weights.uniform <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE
    )$participants.weights
    weights.age <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE,
      weigh_dayofweek = TRUE
    )$participants.weights
    weights.dayofweek <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE,
      weigh_age = TRUE
    )$participants.weights
    weights.both <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE,
      weigh_age = TRUE,
      weigh_dayofweek = TRUE
    )$participants.weights

    expect_equal(
      sum(weights.uniform[, weight * proportion]),
      1,
      tolerance = 1e-8
    )
    expect_equal(sum(weights.age[, weight * proportion]), 1, tolerance = 1e-8)
    expect_equal(
      sum(weights.dayofweek[, weight * proportion]),
      1,
      tolerance = 1e-8
    )
    expect_equal(sum(weights.both[, weight * proportion]), 1, tolerance = 1e-8)
  })
})

test_that("The weights with threshold", {
  suppressWarnings({
    # get weights without and with a threshold of 3 and 50
    weights.nothreshold <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE,
      weigh_age = TRUE,
      weigh_dayofweek = TRUE,
      weight_threshold = NA
    )$participants.weights
    weights.threshold3 <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE,
      weigh_age = TRUE,
      weigh_dayofweek = TRUE,
      weight_threshold = 3
    )$participants.weights
    weights.threshold50 <- contact_matrix(
      survey = polymod,
      age_limits = c(0, 18),
      return_part_weights = TRUE,
      weigh_age = TRUE,
      weigh_dayofweek = TRUE,
      weight_threshold = 50
    )$participants.weights

    # make sure they add up to the sample size
    expect_equal(
      sum(weights.nothreshold[, weight * proportion]),
      1,
      tolerance = 1e-8
    )
    expect_equal(
      sum(weights.threshold3[, weight * proportion]),
      1,
      tolerance = 1e-8
    )
    expect_equal(
      sum(weights.threshold50[, weight * proportion]),
      1,
      tolerance = 1e-8
    )

    # check threshold values (include 2.5% margin due to the standardisation)
    expect_gt(max(weights.nothreshold$weight), 3 * 1.025)
    expect_lt(max(weights.threshold3$weight), 3 * 1.025) #
    # include small margin due to the standardisation
    expect_gt(max(weights.threshold50$weight), 3 * 1.02)
    # include small margin due to the standardisation
    expect_lt(max(weights.threshold50$weight), 50 * 1.025)
  })
})


test_that("participants with missing contact age are handled", {
  num.part <- nrow(polymod$participants)
  num.part.missing.age <- sum(is.na(polymod$participants$part_age))

  # keep missing participant and contact ages ==>> get original sample size
  expect_identical(
    sum(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_participant_age = "keep",
        missing_contact_age = "keep"
      )$participants$participants
    ),
    num.part
  )

  # remove missing participant ages
  # => get original sample size - num.part.missing.age
  expect_identical(
    sum(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_participant_age = "remove",
        missing_contact_age = "keep"
      )$participants$participants
    ),
    num.part - num.part.missing.age
  )

  # remove missing participant ages
  # => get original sample size - num.part.missing.age
  expect_lt(
    sum(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_participant_age = "remove",
        missing_contact_age = "remove"
      )$participants$participants
    ),
    num.part - num.part.missing.age
  )

  # keep missing contact ages ==>> additional column in contact matrix
  expect_identical(
    ncol(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_contact_age = "remove"
      )$matrix
    ),
    1L
  )

  expect_identical(
    ncol(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_contact_age = "keep"
      )$matrix
    ),
    2L
  )

  expect_identical(
    ncol(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_contact_age = "ignore"
      )$matrix
    ),
    1L
  )

  expect_identical(
    ncol(
      contact_matrix(
        survey = polymod,
        age_limits = 0,
        missing_contact_age = "sample"
      )$matrix
    ),
    1L
  )
})


test_that("user-defined reference populations with open age groups", {
  suppressWarnings({
    survey.pop <- data.frame(
      lower.age.limit = c(0, 4, 15),
      population = c(4e6, 1e5, 6e6)
    )

    # to handle the open ended age group in the survey.pop
    expect_identical(
      nrow(
        contact_matrix(
          polymod,
          age_limits = c(0, 18, 60),
          symmetric = TRUE, # to make sure that demography is returned
          survey_pop = survey.pop
        )$demography
      ),
      3L
    )

    # to check the column names
    expect_identical(
      names(
        contact_matrix(
          polymod,
          age_limits = c(0, 18, 60),
          symmetric = TRUE, # to make sure that demography is returned
          survey_pop = survey.pop
        )$demography
      )[1],
      "age.group"
    )

    polymod_nocountry <- polymod
    polymod_nocountry$participants$country <- NULL

    expect_snapshot(
      cran = FALSE,
      error = TRUE,
      contact_matrix(
        polymod_nocountry,
        age_limits = c(0, 18, 60),
        symmetric = TRUE, # to make sure that demography is returned
        survey_pop = "dummy"
      )
    )

    expect_error(contact_matrix(
      polymod_nocountry,
      age_limits = c(0, 18, 60),
      symmetric = TRUE, # to make sure that demography is returned
      survey_pop = "dummy"
    ))
  })
})

test_that("The absence of reference population info is going well", {
  suppressWarnings({
    polymod_nocountry <- polymod
    polymod_nocountry$participants$country <- NULL

    # no reference population given
    expect_identical(
      nrow(
        contact_matrix(
          polymod_nocountry,
          age_limits = c(0, 18, 60),
          symmetric = TRUE # to make sure that demography is returned
        )$demography
      ),
      3L
    )

    # to check the column names
    expect_identical(
      names(
        contact_matrix(
          polymod_nocountry,
          age_limits = c(0, 18, 60),
          symmetric = TRUE # to make sure that demography is returned
        )$demography
      )[1],
      "age.group"
    )
  })
})

test_that("Contact matrices per capita can be provided", {
  suppressWarnings({
    # get contact matrix per capita
    expect_type(
      contact_matrix(
        polymod,
        age_limits = c(0, 18, 60),
        per_capita = TRUE
      )$matrix.per.capita,
      "double"
    )

    # per capita matrix is not returned when counts=TRUE
    expect_null(
      contact_matrix(
        polymod,
        age_limits = c(0, 18, 60),
        per_capita = TRUE,
        counts = TRUE
      )$matrix.per.capita
    )

    # per capita matrix is not returned when split=TRUE
    expect_null(
      contact_matrix(
        polymod,
        age_limits = c(0, 18, 60),
        per_capita = TRUE,
        split = TRUE
      )$matrix.per.capita
    )
  })
})

test_that("Symmetric contact matrices per capita are actually symmetric", {
  suppressWarnings({
    # get contact matrix per capita
    matrix.per.capita <- contact_matrix(
      polymod,
      age_limits = c(0, 18, 60),
      symmetric = TRUE, # to make sure that demography is returned
      per_capita = TRUE
    )$matrix.per.capita

    expect_true(isSymmetric(matrix.per.capita, check.attributes = FALSE))
  })
})


test_that("Contact matrices per capita are also generated when bootstrapping", {
  suppressWarnings({
    # get contact matrix per capita
    expect_length(
      contact_matrix(polymod, age_limits = c(0, 18, 60), per_capita = TRUE),
      4
    )

    # get no contact matrix per capita
    expect_length(
      contact_matrix(polymod, age_limits = c(0, 18, 60), per_capita = FALSE),
      2
    )
  })
})


test_that("symmetric matrices with large norm weights warn", {
  expect_snapshot_warning(
    cran = FALSE,
    contact_matrix(
      survey = polymod,
      age_limits = c(0, 90),
      symmetric = TRUE
    )
  )
  expect_warning(
    contact_matrix(
      survey = polymod,
      age_limits = c(0, 90),
      symmetric = TRUE
    ),
    "artefacts after making the matrix symmetric"
  )
})

test_that("contacts below age limits excluded regardless of setting", {
  expect_identical(
    ncol(
      contact_matrix(
        polymod,
        age_limits = c(10, 50),
        missing_contact_age = "remove"
      )$matrix
    ),
    2L
  )
  expect_identical(
    ncol(
      contact_matrix(
        polymod,
        age_limits = c(10, 50),
        missing_contact_age = "sample"
      )$matrix
    ),
    2L
  )
  expect_identical(
    ncol(
      contact_matrix(
        polymod,
        age_limits = c(10, 50),
        missing_contact_age = "keep"
      )$matrix
    ),
    3L
  ) # extra column for ages outside age limits (= NA)
  expect_identical(
    ncol(
      contact_matrix(
        polymod,
        age_limits = c(10, 50),
        missing_contact_age = "ignore"
      )$matrix
    ),
    2L
  )
})

test_that("sample.all.age.groups errors when age group has no participants", {
  withr::local_options(lifecycle_verbosity = "quiet")
  # Create a survey with no participants in a specific age range
  polymod_limited <- get_survey(polymod)
  # Keep only participants aged 20+
  polymod_limited$participants <-
    polymod_limited$participants[part_age_exact >= 20]

  expect_error(
    contact_matrix(
      survey = polymod_limited,
      age_limits = c(0, 10, 20, 30),
      sample_participants = TRUE,
      sample_all_age_groups = TRUE
    ),
    "Cannot sample all age groups"
  )
})

test_that("sample_all_age_groups with bootstrap succeeds with enough tries", {
  # Test that sample_all_age_groups = TRUE works with sample_participants
  set.seed(42)
  result <- suppressMessages(
    contact_matrix(
      survey = polymod,
      countries = "United Kingdom",
      age_limits = c(0, 18, 65),
      sample_participants = TRUE,
      sample_all_age_groups = TRUE
    )
  )
  expect_true(is.matrix(result$matrix))
  expect_identical(nrow(result$matrix), 3L)
  expect_identical(ncol(result$matrix), 3L)
  # All age groups should be represented (no NA rows)
  expect_false(anyNA(rowSums(result$matrix)))
})

test_that("contact_matrix warns for multiple observations per participant", {
  # Create a survey with duplicate participants
  survey_dup <- polymod
  survey_dup$participants <- rbind(
    survey_dup$participants,
    survey_dup$participants[1:100, ]
  )

  expect_warning(
    contact_matrix(survey = survey_dup, countries = "United Kingdom"),
    "multiple observations per participant"
  )
})

test_that("deprecated argument names produce warnings", {
  # Test contact_matrix() deprecated arguments
  lifecycle::expect_deprecated(
    contact_matrix(polymod, age.limits = c(0, 18))
  )
  lifecycle::expect_deprecated(
    contact_matrix(polymod, age_limits = c(0, 18), weigh.dayofweek = TRUE)
  )

  # Test pop_age() deprecated arguments
  ages_it <- wpp_age("Italy", 2015)
  lifecycle::expect_deprecated(
    pop_age(ages_it, age.limits = c(0, 18, 65))
  )

  # Test as_contact_survey() deprecated arguments
  survey_list <- list(
    participants = polymod$participants,
    contacts = polymod$contacts,
    reference = polymod$reference
  )
  lifecycle::expect_deprecated(
    as_contact_survey(survey_list, id.column = "part_id")
  )
})

#' Reduce the number of age groups given a broader set of limits
#'
#' Operates on lower limits
#' @param x vector of limits
#' @param limits new limits
#' @return vector with the new age groups
#' @examples
#' reduce_agegroups(seq_len(20), c(0, 5, 10))
#' @export
reduce_agegroups <- function(x, limits) {
  ret <- x[NA]
  int <- findInterval(x, sort(limits))
  ret[!is.na(int) & int > 0] <-
    limits[int[!is.na(int) & int > 0]]
  ret
}

#' Assemble a contact survey with new participant/contact data
#'
#' Creates a new survey object preserving all fields from the original,
#' replacing only `participants` and `contacts` with the supplied data.
#'
#' @param x a `contact_survey` object
#' @param participants new participants data.table
#' @param contacts new contacts data.table
#' @returns a `contact_survey` object with all fields from `x` preserved
#' @keywords internal
assemble_survey <- function(x, participants, contacts) {
  result <- x
  result$participants <- participants
  result$contacts <- contacts
  result
}

#' Deep copy a contact survey
#'
#' Creates a deep copy of a `contact_survey` object, including its
#' `participants` and `contacts` data.tables.
#'
#' @param survey a `contact_survey` object
#' @returns a deep copy of the survey
#' @keywords internal
copy_survey <- function(survey) {
  result <- survey
  result$participants <- data.table::copy(survey$participants)
  result$contacts <- data.table::copy(survey$contacts)
  result
}

#' Resolve which table a filter expression targets
#'
#' Examines the variables referenced in a filter expression and determines
#' whether they belong to the participants table, the contacts table, or
#' neither. Errors if columns span both tables ambiguously.
#'
#' @param ref_vars character vector of variable names from the expression
#' @param participants a data.table of participants
#' @param contacts a data.table of contacts
#' @return `"participant"`, `"contact"`, or `"none"`
#' @noRd
resolve_filter_target <- function(ref_vars, participants, contacts) {
  part_cols <- intersect(ref_vars, colnames(participants))
  cont_cols <- intersect(ref_vars, colnames(contacts))

  ## Columns in both tables: allow if they are only key columns (part_id),
  ## otherwise error.
  shared <- intersect(part_cols, cont_cols)
  if (length(shared) > 0) {
    non_key <- setdiff(shared, "part_id")
    if (length(non_key) > 0) {
      cli::cli_abort(
        "Expression references columns from both participants \\
         ({.val {part_cols}}) and contacts ({.val {cont_cols}}). \\
         Filter one table at a time, e.g. \\
         {.code survey[part_col == x][cnt_col == y]}."
      )
    }
    ## Shared key columns only — treat as participant-side
    cont_cols <- setdiff(cont_cols, shared)
  }

  found_in_part <- length(part_cols) > 0
  found_in_cont <- length(cont_cols) > 0

  if (found_in_part && found_in_cont) {
    cli::cli_abort(
      "Expression references columns from both participants \\
       ({.val {part_cols}}) and contacts ({.val {cont_cols}}). \\
       Filter one table at a time, e.g. \\
       {.code survey[part_col == x][cnt_col == y]}."
    )
  }

  if (found_in_part) {
    return("participant")
  }
  if (found_in_cont) {
    return("contact")
  }

  unknown <- setdiff(ref_vars, c(colnames(participants), colnames(contacts)))
  if (length(unknown) > 0) {
    cli::cli_warn(
      "Column{?s} {.val {unknown}} not found in participants or contacts."
    )
  }
  "none"
}

#' Subset a contact survey
#'
#' @description
#' Filters a `contact_survey` object using an expression. The expression is
#' evaluated against whichever table(s) contain the referenced columns
#' (participants, contacts, or both). When participants are filtered, contacts
#' are automatically pruned to matching `part_id`s.
#'
#' @param x a `contact_survey` object
#' @param i an expression to evaluate as a row filter (e.g.
#'   `country == "United Kingdom"`)
#' @param ... ignored
#' @returns a filtered `contact_survey` object
#'
#' @examples
#' data(polymod)
#' polymod[country == "United Kingdom"]
#'
#' @method [ contact_survey
#' @export
#' @autoglobal
`[.contact_survey` <- function(x, i, ...) {
  expr <- substitute(i)
  if (missing(i) || is.null(expr)) {
    return(assemble_survey(x, copy(x$participants), copy(x$contacts)))
  }

  participants <- copy(x$participants)
  contacts <- copy(x$contacts)

  ref_vars <- all.vars(expr)

  if (length(ref_vars) == 0) {
    cli::cli_abort(
      "Column-based expressions are required, e.g. \\
       {.code survey[country == \"UK\"]}. \\
       Numeric or logical indexing is not supported."
    )
  }

  target <- resolve_filter_target(ref_vars, participants, contacts)

  if (target == "none") {
    return(assemble_survey(x, participants, contacts))
  }

  env <- parent.frame()

  if (target == "participant") {
    rows <- eval(expr, participants, env)
    participants <- participants[rows]
    contacts <- contacts[part_id %in% participants$part_id]
  } else {
    rows <- eval(expr, contacts, env)
    contacts <- contacts[rows]
  }

  assemble_survey(x, participants, contacts)
}

#' @autoglobal
has_names <- function(x, nm) {
  all(nm %in% names(x))
}

#' Warn if survey has multiple observations per participant
#'
#' @description
#' Issues a warning when a survey contains multiple observations per
#' participant (more rows than unique part_id values).
#'
#' @param participants participant data.table
#' @param observation_key optional column name(s) identifying observations
#' @param filter_hint character; "pipeline" for pipeline-style hint or
#'   "legacy" for contact_matrix-style hint
#' @returns NULL invisibly
#' @keywords internal
#' @autoglobal
warn_multiple_observations <- function(
  participants,
  observation_key = NULL,
  filter_hint = c("pipeline", "legacy")
) {
  filter_hint <- match.arg(filter_hint)
  n_participants <- uniqueN(participants$part_id)
  n_rows <- nrow(participants)
  if (n_participants >= n_rows) {
    return(invisible(NULL))
  }

  has_obs_key <- !is.null(observation_key) && length(observation_key) > 0
  hint <- if (has_obs_key && filter_hint == "pipeline") {
    cli::format_inline(
      "Use {.code survey[{observation_key} == ...]} to select specific \\
       observations before calling {.fn compute_matrix}."
    )
  } else if (has_obs_key) {
    cli::format_inline(
      "Use {.arg filter} to select by {.val {observation_key}}."
    )
  } else if (filter_hint == "pipeline") {
    "Filter the survey with {.code survey[...]} to select specific \\
     observations before calling {.fn compute_matrix}."
  } else {
    "Use the {.arg filter} argument to select specific observations."
  }

  cli::cli_warn(c(
    "Survey contains multiple observations per participant \\
     ({n_rows} rows, {n_participants} unique participants).",
    "*" = "Results will aggregate across all observations.",
    i = hint
  ))

  invisible(NULL)
}

#' Impute ages from ranges (generic helper)
#'
#' @description
#' Generic function to impute ages from min/max ranges. Works for both
#' participant and contact data by specifying the column prefix.
#'
#' @param data A data.table containing age data
#' @param prefix Column name prefix: "part_age" for participants, "cnt_age" for
#'   contacts
#' @param estimate Imputation method: "mean", "sample", or "missing"
#' @returns The data with ages imputed according to the specified method
#' @autoglobal
#' @keywords internal
impute_ages <- function(
  data,
  prefix,
  estimate = c("mean", "sample", "missing")
) {
  if (!is.data.frame(estimate)) {
    estimate <- rlang::arg_match(estimate)
  }

  # Build column names from prefix
  age_col <- prefix
  exact_col <- paste0(prefix, "_exact")
  min_col <- paste0(prefix, "_est_min")
  max_col <- paste0(prefix, "_est_max")

  age_cols_in_data <- has_names(data, c(min_col, max_col))
  if (!age_cols_in_data || identical(estimate, "missing")) {
    return(data)
  }

  if (identical(estimate, "mean")) {
    # Impute using mean of min/max range
    data[
      is.na(get(exact_col)) &
        !is.na(get(min_col)) &
        !is.na(get(max_col)),
      (age_col) := as.integer(rowMeans(.SD)),
      .SDcols = c(min_col, max_col)
    ]
  } else if (identical(estimate, "sample")) {
    # Impute by sampling uniformly from range; runif() excludes its upper
    # bound, so use max + 1 to make the integer draw inclusive of max.
    data[
      is.na(get(age_col)) &
        !is.na(get(min_col)) &
        !is.na(get(max_col)) &
        get(min_col) <= get(max_col),
      (age_col) := as.integer(runif(.N, get(min_col), get(max_col) + 1L))
    ]
  } else if (is.data.frame(estimate)) {
    # Impute by sampling from a supplied age distribution within [min, max]
    rows_to_impute <- data[,
      which(
        is.na(get(age_col)) &
          !is.na(get(min_col)) &
          !is.na(get(max_col)) &
          get(min_col) <= get(max_col)
      )
    ]
    if (length(rows_to_impute) > 0) {
      data[
        rows_to_impute,
        (age_col) := sample_from_age_distribution(
          get(min_col),
          get(max_col),
          estimate
        )
      ]
    }
  }

  data
}

#' Sample ages from a distribution within `[min, max]` bands
#' @param mins integer vector of lower bounds
#' @param maxs integer vector of upper bounds
#' @param distribution data.frame with `age` and `proportion` columns
#' @returns integer vector of sampled ages
#' @keywords internal
sample_from_age_distribution <- function(mins, maxs, distribution) {
  n <- length(mins)
  result <- integer(n)
  fell_back <- FALSE
  for (i in seq_len(n)) {
    lo <- mins[i]
    hi <- maxs[i]
    sub_dist <- distribution[distribution$age >= lo & distribution$age <= hi, ]
    if (nrow(sub_dist) == 0 || sum(sub_dist$proportion) == 0) {
      fell_back <- TRUE
      # Upper bound inclusive: runif() excludes its upper bound, so use hi + 1
      result[i] <- as.integer(runif(1, lo, hi + 1))
    } else if (nrow(sub_dist) == 1) {
      result[i] <- sub_dist$age
    } else {
      result[i] <- sample(sub_dist$age, size = 1, prob = sub_dist$proportion)
    }
  }
  if (fell_back) {
    cli::cli_warn(
      "Age distribution has no coverage for some age bands; \\
       falling back to uniform sampling for those contacts."
    )
  }
  result
}

#' Impute participant ages
#'
#' @description
#' Imputes participant survey data, where variables are named:
#'   "part_age_est_min" and "part_age_est_max". Uses mean imputation, sampling
#'   (hot deck), or leaves them as missing. These are controlled by the
#'   `estimate` argument.
#'
#' @param participants A survey data set of participants
#' @param estimate if set to "mean" (default), people whose ages are given as a
#'   range (in columns named "..._est_min" and "..._est_max") but not exactly
#'   (in a column named "..._exact") will have their age set to the mid-point of
#'   the range; if set to "sample", the age will be sampled from the range; if
#'   set to "missing", age ranges will be treated as missing
#'
#' @returns
#' The participant data, potentially with participant ages imputed depending on
#'   the `estimate` method and whether age columns are present in the data.
#'
#' @autoglobal
#' @keywords internal
impute_participant_ages <- function(
  participants,
  estimate = c("mean", "sample", "missing")
) {
  impute_ages(data = participants, prefix = "part_age", estimate = estimate)
}

#' @autoglobal
drop_ages_below_age_limit <- function(data, age_limits) {
  data[is.na(cnt_age) | cnt_age >= min(age_limits), ]
}

#' Impute contact ages
#'
#' @description
#' Imputes contact survey data, where variables are named:
#'   "cnt_age_est_min" and "cnt_age_est_max". Uses mean imputation, sampling
#'   (hot deck), or leaves them as missing. These are controlled by the
#'   `estimate` argument.
#'
#' @param contacts a survey data set of contacts
#' @param estimate if set to "mean" (default), contacts whose ages are given as
#'   a range (in columns named "..._est_min" and "..._est_max") but not exactly
#'   (in a column named "..._exact") will have their age set to the mid-point of
#'   the range; if set to "sample", the age will be sampled from the range; if
#'   set to "missing", age ranges will be treated as missing
#'
#' @returns
#' The contact data, potentially with contact ages imputed depending on the
#'   `estimate` method and whether age columns are present in the data.
#'
#' @autoglobal
#' @keywords internal
impute_contact_ages <- function(
  contacts,
  estimate = c("mean", "sample", "missing")
) {
  impute_ages(data = contacts, prefix = "cnt_age", estimate = estimate)
}

#' @autoglobal
drop_missing_contact_ages <- function(contacts, missing_action) {
  if (missing_action == "ignore" && nrow(contacts[is.na(cnt_age)]) > 0) {
    contacts <- contacts[!is.na(cnt_age), ]
  }
  contacts
}

#' @autoglobal
max_participant_age <- function(data) {
  if (has_names(data, c("part_age_est_max", "part_age_exact"))) {
    part_age_data <- c(data[, part_age_exact], data[, part_age_est_max])
    max_year <- max(part_age_data, na.rm = TRUE) + 1
  } else {
    max_year <- max(data[, part_age], na.rm = TRUE) + 1
  }
  max_year
}

#' @autoglobal
create_age_breaks <- function(age_limits, max_age) {
  c(age_limits, max(max_age, max(age_limits) + 1))
}

#' @autoglobal
get_age_group_lower_limits <- function(age_limits) {
  age_limits
}

#' @autoglobal
get_age_limits <- function(survey) {
  participants <- survey$participants
  contacts <- survey$contacts

  part_ages <- if ("part_age" %in% colnames(participants)) {
    as.integer(participants[, part_age])
  } else if ("part_age_exact" %in% colnames(participants)) {
    as.integer(participants[, part_age_exact])
  } else {
    integer(0)
  }

  cnt_ages <- if ("cnt_age" %in% colnames(contacts)) {
    as.integer(contacts[, cnt_age])
  } else if ("cnt_age_exact" %in% colnames(contacts)) {
    as.integer(contacts[, cnt_age_exact])
  } else {
    integer(0)
  }

  all_ages <- c(part_ages, cnt_ages)
  unique_non_missing_ages <- unique(all_ages[!is.na(all_ages)])
  union(0, sort(unique_non_missing_ages))
}

#' @autoglobal
filter_countries <- function(participants, countries) {
  multiple_countries <- length(countries) > 0
  country_col_in_participants <- "country" %in% colnames(participants)
  if (multiple_countries && country_col_in_participants) {
    countries <- flexible_countrycode(countries)
    participants <- participants[country %in% countries]
    if (nrow(participants) == 0) {
      cli::cli_abort(
        "No participants left after selecting countries: {.val {countries}}"
      )
    }
  }
  participants
}

#' Add age column from exact age (generic helper)
#'
#' @description
#' Generic function to add an age column from an exact age column. Works for
#' both participant and contact data by specifying the column prefix.
#' If `<prefix>_exact` exists, it overwrites `<prefix>` with its values.
#' Otherwise, it creates `<prefix>` with NA values if it doesn't exist.
#'
#' @param data A data.table containing age data
#' @param prefix Column name prefix: "part_age" for participants, "cnt_age" for
#'   contacts
#' @returns The data with the age column set from exact ages or
#'   initialised to NA
#' @autoglobal
#' @keywords internal
add_age <- function(data, prefix) {
  age_col <- prefix
  exact_col <- paste0(prefix, "_exact")

  if (exact_col %in% colnames(data)) {
    data <- data[, (age_col) := as.integer(get(exact_col))]
  } else if (!(age_col %in% colnames(data))) {
    data <- data[, (age_col) := NA_integer_]
  }
  data
}

#' @autoglobal
add_part_age <- function(participants) {
  add_age(data = participants, prefix = "part_age")
}

#' @autoglobal
add_contact_age <- function(contacts) {
  add_age(data = contacts, prefix = "cnt_age")
}

#' @autoglobal
drop_invalid_ages <- function(
  participants,
  missing_action,
  age_limits
) {
  if (is.null(age_limits)) {
    cli::cli_abort("{.arg age_limits} must be provided")
  }
  ppt_no_age_info <- participants[is.na(part_age) | part_age < min(age_limits)]
  no_age_info <- nrow(ppt_no_age_info) > 0
  if (missing_action == "remove" && no_age_info) {
    participants <- participants[!is.na(part_age) & part_age >= min(age_limits)]
  }
  participants
}

#' @autoglobal
drop_invalid_contact_ages <- function(
  contacts,
  participants,
  missing_action
) {
  if (missing_action == "remove" && nrow(contacts[is.na(cnt_age)]) > 0) {
    missing.age.id <- contacts[is.na(cnt_age), part_id]
    participants <- participants[!(part_id %in% missing.age.id)]
  }
  participants
}

## convert factors to integers, preserving numeric values
#' @autoglobal
convert_factor_to_integer <- function(
  data,
  cols
) {
  which_factors <- sapply(data, is.factor)
  factor_cols <- intersect(cols, names(data)[which_factors])
  data[,
    (factor_cols) := lapply(.SD, function(x) {
      # Convert factor levels to integers - non-numeric levels become NA
      num_levels <- suppressWarnings(as.integer(levels(x)))
      # Warn if any level failed to parse (i.e. non-numeric)
      if (any(is.na(num_levels) & !is.na(levels(x)))) {
        cli::cli_warn("Non-numeric factor levels found; these will become NA")
      }
      num_levels[x]
    }),
    .SDcols = factor_cols
  ]
}

## check if any filters have been requested
#' @autoglobal
apply_data_filter <- function(
  survey,
  survey_type,
  filter,
  call = rlang::caller_env()
) {
  if (!is.null(filter)) {
    missing_columns <- list()
    for (table in survey_type) {
      if (nrow(survey[[table]]) > 0) {
        missing_columns <- c(
          missing_columns,
          list(setdiff(names(filter), colnames(survey[[table]])))
        )
        ## filter contact data
        for (column in names(filter)) {
          if (column %in% colnames(survey[[table]])) {
            survey[[table]] <- survey[[table]][get(column) == filter[[column]]]
          }
        }
      }
    }
    missing_all <- do.call(intersect, missing_columns)
    if (length(missing_all) > 0) {
      cli::cli_warn(
        message = "Filter column{?s}: {.val {missing_all}} not found.",
        call = call
      )
    }
  }
  survey
}

# converts from [0,1) [1,5) [5,15) [15,80) to [0,1) [1,5) [5,15) [15,Inf)
#' @autoglobal
final_age_group_label <- function(age_groups) {
  last <- age_groups[length(age_groups)]
  lower <- sub("\\[([0-9]+),.*$", "\\1", last)
  age_groups[length(age_groups)] <- paste0("[", lower, ",Inf)")
  age_groups
}

# adjust age.group.breaks to the lower and upper ages in the survey
#' @autoglobal
adjust_ppt_age_group_breaks <- function(
  participants,
  age_limits
) {
  max_age <- max_participant_age(participants)

  participants[,
    lower.age.limit := reduce_agegroups(
      x = part_age,
      limits = age_limits
    )
  ]

  part_age_group_breaks <- create_age_breaks(age_limits, max_age)

  participants[,
    age.group := cut(
      participants[, part_age],
      breaks = part_age_group_breaks,
      right = FALSE
    )
  ]

  age.groups <- age_group_labels(participants)

  participants[,
    age.group := factor(
      age.group,
      levels = levels(age.group),
      labels = age.groups
    )
  ]

  part_age_group_present <- get_age_group_lower_limits(age_limits)

  participants <- add_upper_age_limits(
    participants = participants,
    part_age_group_present = part_age_group_present,
    part_age_group_breaks = part_age_group_breaks
  )

  participants
}

#' @autoglobal
age_group_labels <- function(participants) {
  age_groups <- participants[, levels(age.group)]
  age_groups <- final_age_group_label(age_groups)
  age_groups
}

#' @autoglobal
add_upper_age_limits <- function(
  participants,
  part_age_group_present,
  part_age_group_breaks
) {
  lower_upper_age_limits <- data.table(
    lower.age.limit = part_age_group_present,
    upper.age.limit = part_age_group_breaks[-1]
  )

  participants <- merge(
    participants,
    lower_upper_age_limits,
    by = "lower.age.limit",
    all.x = TRUE
  )
  participants
}

#' @autoglobal
survey_pop_from_data <- function(survey_pop, part_age_group_present) {
  survey_pop <- data.table(survey_pop)
  # make sure max survey_pop age exceeds participant age group breaks
  if (max(survey_pop$lower.age.limit) < max(part_age_group_present)) {
    survey_pop <- rbind(
      survey_pop,
      list(max(part_age_group_present + 1), 0)
    )
  }
  survey_pop
}

#' @autoglobal
get_survey_countries <- function(survey_pop, countries, participants) {
  if (!is.null(survey_pop)) {
    ## survey population is given as vector of countries
    survey_countries <- survey_pop
  } else if (!is.null(countries)) {
    ## survey population not given but countries requested from
    ## survey - get population data from those countries
    survey_countries <- countries
  } else if ("country" %in% colnames(participants)) {
    ## neither survey population nor country names given - try to
    ## guess country or countries surveyed from participant data
    survey_countries <- unique(participants[, country])
  }
  survey_countries
}

#' @autoglobal
survey_is_representative <- function(countries, participants, survey_pop) {
  no_countries <- is.null(countries) && !("country" %in% colnames(participants))
  survey_representative <- is.null(survey_pop) && no_countries
  survey_representative
}

#' @autoglobal
survey_pop_from_countries <- function(
  survey_pop,
  countries,
  participants,
  age_limits,
  call = rlang::caller_env()
) {
  # no countries, and no survey_pop
  survey_representative <- survey_is_representative(
    countries = countries,
    participants = participants,
    survey_pop = survey_pop
  )

  warn_if_no_survey_countries(survey_representative, call = call)

  # there aren't countries or survey pop, get the countries
  if (!survey_representative) {
    survey_countries <- get_survey_countries(
      survey_pop = survey_pop,
      countries = countries,
      participants = participants
    )
    ## get population data for countries from 'wpp' package
    country_pop <- data.table(wpp_age(survey_countries))

    country_pop$country <- normalise_country_names(country_pop$country)

    ## check if survey data are from a specific year - in that case
    ## use demographic data from that year, otherwise latest
    if ("year" %in% colnames(participants)) {
      survey_year <- participants[, median(year, na.rm = TRUE)]
    } else {
      survey_year <- country_pop[, max(year, na.rm = TRUE)]
      cli::cli_warn(
        "No information on {.val year} found in the data. Will use
            {.val {survey_year}} population data."
      )
    }

    ## check if any survey countries are not in wpp
    check_any_missing_countries(survey_countries, country_pop)

    ## get demographic data closest to survey year
    country_pop_year <- unique(country_pop[, year])
    survey_year <- min(
      country_pop_year[which.min(abs(survey_year - country_pop_year))]
    )
    survey_pop <- country_pop[year == survey_year][,
      list(population = sum(population)),
      by = "lower.age.limit"
    ]
  }

  if (survey_representative) {
    survey_pop <- participants[, list(population = .N), by = lower.age.limit]
    survey_pop <- survey_pop[!is.na(lower.age.limit)]
    if ("year" %in% colnames(participants)) {
      survey_year <- participants[, median(year, na.rm = TRUE)]
    } else {
      survey_year <- NULL
    }
  }

  list(
    survey_year = survey_year,
    survey_pop = survey_pop
  )
}

#' @autoglobal
survey_pop_year <- function(
  survey_pop,
  countries,
  participants,
  age_limits
) {
  if (is.null(survey_pop) || is.character(survey_pop)) {
    survey_pop_info <- survey_pop_from_countries(
      survey_pop = survey_pop,
      countries = countries,
      participants = participants,
      age_limits = age_limits
    )
    survey_pop <- survey_pop_info$survey_pop
    survey_year <- survey_pop_info$survey_year
  } else {
    part_age_group_present <- get_age_group_lower_limits(age_limits)
    # survey_pop is a data frame with 'lower.age.limit' and 'population'
    survey_pop <- survey_pop_from_data(survey_pop, part_age_group_present)

    # add dummy survey_year
    survey_year <- NA_integer_
  }

  list(
    survey_pop = survey_pop,
    survey_year = survey_year
  )
}

#' @autoglobal
add_survey_upper_age_limit <- function(survey, age_breaks) {
  # add upper.age.limit after sorting the survey ages (and add
  # maximum age > given ages)
  survey <- survey[order(lower.age.limit), ]
  # if any lower age limits are missing remove them
  survey <- survey[!is.na(population)]
  survey$upper.age.limit <- unlist(c(
    survey[-1, "lower.age.limit"],
    1 + max(survey$lower.age.limit, age_breaks)
  ))
  survey
}

#' @autoglobal
survey_pop_reference <- function(survey_pop, ...) {
  data.table(
    pop_age(
      survey_pop,
      seq(
        min(survey_pop$lower.age.limit),
        max(survey_pop$upper.age.limit)
      ),
      ...
    )
  )
}

#' @autoglobal
adjust_survey_age_groups <- function(survey_pop, part_age_group_present, ...) {
  survey_pop_max <- max(survey_pop$upper.age.limit)
  survey_pop <- data.table(pop_age(survey_pop, part_age_group_present, ...))

  ## use the actual lower.age.limits from survey_pop (which may be a subset
  ## of part_age_group_present if population data doesn't cover all ages)
  actual_age_limits <- survey_pop$lower.age.limit

  ## set upper age limits
  survey_pop[,
    upper.age.limit := c(actual_age_limits[-1], survey_pop_max)
  ]
}

#' @autoglobal
weight_by_day_of_week <- function(
  participants,
  call = rlang::caller_env()
) {
  found_dayofweek <- FALSE
  if ("dayofweek" %in% colnames(participants)) {
    ## Add column sum_weight: Number of entries on weekdays / weekends
    participants[,
      sum_weight := nrow(.SD),
      by = (dayofweek %in% 1:5),
    ]

    ## The sum of the weights on weekdays is 5
    participants[dayofweek %in% 1:5, weight := 5 / sum_weight]
    ## The sum of the weights on weekend is 2
    participants[!(dayofweek %in% 1:5), weight := 2 / sum_weight]

    participants[, sum_weight := NULL]
    found_dayofweek <- TRUE

    # add boolean for "weekday"
    participants[, is.weekday := dayofweek %in% 1:5]
  }
  if (!found_dayofweek) {
    cli::cli_warn(
      message = c(
        "{.code weigh_dayofweek} is {.val TRUE}, but no {.col dayofweek} \\
          column in the data.",
        # nolint start
        "i" = "Will ignore."
        # nolint end
      ),
      call = call
    )
  }
  participants
}

#' @autoglobal
weight_by_age <- function(participants, survey_pop_full) {
  # get number and proportion of participants by age
  participants[, age.count := .N, by = part_age]
  participants[, age.proportion := age.count / .N]

  # get reference population by age (absolute and proportional)
  part_age_all <- range(unique(participants[, part_age]))
  survey_pop_detail <- data.table(pop_age(
    survey_pop_full,
    seq(part_age_all[1], part_age_all[2] + 1)
  ))
  names(survey_pop_detail) <- c("part_age", "population.count")
  survey_pop_detail[,
    population.proportion := population.count / sum(population.count)
  ]

  # merge reference and survey population data
  participants <- merge(
    participants,
    survey_pop_detail,
    by = "part_age"
  )

  # calculate age-specific weights
  participants[, weight.age := population.proportion / age.proportion]

  # merge 'weight.age' into 'weight'
  participants[, weight := weight * weight.age]

  ## Remove the additional columns
  participants[, age.count := NULL]
  participants[, age.proportion := NULL]
  participants[, population.count := NULL]
  participants[, population.proportion := NULL]
  participants[, weight.age := NULL]
}

#' @autoglobal
weigh_by_user_defined <- function(participants, weights) {
  for (i in seq_along(weights)) {
    if (weights[i] %in% colnames(participants)) {
      ## Compute the overall weight
      participants[, weight := weight * get(weights[i])]
    }
  }
  participants
}

#' Post-stratification weight normalisation
#'
#' @description
#' Normalises participant weights within groups so that they sum to the number
#' of participants in each group. Optionally truncates extreme weights to a
#' threshold and re-normalises.
#'
#' @param participants participant data.table with a `weight` column
#' @param by character; column name(s) to group by (default "age.group")
#' @param threshold numeric; if provided, weights above this value are capped
#'   and the weights are re-normalised (default NULL)
#' @returns the participants data.table (modified by reference)
#' @keywords internal
#' @autoglobal
normalise_weights <- function(
  participants,
  by = "age.group",
  threshold = NULL
) {
  participants[, weight := weight / sum(weight) * .N, by = c(by)]

  if (!is.null(threshold) && !is.na(threshold)) {
    participants[weight > threshold, weight := threshold]
    participants[, weight := weight / sum(weight) * .N, by = c(by)]
  }

  invisible(participants)
}

## merge participants and contacts into a single data table
#' @autoglobal
merge_participants_contacts <- function(participants, contacts) {
  setkey(participants, part_id)

  # Merge participants with contacts, allowing Cartesian products as one
  # participant can have multiple contacts
  contacts <- merge(
    x = contacts,
    y = participants,
    by = "part_id",
    all = FALSE,
    allow.cartesian = TRUE,
    suffixes = c(".cont", ".part")
  )

  setkey(contacts, part_id)

  contacts
}

## some contacts in the age group have an age, sample from these
#' @autoglobal
sample_present_age <- function(contacts, this_age_group) {
  contacts[
    is.na(cnt_age) & age.group == this_age_group,
    cnt_age := sample(
      contacts[
        !is.na(cnt_age) & age.group == this_age_group,
        cnt_age
      ],
      size = .N,
      replace = TRUE
    )
  ]
}

#' @autoglobal
sample_uniform_age <- function(contacts, this_age_group) {
  min_contact_age <- contacts[, min(cnt_age, na.rm = TRUE)]
  max_contact_age <- contacts[, max(cnt_age, na.rm = TRUE)]
  contacts[
    is.na(cnt_age) & age.group == this_age_group,
    cnt_age := as.integer(floor(runif(
      .N,
      min = min_contact_age,
      max = max_contact_age + 1
    )))
  ]
}

#' @autoglobal
impute_age_by_sample <- function(contacts) {
  for (this_age_group in unique(contacts[is.na(cnt_age), age.group])) {
    ## first, deal with missing age
    if (nrow(contacts[!is.na(cnt_age) & age.group == this_age_group]) > 0) {
      ## some contacts in the age group have an age, sample from these
      contacts <- sample_present_age(contacts, this_age_group)
    } else if (nrow(contacts[!is.na(cnt_age), ]) > 0) {
      ## no contacts in the age group have an age, sample uniformly
      contacts <- sample_uniform_age(contacts, this_age_group)
    }
  }
  # make sure the final set does not contain NA's anymore
  contacts <- contacts[!is.na(cnt_age), ]

  contacts
}

#' @autoglobal
add_contact_age_groups <- function(
  contacts,
  age_breaks,
  age_groups
) {
  max_contact_age <- contacts[, max(cnt_age, na.rm = TRUE) + 1]

  if (max_contact_age > max(age_breaks)) {
    age_breaks[length(age_breaks)] <- max_contact_age
  }
  contacts[,
    contact.age.group := cut(
      cnt_age,
      breaks = age_breaks,
      labels = age_groups,
      right = FALSE
    )
  ]
}

#' @autoglobal
create_bootstrap_weights <- function(part_sample) {
  sample_table <- data.table(id = part_sample, weight = 1)
  sample_table <- sample_table[,
    list(bootstrap.weight = sum(weight)),
    by = id
  ]
  setnames(sample_table, "id", "part_id")
  setkey(sample_table, part_id)
  sample_table
}

#' @autoglobal
sample_from_participants <- function(
  participants,
  contacts,
  age_limits,
  sample_all_age_groups,
  max.tries = 1000
) {
  participant_ids <- unique(participants$part_id)

  ## check upfront if sampling all age groups is possible
  if (sample_all_age_groups) {
    present_age_limits <- unique(participants$lower.age.limit)
    missing_age_limits <- setdiff(age_limits, present_age_limits)
    if (length(missing_age_limits) > 0) {
      cli::cli_abort(
        "Cannot sample all age groups: no participants in age groups
        starting at {.val {missing_age_limits}}."
      )
    }
  }

  good_sample <- FALSE
  tries <- 0L
  while (!good_sample) {
    tries <- tries + 1L
    if (sample_all_age_groups && tries > max.tries) {
      cli::cli_abort(
        "Failed to draw a bootstrap sample covering all age groups after
        {.val {max.tries}} attempts."
      )
    }

    ## take a sample from the participants
    part_sample <- sample(participant_ids, replace = TRUE)
    part_age_limits <- unique(
      participants[part_id %in% part_sample, lower.age.limit]
    )
    age_limits_match_part <- setequal(age_limits, part_age_limits)
    good_sample <- !sample_all_age_groups || age_limits_match_part

    sample_table <- create_bootstrap_weights(part_sample)

    sampled_contacts <- merge(contacts, sample_table, by = "part_id")
    sampled_contacts[, sampled.weight := weight * bootstrap.weight]

    sampled_participants <- merge(participants, sample_table)
    sampled_participants[, sampled.weight := weight * bootstrap.weight]
  }

  list(
    sampled_contacts = sampled_contacts,
    sampled_participants = sampled_participants
  )
}

#' @autoglobal
sample_contacts_participants <- function(
  sample_participants,
  participants,
  contacts,
  age_limits,
  sample_all_age_groups,
  max.tries = 1000
) {
  if (sample_participants) {
    sampled_contacts_participants <- sample_from_participants(
      participants,
      contacts,
      age_limits,
      sample_all_age_groups,
      max.tries
    )
  } else {
    ## just use all participants
    sampled_contacts_participants <- list(
      sampled_contacts = contacts[, sampled.weight := weight],
      sampled_participants = participants[, sampled.weight := weight]
    )
  }
  sampled_contacts_participants
}

#' @autoglobal
weighted_matrix_array <- function(contacts) {
  weighted_matrix <- xtabs(
    data = contacts,
    formula = sampled.weight ~ age.group + contact.age.group,
    addNA = TRUE
  )

  array(
    weighted_matrix,
    dim = dim(weighted_matrix),
    dimnames = dimnames(weighted_matrix)
  )
}

#' @autoglobal
calculate_weighted_matrix <- function(
  sampled_contacts = sampled_contacts,
  sampled_participants = sampled_participants,
  survey_pop = survey_pop,
  symmetric,
  counts,
  symmetric_norm_threshold
) {
  weighted_matrix <- weighted_matrix_array(
    contacts = sampled_contacts
  )

  if (!counts) {
    ## normalise to give mean number of contacts
    weighted_matrix <- normalise_weights_to_counts(
      sampled_participants = sampled_participants,
      weighted_matrix = weighted_matrix
    )
  }

  warn_symmetric_counts_na(symmetric, counts, weighted_matrix)
  matrix_not_scalar <- prod(dim(as.matrix(weighted_matrix))) > 1
  na_in_weighted_mtx <- na_in_weighted_matrix(weighted_matrix)
  if (symmetric && matrix_not_scalar && !na_in_weighted_mtx) {
    weighted_matrix <- normalise_weighted_matrix(
      survey_pop = survey_pop,
      weighted_matrix = weighted_matrix,
      symmetric_norm_threshold = symmetric_norm_threshold
    )
  }
  weighted_matrix
}

#' @autoglobal
normalise_weights_to_counts <- function(sampled_participants, weighted_matrix) {
  ## normalise to give mean number of contacts
  ## calculate normalisation vector
  norm_vector <- c(xtabs(
    data = sampled_participants,
    formula = sampled.weight ~ age.group,
    addNA = TRUE
  ))

  ## normalise contact matrix
  weighted_matrix <- weighted_matrix / norm_vector

  ## set non-existent data to NA
  weighted_matrix[is.nan(weighted_matrix)] <- NA_real_

  weighted_matrix
}

## construct a warning in case there are NAs
#' @autoglobal
build_na_warning <- function(weighted_matrix) {
  na_headers <- anyNA(dimnames(weighted_matrix), recursive = TRUE)
  na_content <- anyNA(weighted_matrix)
  na_present <- na_headers || na_content

  warning_suggestion <- NULL
  if (na_present) {
    warning_suggestion <- "  Consider "
    if (na_headers) {
      warning_suggestion <- paste0(warning_suggestion, "setting ")
      suggested_options <- NULL
      if (anyNA(rownames(weighted_matrix))) {
        suggested_options <- c(suggested_options, "'missing.participant.age'")
      }
      if (anyNA(colnames(weighted_matrix))) {
        suggested_options <- c(suggested_options, "'missing.contact.age'")
      }

      warning_suggestion <-
        paste0(warning_suggestion, paste(suggested_options, collapse = " and "))
      if (na_content) {
        warning_suggestion <- paste0(warning_suggestion, ", and ")
      } else {
        warning_suggestion <- paste0(warning_suggestion, ".")
      }
    }
    if (na_content) {
      warning_suggestion <- paste0(
        warning_suggestion,
        "adjusting the age limits."
      )
    }
  }
  warning_suggestion
}

#' @autoglobal
na_in_weighted_matrix <- function(weighted_matrix) {
  na_headers <- anyNA(dimnames(weighted_matrix), recursive = TRUE)
  na_content <- anyNA(weighted_matrix)
  na_present <- na_headers || na_content

  na_present
}

#' @autoglobal
normalisation_factors <- function(normalised_matrix, weighted_matrix) {
  normalisation_fctr <- c(
    normalised_matrix / weighted_matrix,
    weighted_matrix / normalised_matrix
  )
  normalisation_fctr <- normalisation_fctr[
    !is.infinite(normalisation_fctr) & !is.na(normalisation_fctr)
  ]
  normalisation_fctr
}

#' @autoglobal
normalise_weighted_matrix <- function(
  survey_pop,
  weighted_matrix,
  symmetric_norm_threshold,
  call = rlang::caller_env()
) {
  ## set c_{ij} N_i and c_{ji} N_j (which should both be equal) to
  ## 0.5 * their sum; then c_{ij} is that sum / N_i
  normalised_weighted_matrix <- survey_pop$population * weighted_matrix
  normalised_weighted_matrix <- 0.5 /
    survey_pop$population *
    (normalised_weighted_matrix + t(normalised_weighted_matrix))

  warn_norm_fct_exceed_thresh(
    normalised_weighted_matrix = normalised_weighted_matrix,
    weighted_matrix = weighted_matrix,
    symmetric_norm_threshold = symmetric_norm_threshold
  )

  normalised_weighted_matrix
}

#' @autoglobal
mean_contacts_per_person <- function(population, num_contacts) {
  mean_contacts <- sum(population * num_contacts) / sum(population)
  mean_contacts
}

#' @autoglobal
get_spectral_radius <- function(weighted_matrix) {
  spectrum_matrix <- weighted_matrix
  spectrum_matrix[is.na(spectrum_matrix)] <- 0
  eigen_values <- eigen(spectrum_matrix, only.values = TRUE)
  # get the largest eigenvalue
  spectral_radius <- as.numeric(eigen_values$values[1])
  spectral_radius
}

#' @autoglobal
split_mean_norm_contacts <- function(
  weighted_matrix,
  population
) {
  ## get rid of name but preserve row and column names
  weighted_matrix <- unname(weighted_matrix)

  num_contacts <- rowSums(weighted_matrix)

  mean_contacts <- mean_contacts_per_person(
    population = population,
    num_contacts = num_contacts
  )
  # Maximum growth rate of the infection process
  # the dominant eigenvalue or the spectral radius
  spectral_radius <- get_spectral_radius(weighted_matrix)
  # normalise: how much more transmission potential from pop. structure
  normalisation <- spectral_radius / mean_contacts
  age_proportions <- population / sum(population)
  weighted_matrix <- diag(1 / num_contacts) %*%
    weighted_matrix %*%
    diag(1 / age_proportions)
  num_contacts <- num_contacts / spectral_radius

  list(
    weighted_matrix = weighted_matrix,
    mean_contacts = mean_contacts,
    normalisation = normalisation,
    contacts = num_contacts
  )
}

#' @autoglobal
matrix_per_capita <- function(weighted_matrix, survey_pop) {
  weighted_matrix_per_capita <- weighted_matrix /
    matrix(
      rep(survey_pop$population, nrow(survey_pop)),
      ncol = nrow(survey_pop),
      byrow = TRUE
    )
  weighted_matrix_per_capita
}

#' @autoglobal
n_participants_per_age_group <- function(participants) {
  participant_population <- data.table(table(
    participants[, age.group],
    useNA = "ifany"
  ))
  setnames(participant_population, c("age.group", "participants"))
  participant_population[, proportion := participants / sum(participants)]
  participant_population
}

#' @autoglobal
return_participant_weights <- function(
  survey_participants,
  weigh_age,
  weigh_dayofweek
) {
  # default
  part_weights <- survey_participants[, .N, by = list(age.group, weight)]
  part_weights <- part_weights[order(age.group, weight), ]

  # add age and/or dayofweek info
  if (weigh_age && weigh_dayofweek) {
    part_weights <- survey_participants[,
      .N,
      by = list(age.group, participant.age = part_age, is.weekday, weight)
    ]
  }

  if (weigh_age && !weigh_dayofweek) {
    part_weights <- survey_participants[,
      .N,
      by = list(age.group, participant.age = part_age, weight)
    ]
  }

  if (weigh_dayofweek && !weigh_age) {
    part_weights <- survey_participants[,
      .N,
      by = list(age.group, is.weekday, weight)
    ]
  }

  # order (from left to right)
  part_weights <- part_weights[order(part_weights), ] # nolint

  # set name of last column
  names(part_weights)[ncol(part_weights)] <- "participants"

  part_weights[, proportion := participants / sum(participants)]
  part_weights[]
}

test_that("get_age_limits uses both participant and contact ages", {
  # Create a mock survey where contacts have higher ages than participants
  mock_survey <- list(
    participants = data.table::data.table(
      part_id = 1:3,
      part_age = c(10, 20, 30)
    ),
    contacts = data.table::data.table(
      part_id = c(1, 1, 2, 3),
      cnt_age_exact = c(15, 50, 25, 80)
    )
  )

  age_limits <- get_age_limits(mock_survey)

  # Should include 0 and all unique ages from both participants and contacts
  expect_true(0 %in% age_limits)
  expect_true(10 %in% age_limits) # participant age
  expect_true(20 %in% age_limits) # participant age
  expect_true(30 %in% age_limits) # participant age
  expect_true(15 %in% age_limits) # contact age
  expect_true(50 %in% age_limits) # contact age
  expect_true(80 %in% age_limits) # contact age
})

test_that("get_age_limits works with only participant ages", {
  mock_survey <- list(
    participants = data.table::data.table(
      part_id = 1:2,
      part_age = c(5, 15)
    ),
    contacts = data.table::data.table(
      part_id = c(1, 2)
      # no age columns
    )
  )

  age_limits <- get_age_limits(mock_survey)

  expect_identical(age_limits, c(0, 5, 15))
})

test_that("get_age_limits works with only contact ages", {
  mock_survey <- list(
    participants = data.table::data.table(
      part_id = 1:2
      # no age columns
    ),
    contacts = data.table::data.table(
      part_id = c(1, 2),
      cnt_age_exact = c(10, 20)
    )
  )

  age_limits <- get_age_limits(mock_survey)

  expect_identical(age_limits, c(0, 10, 20))
})

test_that("get_age_limits handles NA values", {
  mock_survey <- list(
    participants = data.table::data.table(
      part_id = 1:3,
      part_age = c(10, NA, 30)
    ),
    contacts = data.table::data.table(
      part_id = c(1, 2),
      cnt_age_exact = c(NA, 50)
    )
  )

  age_limits <- get_age_limits(mock_survey)

  # Should exclude NAs
  expect_false(anyNA(age_limits))
  expect_identical(age_limits, c(0, 10, 30, 50))
})

polymod_uk <- polymod[country == "United Kingdom"]
polymod_grouped <- assign_age_groups(polymod_uk, age_limits = c(0, 5, 15))

test_that("weigh() with dayofweek groups produces correct weights", {
  result <- weigh(
    polymod_grouped,
    "dayofweek",
    target = c(5, 2),
    groups = list(1:5, c(0, 6))
  )

  ppt <- result$participants
  has_dow <- !is.na(ppt$dayofweek)
  weekday <- has_dow & ppt$dayofweek %in% 1:5
  weekend <- has_dow & ppt$dayofweek %in% c(0, 6)
  no_dow <- is.na(ppt$dayofweek)

  n_weekday <- sum(weekday)
  n_weekend <- sum(weekend)

  expect_equal(unique(ppt$weight[weekday]), 5 / n_weekday, tolerance = 1e-10)
  expect_equal(unique(ppt$weight[weekend]), 2 / n_weekend, tolerance = 1e-10)
  expect_equal(
    unique(ppt$weight[no_dow]),
    7 / nrow(ppt),
    tolerance = 1e-10
  )
})

test_that("weigh() with dayofweek groups has positive weights", {
  result <- weigh(
    polymod_grouped,
    "dayofweek",
    target = c(5, 2),
    groups = list(1:5, c(0, 6))
  )

  has_dow <- !is.na(result$participants$dayofweek)
  expect_true(all(
    result$participants$weight[has_dow] > 0
  ))
})

test_that("weigh() with population df matches weight_by_age()", {
  uk_pop <- wpp_age("United Kingdom", 2005)

  result <- weigh(polymod_grouped, "age.group", target = uk_pop)

  ref <- copy(polymod_grouped)
  ref$participants[, weight := 1]
  age_limits <- c(0, 5, 15)
  part_age_group_present <- age_limits
  survey_pop <- data.table(uk_pop)
  survey_pop <- add_survey_upper_age_limit(
    survey = survey_pop,
    age_breaks = part_age_group_present
  )
  survey_pop_full <- survey_pop_reference(survey_pop)
  ref$participants <- weight_by_age(ref$participants, survey_pop_full)

  expect_equal(
    result$participants$weight,
    ref$participants$weight,
    tolerance = 1e-10
  )
})

test_that("weigh() with named vector works", {
  survey <- copy(polymod_grouped)
  if (!"sex" %in% colnames(survey$participants)) {
    skip("No sex column in polymod participants")
  }
  result <- weigh(survey, "sex", target = c(male = 0.49, female = 0.51))
  expect_true("weight" %in% colnames(result$participants))
  expect_true(all(result$participants$weight > 0, na.rm = TRUE))
})

test_that("weigh() direct numeric works", {
  survey <- copy(polymod_grouped)
  survey$participants[, test_wt := runif(.N, 0.5, 1.5)]
  result <- weigh(survey, "test_wt")
  expect_equal(
    result$participants$weight,
    survey$participants$test_wt,
    tolerance = 1e-10
  )
})

test_that("multiple weigh() calls accumulate", {
  result <- polymod_grouped |>
    weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
    weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6)))

  single <- weigh(
    polymod_grouped,
    "dayofweek",
    target = c(5, 2),
    groups = list(1:5, c(0, 6))
  )

  expect_equal(
    result$participants$weight,
    single$participants$weight^2,
    tolerance = 1e-10
  )
})

test_that("weigh() auto-creates weight column", {
  survey <- copy(polymod_grouped)
  if ("weight" %in% colnames(survey$participants)) {
    survey$participants[, weight := NULL]
  }
  result <- weigh(
    survey,
    "dayofweek",
    target = c(5, 2),
    groups = list(1:5, c(0, 6))
  )
  expect_true("weight" %in% colnames(result$participants))
})

test_that("weigh() errors for missing column", {
  expect_error(
    weigh(polymod_grouped, "nonexistent"),
    "not found"
  )
})

test_that("weigh() errors for non-numeric direct column", {
  expect_error(
    weigh(polymod_grouped, "country"),
    "must be numeric"
  )
})

test_that("weigh_named() warns for unmatched values", {
  survey <- copy(polymod_grouped)
  survey$participants[,
    test_col := sample(c("A", "B", "C"), .N, replace = TRUE)
  ]
  expect_warning(
    weigh(survey, "test_col", target = c(A = 1, B = 2)),
    "C"
  )
})

test_that("weigh_grouped() warns for non-NA unmatched values", {
  expect_warning(
    weigh(
      polymod_grouped,
      "dayofweek",
      target = c(5, 2),
      groups = list(1:3, 4:5)
    ),
    "do not match any group"
  )
})

test_that("weigh_grouped() does not warn when only NAs are unmatched", {
  expect_no_warning(
    weigh(
      polymod_grouped,
      "dayofweek",
      target = c(5, 2),
      groups = list(1:5, c(0, 6))
    )
  )
})

test_that("weigh() warns for empty groups", {
  expect_warning(
    weigh(
      polymod_grouped,
      "dayofweek",
      target = c(5, 2, 1),
      groups = list(1:5, c(0, 6), 99:100)
    ),
    "no matching participants"
  )
})

test_that("weigh() does not modify original", {
  original <- copy(polymod_grouped$participants)
  weigh(
    polymod_grouped,
    "dayofweek",
    target = c(5, 2),
    groups = list(1:5, c(0, 6))
  )
  expect_identical(polymod_grouped$participants, original)
})

# nolint start: nonportable_path_linter
test_that("weigh() errors for mismatched target/groups lengths", {
  # nolint end
  expect_error(
    weigh(
      polymod_grouped,
      "dayofweek",
      target = c(5, 2, 1),
      groups = list(1:5, c(0, 6))
    ),
    "same length"
  )
})

test_that("convert_factor_to_integer preserves numeric values", {
  dt <- data.table::data.table(
    age = factor(c(3, 4, 6, 3))
  )

  result <- convert_factor_to_integer(dt, cols = "age")

  # Should be 3, 4, 6, 3 - not 1, 2, 3, 1 (factor codes)
  expect_identical(result$age, c(3L, 4L, 6L, 3L))
})

test_that("convert_factor_to_integer handles non-sequential numeric levels", {
  dt <- data.table::data.table(
    age = factor(c(10, 20, 30, 10))
  )

  result <- convert_factor_to_integer(dt, cols = "age")

  expect_identical(result$age, c(10L, 20L, 30L, 10L))
})

test_that("convert_factor_to_integer returns NA for non-numeric levels", {
  dt <- data.table::data.table(
    age = factor(c("3", "4", "five", "6"))
  )

  expect_warning(
    convert_factor_to_integer(dt, cols = "age"),
    "Non-numeric factor levels"
  )

  # Non-numeric "five" should become NA, others preserved
  expect_identical(dt$age, c(3L, 4L, NA_integer_, 6L))
})

test_that("convert_factor_to_integer only converts specified columns", {
  dt <- data.table::data.table(
    age = factor(c(10, 20, 30)),
    group = factor(c("a", "b", "c"))
  )

  result <- convert_factor_to_integer(dt, cols = "age")

  expect_identical(result$age, c(10L, 20L, 30L))
  expect_s3_class(result$group, "factor") # group should remain a factor
})

test_that("convert_factor_to_integer ignores non-factor columns", {
  dt <- data.table::data.table(
    age = c(10L, 20L, 30L) # already integer
  )

  result <- convert_factor_to_integer(dt, cols = "age")

  expect_identical(result$age, c(10L, 20L, 30L))
})

#' @title Citation for a survey
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `get_citation()` has been deprecated in favour of
#'   `contactsurveys::get_citation()`.
#'
#' Gets a full citation for a [survey()].
#'
#' @param x a character vector of surveys to cite
#' @return citation as bibentry
#' @importFrom utils bibentry
#' @importFrom rlang %||%
#' @examples
#' # we recommend using the contactsurveys package for get_citation()
#' \dontrun{
#' data(polymod)
#' citation <- contactsurveys::get_citation(polymod)
#' print(citation)
#' print(citation, style = "bibtex")
#' }
#' @export
get_citation <- function(x) {
  lifecycle::deprecate_soft(
    when = "0.5.0",
    what = "get_citation()",
    with = "contactsurveys::get_citation()"
  )
  survey <- get_survey(x)
  if (is.null(survey$reference)) {
    cli::cli_abort("No citation defined for {survey$name %||% 'survey'}.")
  }

  ref <- c(
    list(
      header = gettextf(
        "To cite %s in publications use:",
        survey$reference$title
      )
    ),
    survey$reference
  )

  do.call(bibentry, ref)
}

#' Check that an age column (or its estimated fallbacks) exists
#'
#' @param df a data.frame to check
#' @param age_column the primary age column name
#' @param label a human-readable label for warning messages
#'   (e.g. "participant" or "contact")
#' @return `TRUE` if the column (or fallbacks) exist, `FALSE` otherwise
#' @noRd
check_age_column <- function(
  df,
  age_column,
  label,
  call = rlang::caller_env()
) {
  if (age_column %in% colnames(df)) {
    return(TRUE)
  }

  exact_col <- paste(age_column, "exact", sep = "_")
  min_col <- paste(age_column, "est_min", sep = "_")
  max_col <- paste(age_column, "est_max", sep = "_")

  if (
    (exact_col %in% colnames(df)) ||
      (min_col %in% colnames(df) && max_col %in% colnames(df))
  ) {
    return(TRUE)
  }

  cli::cli_warn(
    "{label} age column {.arg {age_column}} or columns to \\
    estimate {tolower(label)} age ({.arg {exact_col}} or \\
    {.arg {min_col}} and {.arg {max_col}}) do not exist in \\
    the {tolower(label)} data frame.",
    call = call
  )
  FALSE
}

#' @export
check <- function(x, ...) UseMethod("check")
#' @name check
#' @rdname check
#' @title Check contact survey data
#'
#' @description Checks that a survey fulfills all the requirements
#'   to work with the 'contact_matrix' function
#'
#' @param x A [survey()] object
#' @param id.column the column in both the `participants` and
#'   `contacts` data frames that links contacts to participants
#' @param participant.age.column the column in the `participants`
#'   data frame containing participants' age; if this does not
#'   exist, at least columns "..._exact", "..._est_min" and
#'   "..._est_max" must exist (see the
#'   `estimated.participant.age` option in [contact_matrix()])
#' @param country.column the column in the `participants` data
#'   frame containing the country in which the participant was
#'   queried
#' @param year.column the column in the `participants` data frame
#'   containing the year in which the participant was queried
#' @param contact.age.column the column in the `contacts` data
#'   frame containing the age of contacts; if this does not exist,
#'   at least columns "..._exact", "..._est_min" and
#'   "..._est_max" must exist (see the
#'   `estimated.contact.age` option in [contact_matrix()])
#' @param ... ignored
#' @return invisibly returns a character vector of the relevant columns
#' @examples
#' data(polymod)
#' check(polymod)
#' @export
check.contact_survey <- function(
  x,
  id.column = "part_id",
  participant.age.column = "part_age",
  country.column = "country",
  year.column = "year",
  contact.age.column = "cnt_age",
  ...
) {
  lifecycle::deprecate_warn(
    "1.0.0",
    "check()",
    details = paste(
      "Use `as_contact_survey()` instead to construct a `<contact_survey>`",
      "object. This will perform necessary checks."
    )
  )
  chkDots(...)
  if (!is.data.frame(x$participants) || !is.data.frame(x$contacts)) {
    cli::cli_abort(
      "The {.field participants} and {.field contacts} elements of \\
      {.arg x} must be data.frames."
    )
  }

  x <- clean(x)

  success <- TRUE
  if (
    !(id.column %in%
      colnames(x$participants) &&
      id.column %in% colnames(x$contacts))
  ) {
    cli::cli_warn(
      "{.arg id.column} {.val {id.column}} does not exist in both \\
      the participants and contacts data frames."
    )
    success <- FALSE
  }

  if (
    !check_age_column(
      x$participants,
      participant.age.column,
      "Participant"
    )
  ) {
    success <- FALSE
  }

  if (!check_age_column(x$contacts, contact.age.column, "Contact")) {
    success <- FALSE
  }

  if (!(country.column %in% colnames(x$participants))) {
    cli::cli_warn(
      "Country column {.arg {country.column}} does not exist in the \\
      participant data frame."
    )
    success <- FALSE
  }
  if (success) {
    cli::cli_alert("Check OK.")
  } else {
    cli::cli_alert("Check FAILED.")
  }

  invisible(c(
    id = id.column,
    participant.age = participant.age.column,
    country = country.column,
    year = year.column,
    contact.age = contact.age.column
  ))
}

library(data.table)

erroneous_survey <- as_contact_survey(polymod)

erroneous_type1 <- copy(erroneous_survey)
erroneous_type1$participants <- "test"
erroneous_type2 <- copy(erroneous_survey)
erroneous_type2$participants <- 17

test_that("error is thrown if survey contains false data types", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    check(erroneous_type1)
  )
  expect_error(check(erroneous_type1), "must be data.frames")
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    check(erroneous_type2)
  )
  expect_error(check(erroneous_type2), "must be data.frames")
})

erroneous_structure1 <- copy(erroneous_survey)
erroneous_structure1$participants$part_id <- NULL
erroneous_structure2 <- copy(erroneous_survey)
erroneous_structure2$participants$part_age_exact <- NULL
erroneous_structure3 <- copy(erroneous_survey)
erroneous_structure3$contacts$cnt_age_exact <- NULL
erroneous_structure3$contacts$cnt_age_est_min <- NULL

test_that("incorrect structure of data frames is correctly identified", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_warning(check(erroneous_structure1), "does not exist")
  expect_snapshot_warning(
    cran = FALSE,
    check(erroneous_structure1)
  )
  expect_warning(check(erroneous_structure2), "do not exist")
  expect_snapshot_warning(
    cran = FALSE,
    check(erroneous_structure2)
  )
  expect_warning(check(erroneous_structure3), "do not exist")
  expect_snapshot_warning(
    cran = FALSE,
    check(erroneous_structure3)
  )
})

#' Social contact data from 8 European countries
#'
#' A dataset containing social mixing diary data from 8 European countries:
#' Belgium, Germany, Finland, Great Britain, Italy, Luxembourg,
#' The Netherlands and Poland.
#' The Data are fully described in Mossong J, Hens N, Jit M, Beutels P, Auranen
#' K, Mikolajczyk R, et al. (2008) Social Contacts and Mixing Patterns Relevant
#' to the Spread of Infectious Diseases. PLoS Med 5(3): e74.
#'
#' @format A list of two data frames:
#' \describe{
#'   \item{participants}{the study participant, with age, country, year and day
#'   of the week (starting with 1 = Monday)}
#'   \item{contacts}{reported contacts of the study participants. The variable
#'   phys_contact has two levels (1 denotes physical contact while 2 denotes
#'   non-physical contact), duration_multi has five levels (1 is less than 5
#'   minutes while 5 is more than 4 hours, increasing in the order found in
#'   Figure 1 in Mossong et al.), and frequency_multi has five levels (1 is
#'   daily, 2 is weekly, 3 is monthly, 4 is less often, and 5 is first time)}
#'   All other variables are described on the Zenodo repository of the data,
#'   available at \doi{10.5281/zenodo.1043437}
#' }
#' @source \doi{10.1371/journal.pmed.0050074}
"polymod"

#' Handle deprecated argument
#'
#' @param old_arg the deprecated argument value
#' @param new_arg the new argument value
#' @param old_name the old argument name (with dot)
#' @param new_name the new argument name (with underscore)
#' @param fn_name the function name
#' @param version the version when deprecated
#' @return the value to use (new_arg if provided, otherwise old_arg)
#' @keywords internal
#' @autoglobal
deprecate_arg <- function(
  old_arg,
  new_arg,
  old_name,
  new_name,
  fn_name,
  version = "0.5.0"
) {
  if (lifecycle::is_present(old_arg)) {
    lifecycle::deprecate_warn(
      version,
      paste0(fn_name, "(", old_name, ")"),
      paste0(fn_name, "(", new_name, ")")
    )
    old_arg
  } else {
    new_arg
  }
}

#' Flexible country code conversion
#'
#' Automatically detects the country coding scheme and converts
#' to country names. If all input strings are 2 characters,
#' assumes ISO2C format; otherwise assumes country names.
#'
#' @param countries Character vector of country identifiers
#' @return Character vector of country names
#' @importFrom countrycode countrycode
#' @note internal
#' @noRd
flexible_countrycode <- function(countries) {
  if (is.null(countries) || length(countries) == 0) {
    return(character(0))
  }

  countries_in_iso2c <- all(countries %in% countrycode::codelist$iso2c)
  countries_are_iso2c <- all(nchar(countries) == 2) && countries_in_iso2c
  coding_scheme <- ifelse(
    test = countries_are_iso2c,
    yes = "iso2c",
    no = "country.name"
  )
  corrected_countries <- suppressWarnings(
    countrycode(
      sourcevar = countries,
      origin = coding_scheme,
      destination = "country.name"
    )
  )
  check_missing_countries(countries, corrected_countries)
  corrected_countries
}

#' Get a survey, either from its Zenodo repository, a set of files, or a survey variable
#'
#' @description
#'
#' `r lifecycle::badge("deprecated")`
#'
#' `get_survey()` has been deprecated in favour of using
#'   `contactsurveys::download_survey()` and then [load_survey()].
#'
#' Downloads survey data, or extracts them from files, and returns a clean data
#' set. If a survey URL is accessed multiple times, the data will be cached
#' (unless `clear_cache` is set to `TRUE`) to avoid repeated downloads.
#'
#' If survey objects are used repeatedly the downloaded files can be saved and
#' reloaded between sessions then survey objects can be saved/loaded using
#' [base::saveRDS()] and [base::readRDS()], or via the individual survey files
#' that can be downloaded using [download_survey()] and subsequently loaded
#' using [load_survey()].
#'
#' @param clear_cache logical, whether to clear the cache before downloading
#' the survey; by default, the cache is not cleared and so multiple calls of
#' this function to access the same survey will not result in repeated
#' downloads.
#' @importFrom memoise memoise
#' @inheritParams .get_survey
#' @examples
#' \dontrun{
#' list_surveys()
#' peru_doi <- "https://doi.org/10.5281/zenodo.1095664"
#' peru_survey <- get_survey(peru_doi)
#' ## --> We now recommend:
#' peru_survey <- contactsurveys::download_survey(peru_doi)
#' peru_data <- load_survey(peru_survey)
#' }
#' @return a survey in the correct format
#' @export
get_survey <- function(survey, clear_cache = FALSE, ...) {
  lifecycle::deprecate_soft(
    when = "0.5.0",
    what = "get_survey()",
    with = "contactsurveys::download_survey()",
    details = "We recommend using contactsurveys::download_survey() to \\
    download your surveys, and then you can load them with \\
    socialmixr::load_survey()."
  )
  if (
    !("get_survey" %in% names(.socialmixr.env$cached_functions)) ||
      clear_cache
  ) {
    .socialmixr.env$cached_functions$get_survey <- memoise(.get_survey)
  }
  .socialmixr.env$cached_functions$get_survey(survey, ...)
}

#' Internal function to get survey data
#' @autoglobal
#' @param survey a DOI or url to get the survey from, or a [survey()] object (in which case only cleaning is done).
#' @param ... options for [clean()], which is called at the end of this
#' @importFrom data.table copy
#' @keywords internal
.get_survey <- function(survey, ...) {
  if (inherits(survey, "contact_survey")) {
    new_survey <- copy(survey)
  } else if (is.character(survey)) {
    files <- download_survey(survey)
    new_survey <- load_survey(files)
  } else {
    cli::cli_abort(
      "{.arg survey} must be a {.cls contact_survey} object or character."
    )
  }

  new_survey
}

#' Draws an image plot of a contact matrix with a legend strip
#' and the numeric values in the cells.
#'
#' This function combines the R image.plot function with numeric
#' contact rates in the matrix cells.
#'
#' @param mij a contact matrix containing contact rates between
#'   participants of age i (rows) with contacts of age j
#'   (columns). This is the default matrix format of
#'   [contact_matrix()].
#' @param min.legend the color scale minimum (default = 0). Set
#'   to NA to use the minimum value of `mij`.
#' @param max.legend the color scale maximum (default = NA). Set
#'   to NA to use the maximum value of `mij`.
#' @param num.digits the number of digits when rounding the
#'   contact rates (default = 2). Use NA to disable this.
#' @param num.colors the number of color breaks (default = 50)
#' @param main the figure title
#' @param legend.width width of the legend strip in characters.
#'   Default is 1.
#' @param legend.mar width in characters of legend margin.
#'   Default is 5.1.
#' @param legend.shrink amount to shrink the size of legend
#'   relative to the full height or width of the plot. Default
#'   is 0.9.
#' @param cex.lab size of the x and y labels (default: 1.2)
#' @param cex.axis size of the axis labels (default: 0.8)
#' @param cex.text size of the numeric values in the matrix
#'   (default: 1)
#' @param xlab a title for the x axis (default: "Age group
#'   (year)")
#' @param ylab a title for the y axis (default: "Contact age
#'   group (year)")
#' @param color.palette the color palette to use (default:
#'   [heat.colors()]). Other examples are [topo.colors()],
#'   [terrain.colors()] and [hcl.colors()]. User-defined
#'   functions are also possible if they take the number of
#'   colors to be in the palette as function argument.
#' @importFrom grDevices heat.colors
#' @importFrom graphics axis text image par
#' @details This is a function using basic R graphics to
#'   visualise a social contact matrix.
#' @export
#' @examples
#' \dontrun{
#' data(polymod)
#' mij <- contact_matrix(
#'   polymod,
#'   countries = "United Kingdom",
#'   age_limits = c(0, 18, 65)
#' )$matrix
#' matrix_plot(mij)
#' }
#' @author Lander Willem
matrix_plot <- function(
  mij,
  min.legend = 0,
  max.legend = NA,
  num.digits = 2,
  num.colors = 50,
  main,
  xlab,
  ylab,
  legend.width,
  legend.mar,
  legend.shrink,
  cex.lab,
  cex.axis,
  cex.text,
  color.palette = heat.colors
) {
  # check function arguments
  xlab <- ifelse(!missing(xlab), xlab, "Age group (year)")
  ylab <- ifelse(!missing(ylab), ylab, "Contact age group (year)")
  main <- ifelse(!missing(main), main, "Contact rates")
  cex.lab <- ifelse(!missing(cex.lab), cex.lab, 1.2)
  cex.axis <- ifelse(!missing(cex.axis), cex.axis, 0.8)
  cex.text <- ifelse(!missing(cex.text), cex.text, 1)
  legend.width <- ifelse(!missing(legend.width), legend.width, 1)
  legend.mar <- ifelse(!missing(legend.mar), legend.mar, 5.1)
  legend.shrink <- ifelse(!missing(legend.shrink), legend.shrink, 0.9)

  # set colors
  redc <- rev(color.palette(num.colors))

  # set legend scale
  zlim <- range(pretty(mij), na.rm = TRUE)
  if (!is.na(min.legend)) zlim[1] <- min.legend
  if (!is.na(max.legend)) zlim[2] <- max.legend

  # set breaks and midpoints
  breaks <- seq(zlim[1], zlim[2], length = num.colors + 1)
  midpoints <- matrix(
    breaks[-length(breaks)] + diff(breaks) / 2,
    nrow = 1,
    ncol = length(breaks) - 1
  )

  # get plot region for matrix and legend based on current graphical parameters
  # note: based on layout from fields::imagePlot
  char.size <- par()$cin[1] / par()$din[1] # get text character size
  plot_offset <- char.size * par()$mar[4] # space between legend and main plot

  # set legends' plot region
  legend_plot_region <- par()$plt
  legend_plot_region[2] <- 1 - (legend.mar * char.size)
  legend_plot_region[1] <- legend_plot_region[2] - (legend.width * char.size)

  # account for legend.shrink
  pr <- (legend_plot_region[4] - legend_plot_region[3]) *
    ((1 - legend.shrink) / 2)
  legend_plot_region[4] <- legend_plot_region[4] - pr
  legend_plot_region[3] <- legend_plot_region[3] + pr

  # set main matrix' plot region
  main_plot_region <- par()$plt
  main_plot_region[2] <- min(
    main_plot_region[2],
    legend_plot_region[1] - plot_offset
  )

  # defensive check for main and legends' plot region
  dp <- legend_plot_region[2] - legend_plot_region[1]
  legend_plot_region[1] <- min(
    main_plot_region[2] + plot_offset,
    legend_plot_region[1]
  )
  legend_plot_region[2] <- legend_plot_region[1] + dp

  # store old graphical parameters, and ensure they are restored on exit
  # (including on error) so the function leaves no side effects on par().
  old.par <- par(no.readonly = TRUE)
  on.exit(par(old.par), add = TRUE)
  par(plt = main_plot_region)

  # add image plot
  image(
    mij,
    xlab = xlab,
    ylab = ylab,
    main = main,
    cex.lab = cex.lab,
    breaks = breaks,
    col = redc,
    xaxt = "n",
    yaxt = "n"
  )

  # add axis labels
  plt_ticks <- seq(0, 1, length = nrow(mij))
  axis(
    2,
    at = plt_ticks,
    labels = c(colnames(mij)),
    cex.axis = cex.axis,
    tick = FALSE,
    las = 1
  )
  axis(
    1,
    at = plt_ticks,
    labels = c(colnames(mij)),
    cex.axis = cex.axis,
    tick = FALSE
  )

  # add numeric values if num.digits != NA and cex.text > 0
  if (!is.na(num.digits) && !is.na(cex.text) && cex.text > 0) {
    # format results (rounding/scientific)
    if (any(max(mij, na.rm = TRUE) > 1)) {
      mij <- round(mij, digits = num.digits)
    } else {
      mij <- format(mij, digits = num.digits)
    }

    # get grid centers and add values
    e_grid <- expand.grid(plt_ticks, plt_ticks)
    text(e_grid, labels = mij, cex = cex.text)
  }

  # set graphical parameters for the legend
  par(new = TRUE, pty = "m", plt = legend_plot_region, err = -1)

  # include legend bar with axis
  image(
    x = 1:2,
    y = breaks,
    z = midpoints,
    xaxt = "n",
    yaxt = "n",
    xlab = "",
    ylab = "",
    col = redc,
    breaks = breaks
  )
  axis(side = 4, mgp = c(3, 1, 0), las = 2)
}

dta <- contact_matrix(
  polymod,
  countries = "United Kingdom",
  age_limits = c(0, 18, 65),
  per_capita = TRUE
)

test_that("contact matrix can be plotted", {
  pdf(file = NULL)
  expect_no_error(matrix_plot(dta$matrix))
  dev.off()
})

test_that("contact matrix per capita can be plotted", {
  pdf(file = NULL)
  expect_no_error(matrix_plot(dta$matrix.per.capita))
  dev.off()
})

test_that("contact matrix can be plotted with different color palette", {
  pdf(file = NULL)
  expect_no_error(matrix_plot(dta$matrix, color.palette = rainbow))
  dev.off()
})

test_that("contact matrix can be plotted with ad-hoc legend", {
  pdf(file = NULL)
  expect_no_error(matrix_plot(dta$matrix, min.legend = 4, max.legend = 40))
  dev.off()
})

test_that("contact matrix plotted with ad-hoc legend values", {
  pdf(file = NULL)
  expect_no_error(matrix_plot(dta$matrix, min.legend = 4, max.legend = 40))
  dev.off()
})

#' @title Check contact survey data
#'
#' @description Checks that a survey fulfills all the requirements
#'   to work with the 'contact_matrix' function
#'
#' @param x list containing
#'  - an element named 'participants', a data frame containing participant
#'   information
#'  - an element named 'contacts', a data frame containing contact information
#'  - (optionally) an element named 'reference, a list containing information
#'   information needed to reference the survey, in particular it can contain$a
#'   "title", "bibtype", "author", "doi", "publisher", "note", "year"
#' @param id_column the column in both the `participants` and
#'   `contacts` data frames that links contacts to participants
#' @param country_column the column in the `participants` data frame
#'   containing the country in which the participant was queried; if
#'   NULL (default), will use "country" column if present
#' @param year_column the column in the `participants` data frame
#'   containing the year in which the participant was queried; if
#'   NULL (default), will use "year" column if present
#' @param ... additional arguments (currently ignored)
#' @param id.column,country.column,year.column
#'   `r lifecycle::badge("deprecated")` Use the underscore versions
#'   (e.g., `id_column`) instead.
#' @importFrom checkmate assert_list assert_names assert_data_frame
#'   assert_character
#' @importFrom purrr walk
#' @return invisibly returns a character vector of the relevant columns
#' @examples
#' data(polymod)
#' check(polymod)
#' @export
as_contact_survey <- function(
  x,
  id_column = "part_id",
  country_column = NULL,
  year_column = NULL,
  ...,
  id.column = deprecated(),
  country.column = deprecated(),
  year.column = deprecated()
) {
  ## Handle deprecated arguments
  id_column <- deprecate_arg(
    id.column,
    id_column,
    "id.column",
    "id_column",
    "as_contact_survey"
  )
  country_column <- deprecate_arg(
    country.column,
    country_column,
    "country.column",
    "country_column",
    "as_contact_survey"
  )
  year_column <- deprecate_arg(
    year.column,
    year_column,
    "year.column",
    "year_column",
    "as_contact_survey"
  )
  ## check arguments
  assert_list(x, names = "named")
  assert_names(names(x), must.include = c("participants", "contacts"))
  assert_data_frame(x$participants)
  assert_data_frame(x$contacts)
  assert_list(x$reference, names = "named", null.ok = TRUE)
  assert_character(id_column)
  assert_character(year_column, null.ok = TRUE)
  assert_character(country_column, null.ok = TRUE)
  assert_names(colnames(x$participants), must.include = id_column)
  assert_names(colnames(x$contacts), must.include = id_column)

  setnames(x$participants, id_column, "part_id")
  setnames(x$contacts, id_column, "part_id")

  ## Auto-detect country/year columns if not specified
  ## If NULL and default column exists, use it; if NULL and doesn't exist, skip
  participant_cols <- colnames(x$participants)

  if (is.null(country_column) && "country" %in% participant_cols) {
    country_column <- "country"
  }
  if (is.null(year_column) && "year" %in% participant_cols) {
    year_column <- "year"
  }

  ## check optional columns exist if explicitly provided
  to_check <- list(
    country = country_column,
    year = year_column
  )

  walk(
    .x = names(to_check),
    .f = function(column) {
      if (
        !is.null(to_check[[column]]) &&
          !(to_check[[column]] %in% participant_cols)
      ) {
        cli::cli_abort(
          "{.arg {column}} column {.val {to_check[[column]]}} does not exist
        in the participant data frame."
        )
      } else if (!is.null(to_check[[column]])) {
        setnames(x$participants, to_check[[column]], column)
      }
    }
  )

  if (is.null(x$reference)) {
    cli::cli_warn("No reference provided.")
  }

  survey <- new_contact_survey(x$participants, x$contacts, x$reference)
  clean(survey)
}

#' Get survey country population data
#'
#' @description Looks up the country and year inside a survey, or a provided
#' "countries" value, and determines the corresponding demographics in the world
#' population prospects data using [wpp_age()].
#'
#' @param survey A [survey()] object, with column "country" in "participants".
#' @param countries Optional. A character vector of country names. If specified,
#'   this will be used instead of the potential "country" column in
#'   "participants".
#'
#' @returns A data table with population data by age group for the survey
#' countries, aggregated by lower age limit. The function will error if no
#' country information is available from either the survey or countries
#' argument.
#'
#' @importFrom rlang %||%
#' @autoglobal
#' @examples
#' survey_country_population(polymod)
#' survey_country_population(polymod, countries = "Belgium")
#' survey_country_population(polymod, countries = c("Belgium", "Italy"))
#' @export
survey_country_population <- function(survey, countries = NULL) {
  check_if_contact_survey(survey)
  participants <- survey$participants
  survey_country_name <- countries %||% unique(participants$country)
  survey_country_name <- as.character(stats::na.omit(survey_country_name))
  if (is.null(survey_country_name) || length(survey_country_name) == 0) {
    cli::cli_abort(
      message = c(
        "Country name must be provided in {.var survey} or {.var countries}",
        # nolint start
        "i" = "{.var survey}: {.code survey$participants$country} is: \\
      {.val NULL}",
        "i" = "{.var countries} is: {.val {countries}}"
        # nolint end
      )
    )
  }
  ## get population data for countries from 'wpp' package
  country_pop <- data.table(wpp_age(survey_country_name))

  country_pop$country <- normalise_country_names(country_pop$country)

  ## check if survey data are from a specific year - in that case
  ## use demographic data from that year, otherwise latest
  if ("year" %in% colnames(participants)) {
    survey_year <- participants[, median(year, na.rm = TRUE)]
  } else {
    survey_year <- country_pop[, max(year, na.rm = TRUE)]
    cli::cli_warn(
      "No information on year found in the data. Will use
            {survey_year} population data."
    )
  }

  ## check if any survey countries are not in wpp
  check_any_missing_countries(survey_country_name, country_pop)

  ## get demographic data closest to survey year
  country_pop_year <- unique(country_pop[, year])
  survey_year <- min(
    country_pop_year[which.min(abs(survey_year - country_pop_year))]
  )
  survey_pop <- country_pop[year == survey_year][,
    list(population = sum(population)),
    by = "lower.age.limit"
  ]
  survey_pop
}

test_that("survey_country_population() works", {
  expect_snapshot(
    as.data.frame(survey_country_population(polymod))
  )
  expect_snapshot(
    as.data.frame(survey_country_population(polymod, countries = "Belgium"))
  )
  expect_snapshot(
    as.data.frame(survey_country_population(
      polymod,
      countries = c("Belgium", "Italy")
    ))
  )
  expect_snapshot(
    as.data.frame(survey_country_population(polymod, countries = "Australia"))
  )
})

test_that("survey_country_population() errors appropriately", {
  # When no country information provided
  polymod_copy <- polymod
  polymod_copy$participants$country <- NULL
  expect_snapshot(
    error = TRUE,
    survey_country_population(polymod_copy)
  )
})

# Deprecated functions --------------------------------------------------------
# These functions have been deprecated in favour of the contactsurveys package.
# They will be removed in a future version of socialmixr.

#' Get a survey, either from its Zenodo repository, a set of
#' files, or a survey variable
#'
#' @description
#'
#' `r lifecycle::badge("deprecated")`
#'
#' `get_survey()` has been deprecated in favour of using
#'   `contactsurveys::download_survey()` and then [load_survey()].
#'
#' Downloads survey data, or extracts them from files, and returns a clean data
#' set. If a survey URL is accessed multiple times, the data will be cached
#' (unless `clear_cache` is set to `TRUE`) to avoid repeated downloads.
#'
#' If survey objects are used repeatedly the downloaded files can be saved and
#' reloaded between sessions then survey objects can be saved/loaded using
#' [base::saveRDS()] and [base::readRDS()], or via the individual survey files
#' that can be downloaded using [download_survey()] and subsequently loaded
#' using [load_survey()].
#'
#' @param clear_cache logical, whether to clear the cache before downloading
#' the survey; by default, the cache is not cleared and so multiple calls of
#' this function to access the same survey will not result in repeated
#' downloads.
#' @importFrom memoise memoise
#' @inheritParams .get_survey
#' @examples
#' \dontrun{
#' list_surveys()
#' peru_doi <- "https://doi.org/10.5281/zenodo.1095664"
#' peru_survey <- get_survey(peru_doi)
#' ## --> We now recommend:
#' peru_survey <- contactsurveys::download_survey(peru_doi)
#' peru_data <- load_survey(peru_survey)
#' }
#' @return a survey in the correct format
#' @export
get_survey <- function(survey, clear_cache = FALSE, ...) {
  lifecycle::deprecate_warn(
    when = "0.5.0",
    what = "get_survey()",
    with = "contactsurveys::download_survey()",
    details = "We recommend using contactsurveys::download_survey() to \\
    download your surveys, and then you can load them with \\
    socialmixr::load_survey()."
  )
  if (
    !("get_survey" %in% names(.socialmixr.env$cached_functions)) ||
      clear_cache
  ) {
    .socialmixr.env$cached_functions$get_survey <- memoise(.get_survey)
  }
  .socialmixr.env$cached_functions$get_survey(survey, ...)
}

#' Internal function to get survey data
#' @autoglobal
#' @param survey a DOI or url to get the survey from, or a [survey()] object.
#' @param ... currently unused
#' @keywords internal
.get_survey <- function(survey, ...) {
  if (inherits(survey, "contact_survey")) {
    new_survey <- copy_survey(survey)
  } else if (is.character(survey)) {
    files <- withr::with_options(
      list(lifecycle_verbosity = "quiet"),
      download_survey(survey)
    )
    new_survey <- load_survey(files)
  } else {
    cli::cli_abort(
      "{.arg survey} must be a {.cls contact_survey} object or character."
    )
  }

  new_survey
}

#' Download a survey from its Zenodo repository
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `download_survey()` has been deprecated in favour of
#'   `contactsurveys::download_survey()`.
#'
#' `download_survey()` downloads survey data from Zenodo.
#'
#' @param survey a URL (see `contactsurveys::list_surveys()`)
#' @param dir a directory to save the files to; if not given, will save to a
#'   temporary directory
#' @param sleep time to sleep between requests to avoid overloading the server
#'   (passed on to \code{\link[base]{Sys.sleep}})
#' @importFrom httr GET content status_code http_error config user_agent
#' @importFrom jsonlite fromJSON toJSON
#' @importFrom curl curl_download
#' @importFrom utils read.csv packageVersion
#' @importFrom xml2 xml_text xml_find_first xml_find_all xml_attr
#' @autoglobal
#' @examples
#' # we recommend using the contactsurveys package for download_survey()
#' \dontrun{
#' # if needed, discover surveys with:
#' contactsurveys::list_surveys()
#' peru_survey <- download_survey("https://doi.org/10.5281/zenodo.1095664")
#' # -->
#' peru_survey <- contactsurveys::download_survey(
#'   "https://doi.org/10.5281/zenodo.1095664"
#' )
#' }
#' @return a vector of filenames that can be used with [load_survey]
#' @seealso load_survey
#' @export
# nolint start: cyclocomp_linter.
download_survey <- function(survey, dir = NULL, sleep = 1) {
  lifecycle::deprecate_warn(
    when = "0.5.0",
    what = "download_survey()",
    with = "contactsurveys::download_survey()"
  )
  if (!is.character(survey) || length(survey) != 1 || is.na(survey)) {
    cli::cli_abort("{.arg survey} must be a single, non-NA character string.")
  }

  survey <- sub("^(https?:\\/\\/(dx\\.)?doi\\.org\\/|doi:)", "", survey)
  survey <- sub("#.*$", "", survey)
  is.doi <- is_doi(survey)
  is.url <- is.doi || grepl("^https?:\\/\\/", survey)

  if (!is.url) {
    cli::cli_abort("{.arg survey} is not a DOI or URL.")
  }

  if (is.doi) {
    survey_url <- paste0("https://doi.org/", survey)
  } else {
    survey_url <- survey
  }

  temp_body <- GET(
    survey_url,
    config = config(
      followlocation = 1
    ),
    user_agent(paste0(
      "http://github.com/epiforecasts/socialmixr R package socialmixr/v.",
      packageVersion("socialmixr")
    ))
  )
  if (status_code(temp_body) == 404) {
    cli::cli_abort("DOI {.val {survey}} not found.")
  }
  if (http_error(temp_body)) {
    cli::cli_abort(
      c(
        "Failed to fetch the requested resource",
        "x" = "The website server returned an HTTP error", # nolint
        "i" = "Check your connection or the server status" # nolint
      )
    )
  }

  parsed_body <- content(temp_body, encoding = "UTF-8")
  parsed_cite <- fromJSON(
    xml_text(
      xml_find_first(parsed_body, '//script[@type="application/ld+json"]')
    )
  )

  reference <- list(
    title = parsed_cite$name,
    bibtype = "Misc",
    author = parsed_cite$author$name,
    year = data.table::year(parsed_cite$datePublished)
  )
  if ("version" %in% names(parsed_cite)) {
    reference[["note"]] <- paste("Version", parsed_cite$version)
  }
  reference[[ifelse(is.doi, "doi", "url")]] <- survey

  links <- xml_attr(
    xml_find_all(
      parsed_body,
      "//link[@type=\"text/csv\" and not(@rel=\"alternate\")]"
    ),
    "href"
  )

  zenodo_links <- data.table(url = links)
  ## only download csv files
  zenodo_links[, file_name := tolower(basename(url))]

  if (anyDuplicated(zenodo_links$file_name) > 0) {
    cli::cli_warn(
      c(
        "Zenodo repository contains files with names that only differ by case.",
        "!" = "This will cause unpredictable behaviour on case-insensitive \\
        file systems.",
        "i" = "Please contact the authors to get this fixed." # nolint
      )
    )
    zenodo_links <- zenodo_links[!duplicated(file_name)]
  }

  if (is.null(dir)) {
    dir <- tempdir()
  }

  cli::cli_inform("Getting {parsed_cite$name}.")

  lcs <- find_common_prefix(zenodo_links$file_name)
  reference_file_path <- file.path(dir, paste0(lcs, "reference.json"))
  reference_json <- toJSON(reference)
  write(reference_json, reference_file_path)

  files <- c(
    reference_file_path,
    vapply(
      seq_len(nrow(zenodo_links)),
      function(i) {
        zenodo_url <- zenodo_links[i, ]$url
        temp <- file.path(dir, zenodo_links[i, ]$file_name)
        cli::cli_inform("Downloading {.url {zenodo_url}}")
        Sys.sleep(sleep)
        curl_download(zenodo_url, temp)
        temp
      },
      ""
    )
  )

  files
}
# nolint end

#' List all surveys available for download
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `list_surveys()` has been deprecated in favour of
#'   `contactsurveys::list_surveys()`.
#'
#' @return character vector of surveys
#' @inheritParams get_survey
#' @examples
#' # we recommend using the contactsurveys package now for listing surveys.
#' \dontrun{
#' contactsurveys::list_surveys()
#' }
#' @export
list_surveys <- function(clear_cache = FALSE) {
  lifecycle::deprecate_warn(
    when = "0.5.0",
    what = "list_surveys()",
    with = "contactsurveys::list_surveys()"
  )

  if (
    !("list_surveys" %in% names(.socialmixr.env$cached_functions)) ||
      clear_cache
  ) {
    .socialmixr.env$cached_functions$list_surveys <- memoise(.list_surveys)
  }

  .socialmixr.env$cached_functions$list_surveys()
}

#' @autoglobal
#' @importFrom oai list_records
#' @keywords internal
.list_surveys <- function() {
  record_list <- tryCatch(
    data.table(list_records(
      "https://zenodo.org/oai2d",
      metadataPrefix = "oai_datacite",
      set = "user-social_contact_data"
    )),
    error = function(e) {
      cli::cli_abort(
        message = c(
          "Failed to retrieve survey list from Zenodo OAI-PMH.",
          "Please retry later",
          "Original error: {conditionMessage(e)}"
        )
      )
    }
  )
  ## find common DOI for different versions, i.e. the "relation" that is a DOI
  relations <- grep("^relation(\\.|$)", colnames(record_list), value = TRUE)
  DOIs <- apply(
    record_list,
    1,
    function(x) {
      matches <- grep("^https://doi.org/.*zenodo", x[relations], value = TRUE)
      if (length(matches) == 0) NA_character_ else matches[1]
    }
  )
  record_list <- record_list[, common_doi := DOIs]
  record_list <- record_list[,
    url := sub("doi:", "https://doi.org/", common_doi, fixed = TRUE)
  ]
  ## get number within version DOI, this is expected to be ascending by version
  record_list <-
    record_list[,
      doi.nb := as.integer(sub("^.*zenodo\\.org:", "", identifier.1))
    ]
  ## save date at which first entered
  record_list <- record_list[, date := min(date), by = common_doi]
  ## order by DOI number and extract newest version
  record_list <- record_list[order(-doi.nb)]
  record_list <- record_list[, .SD[1], by = common_doi]
  ## order by date
  setkey(record_list, date)
  record_list[, list(
    date_added = date,
    title,
    creator,
    url = identifier.2
  )]
}

#' List all countries contained in a survey
#'
#' `r lifecycle::badge("deprecated")`
#'
#' `survey_countries()` has been deprecated in favour of using
#'   `contactsurveys::download_survey()`, and [load_survey()], and then
#'   exploring the country column yourself.
#'
#' @param country.column column in the survey indicating the country
#' @param ... further arguments for [get_survey()]
#' @return list of countries
#' @inheritParams get_survey
#' @examples
#' data(polymod)
#' survey_countries(polymod)
#' ## --> we now recommend
#' \dontrun{
#' doi_peru <- "10.5281/zenodo.1095664" # nolint
#' # download the data with the contactsurveys package
#' peru_survey <- contactsurveys::download_survey(doi_peru)
#' # load the survey with socialmixr
#' peru_data <- socialmixr::load_survey(peru_survey)
#' # find the unique country - assuming your data has a "country" column:
#' unique(peru_data$participants$country)
#' }
#' @export
survey_countries <- function(survey, country.column = "country", ...) {
  lifecycle::deprecate_warn(
    when = "0.5.0",
    what = "survey_countries()",
    with = "contactsurveys::download_survey()",
    details = "We recommend using contactsurveys::download_survey() to \\
    download your surveys, and then you can load them with \\
    socialmixr::load_survey() and explore which countries are in the data."
  )
  survey <- withr::with_options(
    list(lifecycle_verbosity = "quiet"),
    get_survey(survey, ...)
  )
  as.character(unique(survey[["participants"]][[country.column]]))
}

#' @title Citation for a survey
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `get_citation()` has been deprecated in favour of
#'   `contactsurveys::get_citation()`.
#'
#' Gets a full citation for a [survey()].
#'
#' @param x a character vector of surveys to cite
#' @return citation as bibentry
#' @importFrom utils bibentry
#' @importFrom rlang %||%
#' @examples
#' # we recommend using the contactsurveys package for get_citation()
#' \dontrun{
#' data(polymod)
#' citation <- contactsurveys::get_citation(polymod)
#' print(citation)
#' print(citation, style = "bibtex")
#' }
#' @export
get_citation <- function(x) {
  lifecycle::deprecate_warn(
    when = "0.5.0",
    what = "get_citation()",
    with = "contactsurveys::get_citation()"
  )
  survey <- withr::with_options(
    list(lifecycle_verbosity = "quiet"),
    get_survey(x)
  )
  if (is.null(survey$reference)) {
    cli::cli_abort("No citation defined for {survey$name %||% 'survey'}.")
  }

  ref <- c(
    list(
      header = gettextf(
        "To cite %s in publications use:",
        survey$reference$title
      )
    ),
    survey$reference
  )

  do.call(bibentry, ref)
}

# Internal helpers for deprecated functions -----------------------------------

find_common_prefix <- function(vec) {
  if (length(vec) == 0) {
    return("")
  }
  if (length(vec) == 1) {
    return("")
  }
  min_len <- min(nchar(vec))
  # find initial longest common prefix of file names
  i <- 1
  finish <- FALSE
  lcs <- ""
  while (!finish && i <= min_len) {
    initial_bits <- vapply(vec, substr, start = 1, stop = i, "x")
    if (length(unique(initial_bits)) > 1) {
      finish <- TRUE
    } else {
      lcs <- initial_bits[[1]]
      i <- i + 1
    }
  }

  lcs
}

##' Checks if a character string is a DOI
##'
##' @param x Character vector; the string or strings to check
##' @return Logical; \code{TRUE} if \code{x} is a DOI, \code{FALSE} otherwise
##' @author Sebastian Funk
is_doi <- function(x) {
  is.character(x) && grepl("^10\\.[0-9.]{4,}/[-._;()/:A-z0-9]+$", x)
}

test_that("get_citation() gives deprecation warning", {
  lifecycle::expect_deprecated(
    get_citation(polymod)
  )
})

test_that("surveys can be cited", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_s3_class(get_citation(polymod), "bibentry")
})

test_that("missing surveys can't be cited", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_error(get_citation("bogus"), "URL")
})

test_that("multiple DOIs cannot be loaded", {
  withr::local_options(lifecycle_verbosity = "quiet")
  expect_error(get_survey(c(
    "10.5281/zenodo.1095664", # nolint
    "10.5281/zenodo.1127693" # nolint
  )))
})

#' Compute contact matrix from prepared survey data
#'
#' @description
#' Computes a contact matrix from a `contact_survey` that has been processed
#' by [assign_age_groups()] and optionally [weigh()]. This is the final step
#' in the pipeline workflow.
#'
#' For post-processing, pipe the result into [symmetrise()],
#' [split_matrix()], or [per_capita()].
#'
#' @param survey a [survey()] object with age groups assigned (via
#'   [assign_age_groups()])
#' @param counts whether to return counts instead of means
#' @param weight_threshold numeric; if provided, weights above this threshold
#'   are capped to the threshold value and then re-normalised (default NULL)
#' @returns a list with elements `matrix` and `participants`
#'
#' @examples
#' data(polymod)
#' polymod |>
#'   assign_age_groups(age_limits = c(0, 5, 15)) |>
#'   compute_matrix()
#'
#' @importFrom data.table uniqueN
#' @export
#' @autoglobal
compute_matrix <- function(
  survey,
  counts = FALSE,
  weight_threshold = NULL
) {
  check_if_contact_survey(survey)
  survey <- copy_survey(survey)

  ## Warn if survey has multiple observations per participant ------------------
  warn_multiple_observations(
    participants = survey$participants,
    observation_key = survey$observation_key,
    filter_hint = "pipeline"
  )

  if (!"age.group" %in% colnames(survey$participants)) {
    cli::cli_abort(
      "Column {.val age.group} not found in participant data. \\
       Call {.fn assign_age_groups} first."
    )
  }

  if (!"contact.age.group" %in% colnames(survey$contacts)) {
    cli::cli_abort(
      "Column {.val contact.age.group} not found in contact data. \\
       Call {.fn assign_age_groups} first."
    )
  }

  ## Initialise weight if not present ------------------------------------------
  if (!"weight" %in% colnames(survey$participants)) {
    survey$participants[, weight := 1]
  }

  ## Post-stratification normalisation (with optional threshold) ---------------
  normalise_weights(survey$participants, threshold = weight_threshold)

  ## Merge participants and contacts -------------------------------------------
  survey$contacts <- merge_participants_contacts(
    participants = survey$participants,
    contacts = survey$contacts
  )

  ## Build weighted matrix (no bootstrap — use all participants) ---------------
  survey$contacts[, sampled.weight := weight]
  survey$participants[, sampled.weight := weight]

  weighted_matrix <- weighted_matrix_array(
    contacts = survey$contacts
  )

  if (!counts) {
    weighted_matrix <- normalise_weights_to_counts(
      sampled_participants = survey$participants,
      weighted_matrix = weighted_matrix
    )
  }

  part_pop <- n_participants_per_age_group(survey$participants)

  new_contact_matrix(
    matrix = weighted_matrix,
    participants = part_pop[]
  )
}

polymod_uk_grouped <- polymod |>
  (\(s) s[country == "United Kingdom"])() |>
  assign_age_groups(age_limits = c(0, 5, 15))

test_that("compute_matrix() produces a basic contact matrix", {
  result <- compute_matrix(polymod_uk_grouped)
  expect_true("matrix" %in% names(result))
  expect_true("participants" %in% names(result))
  expect_true(is.matrix(result$matrix) || is.array(result$matrix))
  expect_identical(nrow(result$matrix), 3L)
  expect_identical(ncol(result$matrix), 3L)
})

test_that("compute_matrix() counts works", {
  result <- compute_matrix(polymod_uk_grouped, counts = TRUE)
  result_mean <- compute_matrix(polymod_uk_grouped, counts = FALSE)
  expect_true(all(
    result$matrix >= result_mean$matrix,
    na.rm = TRUE
  ))
})

test_that("compute_matrix() errors when age.group missing", {
  survey <- copy(polymod)
  survey <- survey[country == "United Kingdom"]
  expect_error(
    compute_matrix(survey),
    "assign_age_groups"
  )
})

test_that("compute_matrix() warns for multiple observations per participant", {
  survey <- polymod_uk_grouped
  # Duplicate participants to simulate longitudinal data
  survey$participants <- rbind(survey$participants, survey$participants)
  expect_warning(
    compute_matrix(survey),
    "multiple observations"
  )
})

test_that("compute_matrix() warning mentions observation_key when present", {
  survey <- polymod_uk_grouped
  survey$participants <- rbind(survey$participants, survey$participants)
  survey$observation_key <- "wave"
  expect_warning(
    compute_matrix(survey),
    "wave"
  )
})

test_that("pipeline matches contact_matrix() without weighting", {
  result_pipe <- polymod |>
    (\(s) s[country == "United Kingdom"])() |>
    assign_age_groups(age_limits = c(0, 5, 15)) |>
    compute_matrix()

  result_legacy <- contact_matrix(
    polymod,
    countries = "United Kingdom",
    age_limits = c(0, 5, 15),
    symmetric = FALSE
  )

  expect_identical(result_pipe$matrix, result_legacy$matrix)
})

test_that("pipeline with dayofweek weighting is close to contact_matrix()", {
  result_pipe <- polymod |>
    (\(s) s[country == "United Kingdom"])() |>
    assign_age_groups(age_limits = c(0, 5, 15)) |>
    weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
    compute_matrix()

  result_legacy <- contact_matrix(
    polymod,
    countries = "United Kingdom",
    age_limits = c(0, 5, 15),
    weigh_dayofweek = TRUE,
    symmetric = FALSE
  )

  ## Small difference remains because legacy lumps NA dayofweek with weekends,

  ## while weigh() assigns them a neutral average weight.
  expect_equal(
    result_pipe$matrix,
    result_legacy$matrix,
    tolerance = 0.03
  )
})

.socialmixr.env <- new.env(parent = emptyenv())
assign("cached_functions", list(), envir = .socialmixr.env)
## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom lifecycle deprecate_soft
## usethis namespace: end
NULL

#' Weigh survey participants
#'
#' @description
#' Applies weights to participants in a `contact_survey` object. Weights are
#' always multiplied into an existing `weight` column (or one is created with
#' value 1), making multiple calls composable.
#'
#' The behaviour depends on the combination of arguments:
#' \describe{
#'   \item{`target = NULL`}{Numeric column: multiply `weight` by column values
#'     directly.}
#'   \item{Unnamed `target` + `groups`}{Map column values to groups, assign
#'     `target[g] / n_in_group` per participant.}
#'   \item{Named `target`}{Names match column values, assign
#'     `target[val] / n_with_val` per participant.}
#'   \item{Data frame `target`}{Post-stratify against population data (expanded
#'     to single-year ages via [pop_age()]).}
#' }
#'
#' @param survey a [survey()] object (must have been processed by
#'   [assign_age_groups()] if using data frame target)
#' @param by column name in the participant data to weigh by
#' @param target target weights: `NULL` for direct numeric weighting, an
#'   unnamed numeric vector (with `groups`), a named numeric vector, or a
#'   data frame with columns `lower.age.limit` and `population`
#' @param groups a list of value sets mapping column values to groups (used
#'   with unnamed `target` vector); must be the same length as `target`
#' @param ... further arguments passed to [pop_age()] when `target` is a data
#'   frame
#' @returns the survey object with updated participant weights
#'
#' @examples
#' data(polymod)
#' # Direct numeric weighting
#' if ("survey_weight" %in% names(polymod$participants)) {
#'   polymod |> weigh("survey_weight")
#' }
#'
#' # Dayofweek weighting with groups (POLYMOD uses 0 = Sunday, 6 = Saturday)
#' polymod |>
#'   weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6)))
#'
#' @export
#' @autoglobal
weigh <- function(survey, by, target = NULL, groups = NULL, ...) {
  check_if_contact_survey(survey)
  survey <- copy_survey(survey)
  participants <- survey$participants

  if (!by %in% colnames(participants)) {
    cli::cli_abort(
      "Column {.val {by}} not found in participant data."
    )
  }

  if (!"weight" %in% colnames(participants)) {
    participants[, weight := 1]
  }

  if (is.null(target) && is.null(groups)) {
    participants <- weigh_direct(participants, by)
  } else if (is.data.frame(target)) {
    participants <- weigh_population(participants, target, ...)
  } else if (!is.null(names(target))) {
    if (!is.null(groups)) {
      cli::cli_warn(
        "{.arg groups} is ignored when {.arg target} is a named vector."
      )
    }
    participants <- weigh_named(participants, by, target)
  } else if (!is.null(groups)) {
    participants <- weigh_grouped(participants, by, target, groups)
  } else {
    cli::cli_abort(
      "Cannot determine weighting method. Provide {.arg groups} with an \\
       unnamed {.arg target}, use a named {.arg target}, or pass a data frame."
    )
  }

  survey$participants <- participants
  survey
}

#' @autoglobal
weigh_direct <- function(participants, by) {
  if (!is.numeric(participants[[by]])) {
    cli::cli_abort(
      "Column {.val {by}} must be numeric for direct weighting \\
       (without {.arg target}). Got {.cls {class(participants[[by]])}}."
    )
  }
  participants[, weight := weight * get(by)]
  participants
}

#' @autoglobal
weigh_grouped <- function(participants, by, target, groups) {
  if (length(target) != length(groups)) {
    cli::cli_abort(
      "{.arg target} (length {length(target)}) and {.arg groups} \\
       (length {length(groups)}) must have the same length."
    )
  }

  col_vals <- participants[[by]]
  group_idx <- rep(NA_integer_, length(col_vals))
  for (g in seq_along(groups)) {
    group_idx[col_vals %in% groups[[g]]] <- g
  }

  group_counts <- tabulate(group_idx, nbins = length(groups))
  empty <- which(group_counts == 0L)
  if (length(empty) > 0L) {
    cli::cli_warn(
      "Group{?s} {.val {empty}} ha{?s/ve} no matching participants; \\
       their target weights will be ignored."
    )
    target[empty] <- 0
    group_counts[empty] <- 1L
  }
  unmatched_non_na <- is.na(group_idx) & !is.na(col_vals)
  n_unmatched <- sum(unmatched_non_na)
  if (n_unmatched > 0) {
    cli::cli_warn(
      "{n_unmatched} participant{?s} ha{?s/ve} values in {.val {by}} \\
       that do not match any group; an average weight will be used."
    )
  }

  n_total <- length(col_vals)
  weight_factor <- ifelse(
    is.na(group_idx),
    sum(target) / n_total,
    target[group_idx] / group_counts[group_idx]
  )

  participants[, weight := weight * weight_factor]
  participants
}

#' @autoglobal
weigh_named <- function(participants, by, target) {
  col_vals <- as.character(participants[[by]])
  val_counts <- table(col_vals)
  matched_target <- target[col_vals]
  matched_counts <- as.numeric(val_counts[col_vals])

  unmatched <- setdiff(unique(col_vals[!is.na(col_vals)]), names(target))
  n_unmatched <- length(unmatched)
  if (n_unmatched > 0) {
    cli::cli_warn(
      "{n_unmatched} value{?s} in column {.val {by}} not found in \\
       {.arg target} names ({.val {unmatched}}); \\
       {?its/their} weight{?s} will be set to {.val NA}."
    )
  }

  weight_factor <- ifelse(
    is.na(matched_target),
    NA_real_,
    matched_target / matched_counts
  )

  participants[, weight := weight * weight_factor]
  participants
}

#' @autoglobal
weigh_population <- function(participants, target, ...) {
  if (!all(c("lower.age.limit", "population") %in% colnames(target))) {
    cli::cli_abort(
      "Data frame {.arg target} must have columns {.val lower.age.limit} \\
       and {.val population}."
    )
  }

  if (!"part_age" %in% colnames(participants)) {
    cli::cli_abort(
      "Column {.val part_age} not found in participant data. \\
       Run {.fn assign_age_groups} first."
    )
  }

  survey_pop_full <- data.table(target)
  if (!"upper.age.limit" %in% colnames(survey_pop_full)) {
    age_breaks <- agegroups_to_limits(participants$age.group)
    survey_pop_full <- add_survey_upper_age_limit(
      survey = survey_pop_full,
      age_breaks = age_breaks
    )
  }
  survey_pop_full <- survey_pop_reference(survey_pop_full, ...)

  participants <- weight_by_age(participants, survey_pop_full)
  participants
}

#' Search for a pair of columns that, with a base ID, form a unique key
#'
#' @param data a data.table
#' @param base_id the base identifier column name
#' @param candidates character vector of candidate column names
#' @param n_rows the number of rows in `data`
#' @return a character vector of the key columns, or `NULL` if no pair works
#' @noRd
find_pair_key <- function(data, base_id, candidates, n_rows) {
  for (i in seq_along(candidates)) {
    for (j in seq_len(i - 1)) {
      cols <- c(base_id, candidates[j], candidates[i])
      if (uniqueN(data, by = cols) == n_rows) {
        return(cols)
      }
    }
  }
  NULL
}

#' Find the minimal unique key for a data.table
#'
#' Given a data.table and a base identifier column, finds the minimal set of
#' additional columns needed to uniquely identify each row.
#'
#' @param data A data.table
#' @param base_id The base identifier column name (default: "part_id")
#' @return A character vector of column names that form the unique key
#' @autoglobal
#' @keywords internal
find_unique_key <- function(data, base_id = "part_id") {
  n_rows <- nrow(data)

  # Already unique?
  if (uniqueN(data[[base_id]]) == n_rows) {
    return(base_id)
  }

  # All other columns, sorted by number of unique values (ascending)
  # Prefer columns with fewer values - makes for simpler/more meaningful keys
  candidates <- setdiff(names(data), base_id)
  # Exclude internal tracking columns
  candidates <- candidates[!startsWith(candidates, "..")]
  candidates <- candidates[order(vapply(
    candidates,
    function(x) uniqueN(data[[x]]),
    integer(1)
  ))]

  # Try single columns first
  for (col in candidates) {
    if (uniqueN(data, by = c(base_id, col)) == n_rows) {
      return(c(base_id, col))
    }
  }

  # Try pairs
  find_pair_key(data, base_id, candidates, n_rows)
}

#' @autoglobal
extract_reference <- function(files) {
  reference_files <- grep("\\.json$", files, value = TRUE, ignore.case = TRUE)
  if (length(reference_files) == 0) {
    return(NULL)
  }
  selected <- reference_files[[1]]
  if (length(reference_files) > 1) {
    cli::cli_warn(
      "Multiple JSON files found; using {.file {basename(selected)}}."
    )
  }
  fromJSON(selected)
}

#' @autoglobal
extract_type_common_csv <- function(
  type,
  survey_files,
  call = rlang::caller_env()
) {
  main_file <- grep(
    pattern = paste0("_", type, "s?_common.*\\.csv$"),
    x = survey_files,
    value = TRUE
  )
  if (length(main_file) == 0) {
    cli::cli_abort(
      message = "Need a CSV matching _{type}s?_common*.csv, but none found.",
      call = call
    )
  }
  main_file
}

#' @autoglobal
#' @importFrom data.table rbindlist
join_compatible_files <- function(survey_files, contact_data) {
  ## join files that can be joined
  for (file1 in survey_files) {
    if (!is.null(contact_data[[file1]])) {
      for (file2 in setdiff(survey_files, file1)) {
        if (
          setequal(
            colnames(contact_data[[file1]]),
            colnames(contact_data[[file2]])
          )
        ) {
          contact_data[[file1]] <- rbindlist(
            list(contact_data[[file1]], contact_data[[file2]]),
            fill = TRUE
          )
          contact_data[[file2]] <- NULL
          survey_files <- setdiff(survey_files, file2)
        }
      }
    }
  }
  list(
    contact_data = contact_data,
    survey_files = survey_files
  )
}

#' Identify which survey files share columns with a main table
#' @noRd
get_mergeable_files <- function(survey_files, contact_data, main_cols) {
  can_merge <- vapply(
    survey_files,
    function(x) {
      any(colnames(contact_data[[x]]) %in% main_cols)
    },
    logical(1)
  )
  names(can_merge[can_merge])
}

#' Resolve the unique key for a merged data.table with duplicates
#'
#' Validates a user-provided participant_key or auto-detects one via
#' find_unique_key().
#' @noRd
resolve_longitudinal_key <- function(
  merged,
  participant_key = NULL,
  call = rlang::caller_env()
) {
  if (!is.null(participant_key)) {
    missing_cols <- setdiff(participant_key, names(merged))
    if (
      length(missing_cols) == 0 &&
        anyDuplicated(merged, by = participant_key) == 0L
    ) {
      return(participant_key)
    }
    if (length(missing_cols) > 0) {
      cli::cli_warn(
        "Provided {.arg participant_key} contains column{?s} not found in \\
        merged data: {.val {missing_cols}}; auto-detecting a key instead.",
        call = call
      )
    } else {
      cli::cli_warn(
        "Provided {.arg participant_key} {.val {participant_key}} did not \\
        uniquely identify rows; auto-detecting a key instead.",
        call = call
      )
    }
  }
  find_unique_key(merged, "part_id")
}

#' Try merging a single additional file into a main survey table
#'
#' @return A list with components: merged (data.table or NULL) and detected_key
#'   (character vector or NULL).
#' @autoglobal
#' @noRd
try_merge_one_file <- function(
  file,
  type,
  main_survey,
  contact_data,
  participant_key = NULL,
  call = rlang::caller_env()
) {
  null_result <- list(merged = NULL, detected_key = NULL)

  contact_data[[file]] <- contact_data[[file]][,
    ("..merge_id") := seq_len(.N)
  ]
  common_id <- intersect(
    colnames(contact_data[[file]]),
    colnames(main_survey)
  )
  merged <- tryCatch(
    {
      merge(
        main_survey,
        contact_data[[file]],
        by = common_id,
        all.x = TRUE
      )
    },
    error = function(cond) {
      if (!grepl("cartesian", cond$message, fixed = TRUE)) {
        cli::cli_abort(
          "Merge failed for {.file {basename(file)}} on \\
          {.val {common_id}}: {cond$message}",
          call = call
        )
      }
      NULL
    }
  )

  if (is.null(merged)) {
    return(null_result)
  }

  has_duplicates <- anyDuplicated(merged[, "..main_id", with = FALSE]) > 0
  detected_key <- NULL

  if (has_duplicates) {
    if (type == "contact") {
      return(null_result)
    }
    detected_key <- resolve_longitudinal_key(merged, participant_key, call)
    if (is.null(detected_key)) {
      return(null_result)
    }
    merged[, ("..main_id") := seq_len(.N)]
  }

  warn_merge_quality(merged, contact_data[[file]], common_id, file, type, call)
  merged[, ("..merge_id") := NULL]

  list(merged = merged, detected_key = detected_key)
}

#' Warn about unmatched rows after a merge
#' @noRd
warn_merge_quality <- function(
  merged,
  file_data,
  common_id,
  file,
  type,
  call
) {
  matched_main <- sum(!is.na(merged[["..merge_id"]]))
  unmatched_main <- nrow(merged) - matched_main
  if (unmatched_main > 0) {
    cli::cli_warn(
      "Only {matched_main} matching value{?s} in {.val {common_id}} \\
      column{?s} when pulling {.file {basename(file)}} into \\
      {.val {type}} survey.",
      call = call
    )
  }
  matched_merge <- uniqueN(merged[["..merge_id"]], na.rm = TRUE)
  unmatched_merge <- nrow(file_data) - matched_merge
  if (unmatched_merge > 0) {
    cli::cli_warn(
      "{unmatched_merge} row{?s} could not be matched when pulling \\
      {.file {basename(file)}} into {.val {type}} survey.",
      call = call
    )
  }
}

#' Try merging all compatible files into a single main survey table
#'
#' Iteratively merges files that share columns with the main table, repeating
#' until no further merges are possible.
#' @return A list with merged (the updated main survey), detected_key, and
#'   remaining survey_files.
#' @noRd
merge_all_files <- function(
  type,
  main_survey,
  survey_files,
  contact_data,
  participant_key = NULL,
  call = rlang::caller_env()
) {
  detected_key <- NULL
  merge_files <- get_mergeable_files(
    survey_files,
    contact_data,
    colnames(main_survey)
  )

  while (length(merge_files) > 0) {
    merged_files <- character(0)
    for (file in merge_files) {
      result <- try_merge_one_file(
        file,
        type,
        main_survey,
        contact_data,
        participant_key = participant_key,
        call = call
      )
      if (!is.null(result$merged)) {
        main_survey <- result$merged
        merged_files <- c(merged_files, file)
      }
      if (!is.null(result$detected_key)) {
        detected_key <- result$detected_key
      }
    }
    survey_files <- setdiff(survey_files, merged_files)
    if (length(merged_files) == 0) {
      break
    }
    merge_files <- get_mergeable_files(
      survey_files,
      contact_data,
      colnames(main_survey)
    )
  }

  list(
    merged = main_survey,
    detected_key = detected_key,
    survey_files = survey_files
  )
}

#' Inform user about detected longitudinal data
#' @noRd
inform_longitudinal_key <- function(
  detected_key,
  participant_key = NULL,
  call = rlang::caller_env()
) {
  if (is.null(detected_key)) {
    return(invisible(NULL))
  }
  user_key_matches <- !is.null(participant_key) &&
    setequal(detected_key, participant_key)
  if (user_key_matches) {
    return(invisible(NULL))
  }

  # nolint next: object_usage_linter. Used in cli interpolation.
  key_code <- paste0(
    "c(",
    paste0('"', detected_key, '"', collapse = ", "),
    ")"
  )
  cli::cli_inform(
    c(
      "Detected longitudinal data with unique key: {.val {detected_key}}.",
      "*" = "Will treat individuals with the same {.val part_id} as unique.",
      i = "To suppress this message, use: \\
           {.code load_survey(..., participant_key = {key_code})}"
    ),
    call = call
  )
}

## lastly, merge in any additional files that can be merged
#' @autoglobal
try_merge_additional_files <- function(
  main_types,
  main_surveys,
  survey_files,
  contact_data,
  participant_key = NULL,
  call = rlang::caller_env()
) {
  observation_key <- NULL

  for (type in main_types) {
    result <- merge_all_files(
      type,
      main_surveys[[type]],
      survey_files,
      contact_data,
      participant_key = participant_key,
      call = call
    )
    main_surveys[[type]] <- result$merged[, ("..main_id") := NULL]
    survey_files <- result$survey_files

    inform_longitudinal_key(result$detected_key, participant_key, call)

    if (type == "participant" && !is.null(result$detected_key)) {
      obs_cols <- setdiff(result$detected_key, "part_id")
      if (length(obs_cols) > 0) {
        observation_key <- obs_cols
      }
    }
  }

  for (file in survey_files) {
    cli::cli_warn(
      message = "Could not merge {.file {basename(file)}}.",
      call = call
    )
  }

  list(
    surveys = main_surveys,
    observation_key = observation_key
  )
}

## join files that can be joined
#' @autoglobal
join_possible_files <- function(
  survey_files,
  contact_data,
  main_types,
  main_surveys,
  participant_key = NULL
) {
  survey_contact_data <- join_compatible_files(survey_files, contact_data)
  contact_data <- survey_contact_data$contact_data
  survey_files <- survey_contact_data$survey_files

  ## lastly, merge in any additional files that can be merged
  result <- try_merge_additional_files(
    main_types,
    main_surveys,
    survey_files,
    contact_data,
    participant_key = participant_key
  )

  list(
    surveys = result$surveys,
    observation_key = result$observation_key
  )
}

test_that("find_unique_key returns base_id when already unique", {
  data <- data.table::data.table(
    part_id = 1:100,
    wave = rep(1:2, 50),
    age = sample(20:60, 100, replace = TRUE)
  )
  expect_identical(find_unique_key(data, "part_id"), "part_id")
})

test_that("find_unique_key finds single column extension", {
  data <- data.table::data.table(
    part_id = rep(1:50, each = 2),
    wave = rep(1:2, 50),
    age = sample(20:60, 100, replace = TRUE)
  )
  result <- find_unique_key(data, "part_id")
  expect_identical(result, c("part_id", "wave"))
})

test_that("find_unique_key finds two-column extension", {
  # Create data where part_id + wave + studyDay is needed
  data <- data.table::rbindlist(list(
    data.table::data.table(
      part_id = rep(1:50, each = 2),
      wave = 1,
      studyDay = rep(1:2, 50)
    ),
    data.table::data.table(
      part_id = rep(25:74, each = 2),
      wave = 2,
      studyDay = rep(1:2, 50)
    )
  ))
  result <- find_unique_key(data, "part_id")
  # Should find wave + studyDay (both have 2 unique values)
  expect_length(result, 3)
  expect_true("part_id" %in% result)
  expect_identical(anyDuplicated(data, by = result), 0L)
})

test_that("find_unique_key returns NULL when no unique key exists", {
  # True many-to-many: no combination makes it unique
  data <- data.table::data.table(
    part_id = rep(1:10, each = 10),
    category = rep(rep(c("A", "B"), each = 5), 10)
  )
  expect_null(find_unique_key(data, "part_id"))
})

test_that("find_unique_key ignores internal columns", {
  data <- data.table::data.table(
    part_id = rep(1:50, each = 2),
    wave = rep(1:2, 50),
    ..main_id = 1:100,
    ..merge_id = 1:100
  )
  result <- find_unique_key(data, "part_id")
  expect_identical(result, c("part_id", "wave"))
})

# check if the arguments match from dots to fun_one or fun_two
#' @importFrom methods formalArgs
check_arg_dots_in <- function(dots, fun_one, fun_two) {
  unknown_args <- setdiff(
    names(dots),
    union(formalArgs(fun_one), formalArgs(fun_two))
  )
  any_unknown_args <- length(unknown_args) > 0
  if (any_unknown_args) {
    cli::cli_abort("Unknown argument{?s}: {.val {unknown_args}}.")
  }
}

check_if_contact_survey <- function(
  x,
  arg = rlang::caller_arg(x),
  call = rlang::caller_env()
) {
  if (!inherits(x, "contact_survey")) {
    cli::cli_abort(
      message = "{.arg {arg}} must be a survey object (created using \\
         {.fn survey} or {.fn get_survey}).",
      call = call
    )
  }
}

check_age_limits_increasing <- function(
  x,
  arg = rlang::caller_arg(x),
  call = rlang::caller_env()
) {
  if (!missing(x)) {
    x <- as.integer(x)
    if (anyNA(x) || any(diff(x) <= 0)) {
      cli::cli_abort(
        message = c(
          "{.arg {arg}} must be an increasing integer vector of lower age \\
          limits.",
          "i" = "We see: {.val {x}}" # nolint
        ),
        call = call
      )
    }
  }
}

check_any_missing_countries <- function(
  survey_countries,
  country_pop,
  call = rlang::caller_env()
) {
  missing_countries <- setdiff(
    survey_countries,
    unique(country_pop$country)
  )
  any_missing_country <- length(missing_countries) > 0
  if (any_missing_country) {
    cli::cli_abort(
      message = c(
        "Could not find population data for: {.val {missing_countries}}.",
        i = "Pass population data directly via the {.arg survey_pop} argument."
      ),
      call = call
    )
  }
}

check_missing_countries <- function(
  countries,
  corrected_countries,
  call = rlang::caller_env()
) {
  missing_countries <- countries[which(is.na(corrected_countries))]
  any_missing_countries <- length(missing_countries) > 0
  if (any_missing_countries) {
    cli::cli_abort(
      message = "Survey data not found for: {.val {missing_countries}}.",
      call = call
    )
  }
}

check_files_exist <- function(files, call = rlang::caller_env()) {
  exist <- file.exists(files)
  files_missing <- files[!exist]
  if (length(files_missing) > 0) {
    cli::cli_abort(
      message = "File{?s} {.file {files_missing}} not found.",
      call = call
    )
  }
}

warn_if_counts_and_split <- function(
  counts,
  split,
  call = rlang::caller_env()
) {
  if (counts && split) {
    cli::cli_warn(
      "{.code split = TRUE} does not make sense with {.code counts = TRUE}; \\
        will not split the contact matrix.",
      call = call
    )
  }
}

warn_counts_split_per_capita <- function(
  counts,
  split,
  per_capita,
  call = rlang::caller_env()
) {
  if (per_capita && counts) {
    cli::cli_warn(
      message = "{.code per_capita = TRUE} does not make sense with \\
      {.code counts = TRUE}; will not return the contact matrix per capita.",
      call = call
    )
  }
  if (per_capita && split) {
    cli::cli_warn(
      message = "{.code per_capita = TRUE} does not make sense with \\
      {.code split = TRUE}; will not return the contact matrix per capita.",
      call = call
    )
  }
}

check_na_in_weighted_matrix <- function(
  weighted_matrix,
  split,
  call = rlang::caller_env()
) {
  if (na_in_weighted_matrix(weighted_matrix) && split) {
    ## construct a warning in case there are NAs
    # nolint next: object_usage_linter. Used in cli interpolation.
    warning_suggestion <- build_na_warning(weighted_matrix)
    cli::cli_warn(
      message = c(
        "{.code split = TRUE} does not work with missing data; will not
          split the contact matrix.",
        "i" = "{warning_suggestion}" # nolint
      ),
      call = call
    )
  }
}

warn_symmetric_counts_na <- function(
  symmetric,
  counts,
  weighted_matrix,
  call = rlang::caller_env()
) {
  if (symmetric && counts) {
    cli::cli_warn(
      message = "{.code symmetric = TRUE} does not make sense with
        {.code counts = TRUE}; will not make matrix symmetric.",
      call = call
    )
  }

  if (symmetric && na_in_weighted_matrix(weighted_matrix)) {
    cli::cli_warn(
      message = c(
        "{.code symmetric = TRUE} does not work with missing data; will \\
          not make matrix symmetric.",
        # nolint start
        "i" = "{build_na_warning(weighted_matrix)}"
        # nolint end
      ),
      call = call
    )
  }
}

warn_norm_fct_exceed_thresh <- function(
  normalised_weighted_matrix,
  weighted_matrix,
  symmetric_norm_threshold,
  call = rlang::caller_env()
) {
  # show warning if normalisation factors exceed the symmetric_norm_threshold
  normalisation_fctr <- normalisation_factors(
    normalised_weighted_matrix,
    weighted_matrix
  )

  if (any(normalisation_fctr > symmetric_norm_threshold, na.rm = TRUE)) {
    cli::cli_warn(
      message = c(
        "Large differences in the size of the sub-populations with the \\
            current age breaks are likely to result in artefacts after making \\
            the matrix symmetric.",
        "!" = "Please reconsider the age breaks to obtain more equally \\
            sized sub-populations.",
        # nolint start
        "i" = "Normalization factors: [{round(range(normalisation_fctr, \\
            na.rm = TRUE), digits = 1)}]"
        # nolint end
      ),
      call = call
    )
  }
}


warn_if_no_survey_countries <- function(
  survey_representative,
  call = rlang::caller_env()
) {
  if (survey_representative) {
    cli::cli_warn(
      message = c(
        "No {.arg survey.pop} or {.arg countries} given, and no
              {.arg country} column found in the data.",
        # nolint start
        "i" = "I don't know which population this is from (assuming the \\
              survey is representative)."
        # nolint end
      ),
      call = call
    )
  }
}

#' Change age groups in population data
#'
#' This changes population data to have age groups with the given
#' age_limits, extrapolating linearly between age groups (if more
#' are requested than available) and summing populations (if fewer
#' are requested than available)
#' @return data frame of age-specific population data
#' @importFrom data.table data.table setkeyv
#' @param pop a data frame with columns indicating lower age
#'   limits and population sizes (see 'pop_age_column' and
#'   'pop_column')
#' @param age_limits lower age limits of age groups to extract; if NULL
#'   (default), the population data is returned unchanged
#' @param pop_age_column column in the 'pop' data frame indicating
#'   the lower age group limit
#' @param pop_column column in the 'pop' data frame indicating
#'   the population size
#' @param ... ignored
#' @param age.limits,pop.age.column,pop.column
#'   `r lifecycle::badge("deprecated")` Use the underscore
#'   versions (e.g., `age_limits`) instead.
#'
#' @autoglobal
#' @importFrom utils hasName
#'
#' @examples
#' ages_it_2015 <- wpp_age("Italy", 2015)
#'
#' # Modify the age data.frame to get age groups of 10 years instead of 5
#' pop_age(ages_it_2015, age_limits = seq(0, 100, by = 10))
#'
#' # The function will also automatically interpolate if necessary
#' pop_age(ages_it_2015, age_limits = c(0, 18, 40, 65))
#'
#' @export
pop_age <- function(
  pop,
  age_limits = NULL,
  pop_age_column = "lower.age.limit",
  pop_column = "population",
  ...,
  age.limits = deprecated(),
  pop.age.column = deprecated(),
  pop.column = deprecated()
) {
  chkDots(...)

  ## Handle deprecated arguments
  if (lifecycle::is_present(age.limits)) {
    lifecycle::deprecate_warn(
      "1.0.0",
      "pop_age(age.limits)",
      "pop_age(age_limits)"
    )
    age_limits <- age.limits
  }
  pop_age_column <- deprecate_arg(
    pop.age.column,
    pop_age_column,
    "pop.age.column",
    "pop_age_column",
    "pop_age"
  )
  pop_column <- deprecate_arg(
    pop.column,
    pop_column,
    "pop.column",
    "pop_column",
    "pop_age"
  )

  if (
    !is.data.frame(pop) || !all(hasName(pop, c(pop_age_column, pop_column)))
  ) {
    cli::cli_abort(
      "Expecting {.arg pop} to be a data.frame with columns
      {.arg {pop_age_column}} and {.arg {pop_column}}."
    )
  }

  ## Return early if no age_limits specified - data stays truly unchanged
  if (is.null(age_limits)) {
    return(pop)
  }

  pop <- data.table(pop)
  setkeyv(pop, pop_age_column)

  age_limits <- sort(age_limits)
  max_age <- max(pop[, pop_age_column, with = FALSE])
  missing_ages <- setdiff(
    age_limits[age_limits <= max_age],
    pop[[pop_age_column]]
  )
  if (length(missing_ages) > 0) {
    cli::cli_warn(
      c(
        "Not all age groups represented in population data (5-year age band).",
        # nolint start
        "i" = "Linearly estimating age group sizes from the 5-year bands."
        # nolint end
      )
    )
    ..original.upper.age.limit <- NULL
    pop <- pop[,
      ..original.upper.age.limit := c(pop[[pop_age_column]][-1], NA)
    ]
    pop <- pop[, ..original.lower.age.limit := get(pop_age_column)]
    all_ages <- data.frame(age_limits[
      age_limits <= max(pop[[pop_age_column]])
    ])
    colnames(all_ages) <- pop_age_column
    pop <- merge(pop, all_ages, all = TRUE, by = pop_age_column)
    pop <- pop[, ..segment := cumsum(!is.na(..original.lower.age.limit))]
    pop <- pop[,
      ..original.lower.age.limit := ..original.lower.age.limit[1],
      by = ..segment
    ]
    pop <- pop[,
      ..original.upper.age.limit := ..original.upper.age.limit[1],
      by = ..segment
    ]
    pop <- pop[, paste(pop_column) := get(pop_column)[1], by = ..segment]
    pop <- pop[, ..upper.age.limit := c(pop[[pop_age_column]][-1], NA)]
    pop[
      !is.na(..original.upper.age.limit),
      paste(pop_column) := round(
        get(pop_column) *
          (..upper.age.limit - get(pop_age_column)) /
          (..original.upper.age.limit - ..original.lower.age.limit)
      )
    ]
    pop <- pop[, c(pop_age_column, pop_column), with = FALSE]
  }

  pop <- pop[get(pop_age_column) >= min(age_limits)]
  pop <- pop[,
    paste(pop_age_column) := reduce_agegroups(get(pop_age_column), age_limits)
  ]
  pop <- pop[, list(..population = sum(get(pop_column))), by = pop_age_column]
  setnames(pop, "..population", pop_column)

  setkeyv(pop, pop_age_column)
  as.data.frame(pop)
}

#' Resolve survey population to match matrix age groups
#'
#' @param survey_pop a data frame with columns `lower.age.limit` and
#'   `population` (e.g. from [wpp_age()])
#' @param age_limits numeric vector of age group lower limits from the matrix
#' @param ... passed to [pop_age()] for interpolation
#' @returns a data.table with `lower.age.limit`, `population`, and
#'   `upper.age.limit` aligned to the matrix age groups
#' @keywords internal
#' @autoglobal
resolve_survey_pop <- function(survey_pop, age_limits, ...) {
  if (!is.data.frame(survey_pop)) {
    cli::cli_abort("{.arg survey_pop} must be a data frame.")
  }
  required <- c("lower.age.limit", "population")
  missing_cols <- setdiff(required, colnames(survey_pop))
  if (length(missing_cols) > 0) {
    cli::cli_abort(
      "{.arg survey_pop} must have column{?s} {.val {missing_cols}}."
    )
  }

  survey_pop <- data.table::data.table(survey_pop)
  survey_pop <- add_survey_upper_age_limit(
    survey = survey_pop,
    age_breaks = age_limits
  )
  adjust_survey_age_groups(
    survey_pop = survey_pop,
    part_age_group_present = age_limits,
    ...
  )
}

#' Symmetrise a contact matrix
#'
#' @description
#' Makes a contact matrix symmetric so that \eqn{c_{ij} N_i = c_{ji} N_j},
#' where \eqn{c_{ij}} is the (i, j) entry and \eqn{N_i} is the population
#' of age group i. This is done by replacing each pair with half their sum,
#' weighted by population size.
#'
#' @param x a list as returned by [compute_matrix()], with elements `matrix`
#'   and `participants`
#' @param survey_pop a data frame with columns `lower.age.limit` and
#'   `population` (e.g. from [wpp_age()])
#' @param symmetric_norm_threshold threshold for the normalisation factor
#'   before issuing a warning (default 2)
#' @param ... passed to [pop_age()] for interpolation
#' @returns `x` with `$matrix` replaced by the symmetrised version
#'
#' @examples
#' data(polymod)
#' pop <- wpp_age("United Kingdom", 2005)
#' polymod |>
#'   (\(s) s[country == "United Kingdom"])() |>
#'   assign_age_groups(age_limits = c(0, 5, 15)) |>
#'   compute_matrix() |>
#'   symmetrise(survey_pop = pop)
#'
#' @export
#' @autoglobal
symmetrise <- function(
  x,
  survey_pop,
  symmetric_norm_threshold = 2,
  ...
) {
  if (!is.list(x) || is.null(x$matrix) || is.null(x$participants)) {
    cli::cli_abort(
      "{.arg x} must be a list with elements {.val matrix} and \\
       {.val participants}, as returned by {.fn compute_matrix}."
    )
  }

  age_limits <- agegroups_to_limits(x$participants$age.group)
  resolved_pop <- resolve_survey_pop(
    survey_pop = survey_pop,
    age_limits = age_limits,
    ...
  )

  if (na_in_weighted_matrix(x$matrix)) {
    cli::cli_abort(
      c(
        "Cannot symmetrise a matrix containing {.val NA} values.",
        # nolint start
        "i" = "{build_na_warning(x$matrix)}"
        # nolint end
      )
    )
  }

  if (prod(dim(as.matrix(x$matrix))) <= 1) {
    return(x)
  }

  x$matrix <- normalise_weighted_matrix(
    survey_pop = resolved_pop,
    weighted_matrix = x$matrix,
    symmetric_norm_threshold = symmetric_norm_threshold
  )
  x
}

#' Decompose a contact matrix into mean contacts, normalisation and
#' assortativity
#'
#' @description
#' Splits the contact matrix into the mean number of contacts across the whole
#' population (`mean.contacts`), a normalisation constant (`normalisation`),
#' age-specific contact rates (`contacts`), and an assortativity matrix
#' (replacing `$matrix`). For details, see the "Getting Started" vignette.
#'
#' @inheritParams symmetrise
#' @returns `x` with `$matrix` replaced by the assortativity matrix, plus
#'   additional elements `$mean.contacts`, `$normalisation`, and `$contacts`
#'
#' @examples
#' data(polymod)
#' pop <- wpp_age("United Kingdom", 2005)
#' polymod |>
#'   (\(s) s[country == "United Kingdom"])() |>
#'   assign_age_groups(age_limits = c(0, 5, 15)) |>
#'   compute_matrix() |>
#'   split_matrix(survey_pop = pop)
#'
#' @export
#' @autoglobal
split_matrix <- function(x, survey_pop, ...) {
  if (!is.list(x) || is.null(x$matrix) || is.null(x$participants)) {
    cli::cli_abort(
      "{.arg x} must be a list with elements {.val matrix} and \\
       {.val participants}, as returned by {.fn compute_matrix}."
    )
  }

  age_limits <- agegroups_to_limits(x$participants$age.group)
  resolved_pop <- resolve_survey_pop(
    survey_pop = survey_pop,
    age_limits = age_limits,
    ...
  )

  if (na_in_weighted_matrix(x$matrix)) {
    cli::cli_abort(
      c(
        "Cannot split a matrix containing {.val NA} values.",
        # nolint start
        "i" = "{build_na_warning(x$matrix)}"
        # nolint end
      )
    )
  }

  retained_dimnames <- dimnames(x$matrix)

  splitted <- split_mean_norm_contacts(
    weighted_matrix = x$matrix,
    population = resolved_pop$population
  )

  x$matrix <- splitted$weighted_matrix
  dimnames(x$matrix) <- retained_dimnames
  x[["mean.contacts"]] <- splitted$mean_contacts
  x[["normalisation"]] <- splitted$normalisation
  x[["contacts"]] <- splitted$contacts
  x
}

#' Convert a contact matrix to per-capita rates
#'
#' @description
#' Divides each column of the contact matrix by the population of the
#' corresponding age group, giving the contact rate of age group i with
#' one individual of age group j.
#'
#' @inheritParams symmetrise
#' @returns `x` with `$matrix` replaced by the per-capita version
#'
#' @examples
#' data(polymod)
#' pop <- wpp_age("United Kingdom", 2005)
#' polymod |>
#'   (\(s) s[country == "United Kingdom"])() |>
#'   assign_age_groups(age_limits = c(0, 5, 15)) |>
#'   compute_matrix() |>
#'   per_capita(survey_pop = pop)
#'
#' @export
#' @autoglobal
per_capita <- function(x, survey_pop, ...) {
  if (!is.list(x) || is.null(x$matrix) || is.null(x$participants)) {
    cli::cli_abort(
      "{.arg x} must be a list with elements {.val matrix} and \\
       {.val participants}, as returned by {.fn compute_matrix}."
    )
  }

  age_limits <- agegroups_to_limits(x$participants$age.group)
  resolved_pop <- resolve_survey_pop(
    survey_pop = survey_pop,
    age_limits = age_limits,
    ...
  )

  x$matrix <- matrix_per_capita(
    weighted_matrix = x$matrix,
    survey_pop = resolved_pop
  )
  x
}

## Shared setup ----------------------------------------------------------------
polymod_uk_grouped <- polymod |>
  (\(s) s[country == "United Kingdom"])() |>
  assign_age_groups(age_limits = c(0, 5, 15))

result <- compute_matrix(polymod_uk_grouped)

## Constructor and class -------------------------------------------------------

test_that("compute_matrix() returns a contact_matrix object", {
  expect_true(is_contact_matrix(result))
  expect_s3_class(result, "contact_matrix")
  expect_s3_class(result, "list")
})

test_that("contact_matrix inherits from list", {
  expect_type(result, "list")
})

test_that("$ access works on contact_matrix", {
  expect_true(is.matrix(result$matrix) || is.array(result$matrix))
  expect_s3_class(result$participants, "data.frame")
})

## Age notation ----------------------------------------------------------------

test_that("terminal age group uses [N,Inf) notation", {
  groups <- rownames(result$matrix)
  terminal <- groups[length(groups)]
  expect_match(terminal, "Inf")
})

## S3 methods ------------------------------------------------------------------

test_that("print() runs without error and returns invisibly", {
  expect_output(print(result))
  expect_invisible(print(result))
})

test_that("as.matrix() extracts the matrix element", {
  m <- as.matrix(result)
  expect_identical(m, result$matrix)
})

## Class preservation through postprocess --------------------------------------

pop <- wpp_age("United Kingdom", 2005)

test_that("symmetrise() preserves contact_matrix class", {
  sym <- symmetrise(result, survey_pop = pop)
  expect_s3_class(sym, "contact_matrix")
})

test_that("split_matrix() preserves contact_matrix class", {
  sp <- split_matrix(result, survey_pop = pop)
  expect_s3_class(sp, "contact_matrix")
  expect_true("mean.contacts" %in% names(sp))
  expect_true("normalisation" %in% names(sp))
  expect_true("contacts" %in% names(sp))
})

test_that("per_capita() preserves contact_matrix class", {
  pc <- per_capita(result, survey_pop = pop)
  expect_s3_class(pc, "contact_matrix")
})

test_that("chained pipeline preserves contact_matrix class", {
  chained <- result |>
    symmetrise(survey_pop = pop) |>
    split_matrix(survey_pop = pop)
  expect_s3_class(chained, "contact_matrix")
})

## Shared setup ----------------------------------------------------------------
polymod_uk_grouped <- polymod |>
  (\(s) s[country == "United Kingdom"])() |>
  assign_age_groups(age_limits = c(0, 5, 15))

pop <- wpp_age("United Kingdom", 2005)

result_base <- compute_matrix(polymod_uk_grouped)

## symmetrise ------------------------------------------------------------------

test_that("symmetrise() satisfies reciprocity", {
  sym <- symmetrise(result_base, survey_pop = pop)

  age_limits <- agegroups_to_limits(sym$participants$age.group)
  resolved <- resolve_survey_pop(pop, age_limits) # nolint: namespace_linter.

  # c_ij * N_i should equal c_ji * N_j, i.e. M * N should be symmetric
  n <- resolved$population
  scaled <- sym$matrix * n # M[i,j] * N[i] via column recycling

  expect_equal(unname(scaled), unname(t(scaled)), tolerance = 1e-10)
})

test_that("symmetrise() matches contact_matrix(symmetric = TRUE)", {
  sym <- symmetrise(result_base, survey_pop = pop)

  legacy <- contact_matrix(
    polymod,
    countries = "United Kingdom",
    age_limits = c(0, 5, 15),
    symmetric = TRUE,
    survey_pop = pop
  )

  expect_identical(sym$matrix, legacy$matrix)
})

test_that("symmetrise() errors on NA matrix", {
  bad <- result_base
  bad$matrix[1, 1] <- NA
  expect_error(symmetrise(bad, survey_pop = pop), "NA")
})

test_that("symmetrise() errors on invalid input", {
  expect_error(
    symmetrise(list(matrix = NULL), survey_pop = pop),
    "participants"
  )
  expect_error(symmetrise("not a list", survey_pop = pop), "list")
})

test_that("symmetrise() returns scalar matrix unchanged", {
  one_group <- polymod |>
    (\(s) s[country == "United Kingdom"])() |>
    assign_age_groups(age_limits = 0) |>
    compute_matrix()
  result <- symmetrise(one_group, survey_pop = pop)
  expect_identical(result$matrix, one_group$matrix)
})

## split_matrix ----------------------------------------------------------------

test_that("split_matrix() returns expected elements", {
  sp <- split_matrix(result_base, survey_pop = pop)
  expect_true("mean.contacts" %in% names(sp))
  expect_true("normalisation" %in% names(sp))
  expect_true("contacts" %in% names(sp))
  expect_type(sp$mean.contacts, "double")
  expect_length(sp$mean.contacts, 1)
  expect_type(sp$normalisation, "double")
  expect_length(sp$normalisation, 1)
  expect_type(sp$contacts, "double")
  expect_length(sp$contacts, 3)
})

test_that("split_matrix() matches contact_matrix(split = TRUE)", {
  sp <- split_matrix(result_base, survey_pop = pop)

  legacy <- contact_matrix(
    polymod,
    countries = "United Kingdom",
    age_limits = c(0, 5, 15),
    split = TRUE,
    survey_pop = pop
  )

  expect_identical(sp$matrix, legacy$matrix)
  expect_identical(sp$mean.contacts, legacy$mean.contacts)
  expect_identical(sp$normalisation, legacy$normalisation)
  expect_identical(sp$contacts, legacy$contacts)
})

test_that("split_matrix() errors on NA matrix", {
  bad <- result_base
  bad$matrix[1, 1] <- NA
  expect_error(split_matrix(bad, survey_pop = pop), "NA")
})

test_that("split_matrix() errors on invalid input", {
  expect_error(split_matrix("not a list", survey_pop = pop), "list")
})

## per_capita ------------------------------------------------------------------

test_that("per_capita() replaces $matrix with per-capita rates", {
  pc <- per_capita(result_base, survey_pop = pop)
  expect_true(is.matrix(pc$matrix))
  # Per-capita rates should be smaller than original rates
  expect_true(all(pc$matrix < result_base$matrix))
})

test_that("per_capita() matches contact_matrix(per_capita = TRUE)", {
  pc <- per_capita(result_base, survey_pop = pop)

  legacy <- contact_matrix(
    polymod,
    countries = "United Kingdom",
    age_limits = c(0, 5, 15),
    per_capita = TRUE,
    survey_pop = pop
  )

  expect_identical(pc$matrix, legacy$matrix.per.capita)
})

test_that("per_capita() errors on invalid input", {
  expect_error(per_capita("not a list", survey_pop = pop), "list")
})

## resolve_survey_pop ----------------------------------------------------------

test_that("resolve_survey_pop() errors on missing columns", {
  expect_error(
    resolve_survey_pop(data.frame(x = 1), c(0, 5)), # nolint: namespace_linter.
    "lower.age.limit"
  )
})

test_that("resolve_survey_pop() errors on non-data-frame input", {
  expect_error(
    resolve_survey_pop("not a df", c(0, 5)), # nolint: namespace_linter.
    "data frame"
  )
})

# Generated by roxyglobals: do not edit by hand

utils::globalVariables(c(
  "country", # <clean.contact_survey>
  "..age.value", # <clean.contact_survey>
  "..age.unit", # <clean.contact_survey>
  "..high", # <clean.contact_survey>
  "..low", # <clean.contact_survey>
  "weight", # <compute_matrix>
  "sampled.weight", # <compute_matrix>
  "age", # <contact_age_distribution>
  "proportion", # <contact_age_distribution>
  "N", # <contact_age_distribution>
  "cnt_age", # <drop_ages_below_age_limit>
  "cnt_age", # <drop_missing_contact_ages>
  "part_age_exact", # <max_participant_age>
  "part_age_est_max", # <max_participant_age>
  "part_age", # <max_participant_age>
  "part_age", # <get_age_limits>
  "part_age_exact", # <get_age_limits>
  "cnt_age", # <get_age_limits>
  "cnt_age_exact", # <get_age_limits>
  "country", # <filter_countries>
  "part_age", # <drop_invalid_ages>
  "cnt_age", # <drop_invalid_contact_ages>
  "part_id", # <drop_invalid_contact_ages>
  "lower.age.limit", # <adjust_ppt_age_group_breaks>
  "part_age", # <adjust_ppt_age_group_breaks>
  "age.group", # <adjust_ppt_age_group_breaks>
  "age.group", # <age_group_labels>
  "country", # <get_survey_countries>
  "population", # <survey_pop_from_countries>
  "lower.age.limit", # <survey_pop_from_countries>
  "lower.age.limit", # <add_survey_upper_age_limit>
  "population", # <add_survey_upper_age_limit>
  "upper.age.limit", # <adjust_survey_age_groups>
  "sum_weight", # <weight_by_day_of_week>
  "dayofweek", # <weight_by_day_of_week>
  "weight", # <weight_by_day_of_week>
  "is.weekday", # <weight_by_day_of_week>
  "age.count", # <weight_by_age>
  "part_age", # <weight_by_age>
  "age.proportion", # <weight_by_age>
  "population.proportion", # <weight_by_age>
  "population.count", # <weight_by_age>
  "weight.age", # <weight_by_age>
  "weight", # <weight_by_age>
  "weight", # <weigh_by_user_defined>
  "weight", # <normalise_weights>
  "part_id", # <merge_participants_contacts>
  "cnt_age", # <sample_present_age>
  "age.group", # <sample_present_age>
  "cnt_age", # <sample_uniform_age>
  "age.group", # <sample_uniform_age>
  "cnt_age", # <impute_age_by_sample>
  "age.group", # <impute_age_by_sample>
  "cnt_age", # <add_contact_age_groups>
  "contact.age.group", # <add_contact_age_groups>
  "weight", # <create_bootstrap_weights>
  "id", # <create_bootstrap_weights>
  "part_id", # <create_bootstrap_weights>
  "part_id", # <sample_from_participants>
  "lower.age.limit", # <sample_from_participants>
  "sampled.weight", # <sample_from_participants>
  "weight", # <sample_from_participants>
  "bootstrap.weight", # <sample_from_participants>
  "sampled.weight", # <sample_contacts_participants>
  "weight", # <sample_contacts_participants>
  "age.group", # <n_participants_per_age_group>
  "proportion", # <n_participants_per_age_group>
  "age.group", # <return_participant_weights>
  "weight", # <return_participant_weights>
  "part_age", # <return_participant_weights>
  "is.weekday", # <return_participant_weights>
  "proportion", # <return_participant_weights>
  "participants", # <return_participant_weights>
  "weight", # <contact_matrix>
  "is.weekday", # <contact_matrix>
  "dayofweek", # <contact_matrix>
  "cnt_age", # <contact_matrix>
  "lower.age.limit", # <contact_matrix>
  "age.group", # <contact_matrix>
  "population", # <contact_matrix>
  "part_age", # <contact_matrix>
  "proportion", # <contact_matrix>
  "participants", # <contact_matrix>
  "file_name", # <download_survey>
  "common_doi", # <.list_surveys>
  "doi.nb", # <.list_surveys>
  "identifier.1", # <.list_surveys>
  "title", # <.list_surveys>
  "creator", # <.list_surveys>
  "identifier.2", # <.list_surveys>
  "..original.lower.age.limit", # <pop_age>
  "..segment", # <pop_age>
  "..upper.age.limit", # <pop_age>
  "part_id", # <[.contact_survey>
  "population", # <survey_country_population>
  "weight", # <weigh>
  "weight", # <weigh_direct>
  "weight", # <weigh_grouped>
  "weight", # <weigh_named>
  "sex", # <wpp_age>
  "country", # <wpp_age>
  "name", # <wpp_age>
  "country_code", # <wpp_age>
  "lower.age.limit", # <wpp_age>
  "age", # <wpp_age>
  "female", # <wpp_age>
  "male", # <wpp_age>
  NULL
))

#' Load a survey from local files
#'
#' @description Loads a survey from a local file system. Tables are expected
#'   as csv files, and a reference (if present) as JSON.
#' @param files a vector of file names as returned by [download_survey()]
#' @param participant_key character vector specifying columns that uniquely
#'   identify participant observations. For cross-sectional surveys this is
#'   typically just `"part_id"` (the default). For longitudinal surveys with
#'   multiple observations per participant, specify additional columns like
#'   `c("part_id", "wave")`. When `NULL` (the default), the function will
#'   auto-detect if additional columns are needed and inform you.
#' @param ... options for [clean()], which is called at the end of this
#' @autoglobal
#' @importFrom jsonlite fromJSON
#' @examples
#' \dontrun{
#' list_surveys()
#' peru_files <- download_survey("https://doi.org/10.5281/zenodo.1095664")
#' peru_survey <- load_survey(peru_files)
#'
#' # For longitudinal surveys, specify the unique key explicitly:
#' france_files <- download_survey("https://doi.org/10.5281/zenodo.1157918")
#' france_survey <- load_survey(france_files,
#'   participant_key = c("part_id", "wave", "studyDay")
#' )
#' }
#' @return a survey in the correct format. For longitudinal surveys with
#'   multiple observations per participant, the returned object includes an
#'   `observation_key` field containing the column names (excluding `part_id`)
#'   that distinguish observations for the same participant.
#' @export
load_survey <- function(files, participant_key = NULL, ...) {
  check_files_exist(files)

  # select csv files
  survey_files <- grep("\\.csv$", files, value = TRUE, ignore.case = TRUE)
  reference <- extract_reference(files)

  contact_data <- lapply(survey_files, fread)
  names(contact_data) <- survey_files

  main_types <- c("participant", "contact")
  main_surveys <- list()

  ## first, get the common files
  for (type in main_types) {
    main_file <- extract_type_common_csv(type, survey_files)
    main_surveys[[type]] <- rbindlist(contact_data[main_file], fill = TRUE)
    main_surveys[[type]][, ("..main_id") := seq_len(.N)]
    survey_files <- setdiff(survey_files, main_file)
  }

  ## join files that can be joined
  result <- join_possible_files(
    survey_files,
    contact_data,
    main_types,
    main_surveys,
    participant_key = participant_key
  )
  main_surveys <- result$surveys
  observation_key <- result$observation_key

  new_survey <- as_contact_survey(
    list(
      participants = main_surveys[["participant"]],
      contacts = main_surveys[["contact"]],
      reference = reference
    )
  )

  # Store observation key if longitudinal data was detected
  if (!is.null(observation_key)) {
    new_survey$observation_key <- observation_key
  }

  if (!is.null(new_survey$reference)) {
    cli::cli_inform(
      "Using {new_survey$reference$title}. To cite this in a publication,\\
      use the {.fn get_citation} function."
    )
  }

  new_survey
}

#' @title Contact survey
#' @description Deprecated. Use `as_survey` instead.
#' @param participants a `data.frame` containing information on participants
#' @param contacts a `data.frame` containing information on contacts
#' @param reference a `list` containing information needed to
#'   reference the survey, in particular it can contain a "title",
#'   "bibtype", "author", "doi", "publisher", "note", "year"
#' @return a new survey object
#' @author Sebastian Funk
#' @export
survey <- function(participants, contacts, reference = NULL) {
  lifecycle::deprecate_warn(
    "1.0.0",
    "survey()",
    details = "Use `as_contact_survey()` instead."
  )
  structure(
    list(
      participants = data.table(participants),
      contacts = data.table(contacts),
      reference = reference
    ),
    class = "contact_survey"
  )
}

#' Assign age groups in survey data
#'
#' @description
#' This function processes age data in a survey object. It imputes ages from
#'   ranges, handles missing values, and assigns age groups.
#'
#' @param survey a [survey()] object
#' @param age_limits lower limits of the age groups over which to construct
#'   the matrix. Defaults to NULL. If NULL, age limits are inferred from
#'   participant and contact ages.
#' @param estimated_participant_age if set to "mean" (default), people whose
#'   ages are given as a range (in columns named "..._est_min" and
#'   "..._est_max") but not exactly (in a column named "..._exact") will have
#'   their age set to the mid-point of the range; if set to "sample", the age
#'   will be sampled from the range; if set to "missing", age ranges will be
#'   treated as missing
#' @param estimated_contact_age if set to "mean" (default), contacts whose ages
#'   are given as a range (in columns named "..._est_min" and "..._est_max") but
#'   not exactly (in a column named "..._exact") will have their age set to the
#'   mid-point of the range; if set to "sample", the age will be sampled from
#'   the range; if set to "missing", age ranges will be treated as missing
#' @param missing_participant_age if set to "remove" (default), participants
#'   without age information are removed; if set to "keep", participants with
#'   missing age are kept and treated as a separate age group
#' @param missing_contact_age if set to "remove" (default), participants that
#'   have contacts without age information are removed; if set to "sample",
#'   contacts without age information are sampled from all the contacts of
#'   participants of the same age group; if set to "keep", contacts with missing
#'   age are kept and treated as a separate age group; if set to "ignore",
#'   contact with missing age are ignored in the contact analysis
#' @returns
#' The survey object with processed age data.
#'
#' @importFrom rlang %||%
#' @export
#' @autoglobal
#' @examples
#' polymod_grouped <- assign_age_groups(polymod)
#' polymod_grouped
#' polymod_custom <- assign_age_groups(polymod, age_limits = c(0, 5, 10, 15))
#' polymod_custom
assign_age_groups <- function(
  survey,
  age_limits = NULL,
  estimated_participant_age = c("mean", "sample", "missing"),
  estimated_contact_age = c("mean", "sample", "missing"),
  missing_participant_age = c("remove", "keep"),
  missing_contact_age = c("remove", "sample", "keep", "ignore")
) {
  check_if_contact_survey(survey)
  check_age_limits_increasing(age_limits)
  estimated_participant_age <- rlang::arg_match(estimated_participant_age)
  if (is.data.frame(estimated_contact_age)) {
    estimated_contact_age <- validate_age_distribution(estimated_contact_age)
  } else {
    estimated_contact_age <- rlang::arg_match(estimated_contact_age)
  }
  missing_participant_age <- rlang::arg_match(missing_participant_age)
  missing_contact_age <- rlang::arg_match(missing_contact_age)

  if (missing_contact_age == "sample") {
    lifecycle::deprecate_warn(
      "0.5.0",
      "assign_age_groups(missing_contact_age = 'sample')",
      details = paste(
        "Sampling missing contact ages will be removed in a future version.",
        "Use 'remove' to exclude contacts with missing ages, 'keep' to retain",
        "them as a separate age group, or 'ignore' to drop only those contacts."
      )
    )
  }

  ## set contact age and participant age if it's not in the data
  survey$participants <- add_part_age(survey$participants)
  survey$contacts <- add_contact_age(survey$contacts)

  ## Impute participant ages from ranges ------------------------------------
  survey$participants <- impute_participant_ages(
    participants = survey$participants,
    estimate = estimated_participant_age
  )

  ## convert factors to integers, preserving numeric values
  survey$contacts <- convert_factor_to_integer(
    data = survey$contacts,
    cols = c(
      "cnt_age",
      "cnt_age_est_min",
      "cnt_age_est_max",
      "cnt_age_exact"
    )
  )

  ## Impute contact ages from ranges ------------------------------------------
  survey$contacts <- impute_contact_ages(
    contacts = survey$contacts,
    estimate = estimated_contact_age
  )

  # define age limits if not given
  age_limits <- age_limits %||% get_age_limits(survey)

  ## Process participant ages: handle missing data ----------------------------
  survey$participants <- drop_invalid_ages(
    participants = survey$participants,
    missing_action = missing_participant_age,
    age_limits = age_limits
  )

  ## Process contact ages: handle missing data --------------------------------
  # remove contact ages below the age limit, before dealing with missing ages
  survey$contacts <- drop_ages_below_age_limit(
    data = survey$contacts,
    age_limits = age_limits
  )

  survey$participants <- drop_invalid_contact_ages(
    contacts = survey$contacts,
    participants = survey$participants,
    missing_action = missing_contact_age
  )

  survey$contacts <- drop_missing_contact_ages(
    contacts = survey$contacts,
    missing_action = missing_contact_age
  )

  ## adjust age.group.breaks to the lower and upper ages in the survey ---------
  survey$participants <- adjust_ppt_age_group_breaks(
    participants = survey$participants,
    age_limits = age_limits
  )

  ## assign contact age groups based on participant age groups ----------------
  max_age <- max_participant_age(survey$participants)
  survey$contacts <- add_contact_age_groups(
    contacts = survey$contacts,
    age_breaks = create_age_breaks(age_limits, max_age),
    age_groups = age_group_labels(survey$participants)
  )

  survey
}

#' Convert lower age limits to age groups.
#'
#' Mostly used for plot labelling
#' @param x age limits to transform
#' @param limits lower age limits; if not given, will use all limits in `x`
#' @param notation whether to use bracket notation, e.g. [0,4) or dash
#'   notation, e.g. 0-4)
#' @return Age groups as specified in `notation`
#' @examples
#' limits_to_agegroups(c(0, 5, 10))
#' @export
limits_to_agegroups <- function(
  x,
  limits = sort(unique(x)),
  notation = c("dashes", "brackets")
) {
  if (missing(notation)) {
    cli::cli_warn(
      message = c(
        "In the next version of {.pkg socialmixr}, {.arg notation} will default
        to \"brackets\", instead of \"dashes\".",
        # nolint start
        "i" = "Prevent this using {.code notation = \"dashes\"} in the call \\
        to {.fn limits_to_agegroups}."
        # nolint end
      )
    )
  }
  notation <- match.arg(notation)
  limits <- limits[!is.na(limits)]
  agegroups <- if (length(limits) > 1) {
    if (notation == "brackets") {
      sprintf("[%s,%s)", limits[-length(limits)], limits[-1])
    } else if (notation == "dashes") {
      vapply(
        seq(1, length(limits) - 1),
        function(y) {
          if ((limits[y + 1] - 1) > limits[y]) {
            paste(limits[y], limits[y + 1] - 1, sep = "-")
          } else {
            paste(limits[y])
          }
        },
        ""
      )
    }
  } else {
    NULL
  }
  terminal <- if (notation == "brackets") {
    sprintf("[%s,Inf)", limits[length(limits)])
  } else {
    paste0(limits[length(limits)], "+")
  }
  agegroups <- c(agegroups, terminal)
  agegroups <- factor(agegroups, levels = agegroups, ordered = TRUE)
  names(agegroups) <- limits
  unname(agegroups[as.character(x)])
}

#' Convert age groups to lower age limits
#'
#' Inverse of [limits_to_agegroups()]. Extracts lower age limits from age group
#'   labels.
#' @param x age groups (a factor, as produced by [limits_to_agegroups()] or
#'   [assign_age_groups()])
#' @return a numeric vector of lower age limits
#' @examples
#' agegroups_to_limits(limits_to_agegroups(c(0, 5, 10), notation = "brackets"))
#' @export
agegroups_to_limits <- function(x) {
  lvls <- if (is.factor(x)) levels(x) else unique(as.character(x))
  as.numeric(sub("^\\[?(\\d+).*", "\\1", lvls))
}

test_that("age groups can be created and manipulated", {
  ages <- seq_len(50)
  age_limits <- c(0, 5, 10)
  groups <- reduce_agegroups(ages, age_limits)
  expect_identical(unique(groups), age_limits)
  expect_warning(limits_to_agegroups(groups), "default")
  age_groups <-
    expect_identical(
      as.character(unique(limits_to_agegroups(groups, notation = "brackets"))),
      c("[0,5)", "[5,10)", "[10,Inf)")
    )
  expect_identical(
    as.character(unique(limits_to_agegroups(groups, notation = "dashes"))),
    c("0-4", "5-9", "10+")
  )
})

test_that("age groups are ordered factors", {
  ages <- seq_len(50)
  age_limits <- c(0, 5, 10)
  groups <- reduce_agegroups(ages, age_limits)
  age_groups <- limits_to_agegroups(groups, notation = "dashes")
  expect_s3_class(age_groups, "ordered")
  expect_s3_class(age_groups, "factor")
})

test_that("pop_age doesn't change total population size", {
  ages_it_2015 <- wpp_age("Italy", 2015)

  ages_it_2015_10 <- pop_age(ages_it_2015, age_limits = seq(0, 100, by = 10))

  expect_identical(
    sum(ages_it_2015$population),
    sum(ages_it_2015_10$population)
  )

  # Even with interpolation
  # nolint start: implicit_assignment_linter
  expect_warning(
    ages_it_2015_cat <- pop_age(ages_it_2015, age_limits = c(0, 18, 40, 65)),
    "Linearly estimating"
  )
  # nolint end

  expect_snapshot_warning(
    cran = FALSE,
    pop_age(ages_it_2015, age_limits = c(0, 18, 40, 65))
  )

  expect_identical(
    sum(ages_it_2015$population),
    sum(ages_it_2015_cat$population)
  )
})

test_that("pop_age returns data unchanged when age_limits is NULL", {
  ages_it_2015 <- wpp_age("Italy", 2015)

  # Calling without age_limits should return identical data
  result <- pop_age(ages_it_2015)
  expect_identical(result, ages_it_2015)

  # Explicitly passing NULL should also work
  result_null <- pop_age(ages_it_2015, age_limits = NULL)
  expect_identical(result_null, ages_it_2015)

  # Data.table input should also be returned unchanged
  ages_dt <- data.table::as.data.table(ages_it_2015)
  result_dt <- pop_age(ages_dt)
  expect_identical(result_dt, ages_dt)
})

test_that("pop_age works with custom column names and interpolation", {
  # Create test data with non-standard column names
  pop_data <- data.frame(
    age_lower = c(0, 5, 10, 15, 20),
    pop_count = c(1000, 1200, 1100, 900, 800)
  )

  # Test with interpolation (age_limits not matching existing groups)
  # nolint start: implicit_assignment_linter
  result <- suppressWarnings(
    pop_age(
      pop_data,
      age_limits = c(0, 8, 15),
      pop_age_column = "age_lower",
      pop_column = "pop_count"
    )
  )
  # nolint end

  expect_named(result, c("age_lower", "pop_count"))
  expect_identical(result$age_lower, c(0, 8, 15))
  # Total population should be preserved
  expect_identical(sum(result$pop_count), sum(pop_data$pop_count))
})

test_that("pop_age throws warnings or errors", {
  expect_snapshot(
    error = TRUE,
    cran = FALSE,
    pop_age(3)
  )
  expect_error(pop_age(3), "to be a data.frame")
  expect_warning(wpp_age("Germany", 2011), "Don't have population data")
  expect_snapshot_warning(
    cran = FALSE,
    wpp_age("Germany", 2011)
  )
})

test_that("agegroups_to_limits round-trips (brackets)", {
  limits <- c(0, 5, 10)
  groups <- limits_to_agegroups(limits, notation = "brackets")
  result <- agegroups_to_limits(groups)
  expect_identical(result, limits)
})

test_that("agegroups_to_limits round-trips with limits_to_agegroups (dashes)", {
  limits <- c(0, 5, 10)
  groups <- limits_to_agegroups(limits, notation = "dashes")
  result <- agegroups_to_limits(groups)
  expect_identical(result, limits)
})

test_that("agegroups_to_limits works with character input", {
  groups <- c("[0,5)", "[5,10)", "10+")
  result <- agegroups_to_limits(groups)
  expect_identical(result, c(0, 5, 10))
})

test_that("agegroups_to_limits works with single age group", {
  groups <- factor("0+", levels = "0+", ordered = TRUE)
  result <- agegroups_to_limits(groups)
  expect_identical(result, 0)
})

#' Generate a contact matrix from diary survey data
#'
#' Samples a contact survey
#'
#' @param survey a [survey()] object.
#' @param countries limit to one or more countries; if NULL
#'   (default), will use all countries in the survey; these can be
#'   given as country names or 2-letter (ISO Alpha-2) country
#'   codes.
#' @param survey_pop survey population -- either a data frame with
#'   columns 'lower.age.limit' and 'population', or a character
#'   vector giving the name(s) of a country or countries from the
#'   list that can be obtained via `wpp_countries`; if NULL
#'   (default), will use the country populations from the chosen
#'   countries, or all countries in the survey if `countries` is
#'   NULL.
#' @param age_limits lower limits of the age groups over which to
#'   construct the matrix. If NULL (default), age limits are
#'   inferred from participant and contact ages.
#' @param filter any filters to apply to the data, given as list
#'   of the form (column=filter_value) - only contacts that have
#'   'filter_value' in 'column' will be considered. If multiple
#'   filters are given, they are all applied independently and in
#'   the sequence given. Default value is NULL; no filtering
#'   performed.
#' @param counts whether to return counts (instead of means).
#' @param symmetric whether to make matrix symmetric, such that
#'   \eqn{c_{ij}N_i = c_{ji}N_j}.
#' @param split whether to split the contact matrix into the mean
#'   number of contacts, in each age group (split further into the
#'   product of the mean number of contacts across the whole
#'   population (`mean.contacts`), a normalisation constant
#'   (`normalisation`) and age-specific variation in contacts
#'   (`contacts`)), multiplied with an assortativity matrix
#'   (`assortativity`) and a population multiplier (`demography`).
#'   For more detail on this, see the "Getting Started" vignette.
#' @param sample_participants whether to sample participants
#'   randomly (with replacement); done multiple times this can be
#'   used to assess uncertainty in the generated contact matrices.
#'   See the "Bootstrapping" section in the vignette for how to
#'   do this.
#' @param estimated_participant_age if set to "mean" (default),
#'   people whose ages are given as a range (in columns named
#'   "..._est_min" and "..._est_max") but not exactly (in a
#'   column named "..._exact") will have their age set to the
#'   mid-point of the range; if set to "sample", the age will be
#'   sampled from the range; if set to "missing", age ranges will
#'   be treated as missing
#' @param estimated_contact_age if set to "mean" (default),
#'   contacts whose ages are given as a range (in columns named
#'   "..._est_min" and "..._est_max") but not exactly (in a
#'   column named "..._exact") will have their age set to the
#'   mid-point of the range; if set to "sample", the age will be
#'   sampled from the range; if set to "missing", age ranges will
#'   be treated as missing.
#' @param missing_participant_age if set to "remove" (default),
#'   participants without age information are removed; if set to
#'   "keep", participants with missing age are kept and will
#'   appear in the contact matrix in a row labelled "NA".
#' @param missing_contact_age if set to "remove" (default),
#'   participants that have contacts without age information are
#'   removed; if set to "sample", contacts without age information
#'   are sampled from all the contacts of participants of the same
#'   age group; if set to "keep", contacts with missing age are
#'   kept and will appear in the contact matrix in a column
#'   labelled "NA"; if set to "ignore", contacts without age
#'   information are removed from the analysis (but the
#'   participants that made them are kept).
#' @param weights column name(s) of the participant data of the
#'   [survey()] object with user-specified weights (default =
#'   empty vector).
#' @param weigh_dayofweek whether to weigh social contacts data
#'   by the day of the week (weight (5/7 / N_week / N) for
#'   weekdays and (2/7 / N_weekend / N) for weekends).
#' @param weigh_age whether to weigh social contacts data by the
#'   age of the participants (vs. the populations' age
#'   distribution).
#' @param weight_threshold threshold value for the standardized
#'   weights before running an additional standardisation (default
#'   'NA' = no cutoff).
#' @param symmetric_norm_threshold threshold value for the
#'   normalization weights when `symmetric = TRUE` before showing
#'   a warning that that large differences in the size of the
#'   sub-populations are likely to result in artefacts when making
#'   the matrix symmetric (default 2).
#' @param sample_all_age_groups what to do if sampling
#'   participants (with `sample_participants = TRUE`) fails to
#'   sample participants from one or more age groups; if FALSE
#'   (default), corresponding rows will be set to NA, if TRUE the
#'   sample will be discarded and a new one taken instead.
#' @param sample_participants_max_tries maximum number of attempts
#'   when `sample_all_age_groups = TRUE`; defaults to 1000.
#' @param return_part_weights boolean to return the participant
#'   weights.
#' @param return_demography boolean to explicitly return
#'   demography data that corresponds to the survey data (default
#'   'NA' = if demography data is requested by other function
#'   parameters).
#' @param per_capita whether to return a matrix with contact rates
#'   per capita (default is FALSE and not possible if 'counts=TRUE'
#'   or 'split=TRUE').
# nolint start: line_length_linter.
#' @param survey.pop,age.limits,sample.participants,estimated.participant.age,estimated.contact.age,missing.participant.age,missing.contact.age,weigh.dayofweek,weigh.age,weight.threshold,symmetric.norm.threshold,sample.all.age.groups,sample.participants.max.tries,return.part.weights,return.demography,per.capita `r lifecycle::badge("deprecated")` Use the underscore-separated versions of these arguments instead.
# nolint end
#' @param ... further arguments to pass to [get_survey()],
#'   [check()] and [pop_age()] (especially column names).
#' @return a contact matrix, and the underlying demography of the
#'   surveyed population
#' @importFrom stats xtabs runif median
#' @importFrom utils data globalVariables
#' @importFrom countrycode countrycode
#' @importFrom rlang %||%
#' @import data.table
#' @export
#' @autoglobal
#' @examples
#' data(polymod)
#' contact_matrix(
#'   survey = polymod,
#'   countries = "United Kingdom",
#'   age_limits = c(0, 1, 5, 15)
#' )
#' @author Sebastian Funk
# nolint start: cyclocomp_linter.
contact_matrix <- function(
  survey,
  countries = NULL,
  survey_pop = NULL,
  age_limits = NULL,
  filter = NULL,
  counts = FALSE,
  symmetric = FALSE,
  split = FALSE,
  sample_participants = FALSE,
  estimated_participant_age = c("mean", "sample", "missing"),
  estimated_contact_age = c("mean", "sample", "missing"),
  missing_participant_age = c("remove", "keep"),
  missing_contact_age = c("remove", "sample", "keep", "ignore"),
  weights = NULL,
  weigh_dayofweek = FALSE,
  weigh_age = FALSE,
  weight_threshold = NA,
  symmetric_norm_threshold = 2,
  sample_all_age_groups = FALSE,
  sample_participants_max_tries = 1000,
  return_part_weights = FALSE,
  return_demography = NA,
  per_capita = FALSE,
  ...,
  survey.pop = deprecated(),
  age.limits = deprecated(),
  sample.participants = deprecated(),
  estimated.participant.age = deprecated(),
  estimated.contact.age = deprecated(),
  missing.participant.age = deprecated(),
  missing.contact.age = deprecated(),
  weigh.dayofweek = deprecated(),
  weigh.age = deprecated(),
  weight.threshold = deprecated(),
  symmetric.norm.threshold = deprecated(),
  sample.all.age.groups = deprecated(),
  sample.participants.max.tries = deprecated(),
  return.part.weights = deprecated(),
  return.demography = deprecated(),
  per.capita = deprecated()
) {
  ## Handle deprecated arguments -----------------------------------------------
  survey_pop <- deprecate_arg(
    survey.pop,
    survey_pop,
    "survey.pop",
    "survey_pop",
    "contact_matrix"
  )
  age_limits <- deprecate_arg(
    age.limits,
    age_limits,
    "age.limits",
    "age_limits",
    "contact_matrix"
  )
  sample_participants <- deprecate_arg(
    sample.participants,
    sample_participants,
    "sample.participants",
    "sample_participants",
    "contact_matrix"
  )
  estimated_participant_age <- deprecate_arg(
    estimated.participant.age,
    estimated_participant_age,
    "estimated.participant.age",
    "estimated_participant_age",
    "contact_matrix"
  )
  estimated_contact_age <- deprecate_arg(
    estimated.contact.age,
    estimated_contact_age,
    "estimated.contact.age",
    "estimated_contact_age",
    "contact_matrix"
  )
  missing_participant_age <- deprecate_arg(
    missing.participant.age,
    missing_participant_age,
    "missing.participant.age",
    "missing_participant_age",
    "contact_matrix"
  )
  missing_contact_age <- deprecate_arg(
    missing.contact.age,
    missing_contact_age,
    "missing.contact.age",
    "missing_contact_age",
    "contact_matrix"
  )
  weigh_dayofweek <- deprecate_arg(
    weigh.dayofweek,
    weigh_dayofweek,
    "weigh.dayofweek",
    "weigh_dayofweek",
    "contact_matrix"
  )
  weigh_age <- deprecate_arg(
    weigh.age,
    weigh_age,
    "weigh.age",
    "weigh_age",
    "contact_matrix"
  )
  weight_threshold <- deprecate_arg(
    weight.threshold,
    weight_threshold,
    "weight.threshold",
    "weight_threshold",
    "contact_matrix"
  )
  symmetric_norm_threshold <- deprecate_arg(
    symmetric.norm.threshold,
    symmetric_norm_threshold,
    "symmetric.norm.threshold",
    "symmetric_norm_threshold",
    "contact_matrix"
  )
  sample_all_age_groups <- deprecate_arg(
    sample.all.age.groups,
    sample_all_age_groups,
    "sample.all.age.groups",
    "sample_all_age_groups",
    "contact_matrix"
  )
  sample_participants_max_tries <- deprecate_arg(
    sample.participants.max.tries,
    sample_participants_max_tries,
    "sample.participants.max.tries",
    "sample_participants_max_tries",
    "contact_matrix"
  )
  return_part_weights <- deprecate_arg(
    return.part.weights,
    return_part_weights,
    "return.part.weights",
    "return_part_weights",
    "contact_matrix"
  )
  return_demography <- deprecate_arg(
    return.demography,
    return_demography,
    "return.demography",
    "return_demography",
    "contact_matrix"
  )
  per_capita <- deprecate_arg(
    per.capita,
    per_capita,
    "per.capita",
    "per_capita",
    "contact_matrix"
  )

  ## read arguments and check --------------------------------------------------
  survey_type <- c("participants", "contacts")
  dot.args <- list(...)
  check_arg_dots_in(dot.args, check.contact_survey, pop_age)
  estimated_participant_age <- match.arg(estimated_participant_age)
  estimated_contact_age <- match.arg(estimated_contact_age)
  missing_participant_age <- match.arg(missing_participant_age)
  missing_contact_age <- match.arg(missing_contact_age)

  if (missing_contact_age == "sample") {
    lifecycle::deprecate_warn(
      "0.5.0",
      "contact_matrix(missing_contact_age = 'sample')",
      details = paste(
        "Sampling missing contact ages will be removed in a future version.",
        "Use 'remove' to exclude contacts with missing ages, 'keep' to retain",
        "them as a separate age group, or 'ignore' to drop only those contacts."
      )
    )
  }

  check_if_contact_survey(survey)

  survey <- copy_survey(survey)
  check_age_limits_increasing(age_limits)

  ## Warn if survey has multiple observations per participant ------------------
  warn_multiple_observations(
    participants = survey$participants,
    observation_key = survey$observation_key,
    filter_hint = "legacy"
  )

  ## Filter to specific countries ----------------------------------------------
  # If a survey contains data from multiple countries or if countries specified
  survey$participants <- filter_countries(survey$participants, countries)

  ## Process ages: impute from ranges, handle missing, assign age groups -------
  survey <- assign_age_groups(
    survey,
    age_limits = age_limits,
    estimated_participant_age = estimated_participant_age,
    estimated_contact_age = estimated_contact_age,
    missing_participant_age = missing_participant_age,
    missing_contact_age = missing_contact_age
  )

  ## check if any filters have been requested ----------------------------------
  survey <- apply_data_filter(
    survey = survey,
    survey_type = survey_type,
    filter = filter
  )

  ## recover resolved age_limits from the assigned age groups ------------------
  age_limits <- agegroups_to_limits(survey$participants$age.group)

  ## ---------------------------------------------------------------------------
  ## if split, symmetric, or age weights are requested, get demographic data
  ## (survey population)
  need_survey_pop <- any(
    split,
    symmetric,
    weigh_age,
    isTRUE(return_demography),
    per_capita
  )

  if (need_survey_pop) {
    ## warn if population data will be looked up automatically -----------------
    has_country_info <- !is.null(countries) ||
      "country" %in% colnames(survey$participants)
    if ((is.null(survey_pop) || is.character(survey_pop)) && has_country_info) {
      lifecycle::deprecate_warn(
        when = "0.6.0",
        what = I("Automatic country population lookup in `contact_matrix()`"),
        details = c(
          paste(
            "When `countries` is given (or a `country` column is present)",
            "without `survey_pop`, contact_matrix() currently calls the",
            "soft-deprecated `wpp_age()` to look up population data. This",
            "automatic lookup will be removed in a future release: callers",
            "will then have to supply `survey_pop` whenever `symmetric`,",
            "`split`, `per_capita`, `weigh_age`, or `return_demography` is",
            "TRUE."
          ),
          i = paste(
            "Pass `survey_pop` explicitly to silence this warning, e.g.",
            "`survey_pop = survey_country_population(survey, countries)` or a",
            "data frame from the wpp2024 package."
          )
        )
      )
    }
    ## check if survey population is not given or is a country vector
    survey_pop_info <- survey_pop_year(
      survey_pop = survey_pop,
      countries = countries,
      participants = survey$participants,
      age_limits = age_limits
    )
    survey_pop <- survey_pop_info$survey_pop
    survey.year <- survey_pop_info$survey_year

    part.age.group.present <- get_age_group_lower_limits(age_limits)

    survey_pop <- add_survey_upper_age_limit(
      survey = survey_pop,
      age_breaks = part.age.group.present
    )

    if (weigh_age) {
      ## keep reference of survey_pop
      survey_pop.full <- survey_pop_reference(survey_pop, ...)
    }

    ## adjust age groups by interpolating, in case they don't match between
    ## demographic and survey data
    survey_pop <- adjust_survey_age_groups(
      survey_pop = survey_pop,
      part_age_group_present = part.age.group.present,
      ...
    )
  }

  ## Process weights -----------------------------------------------------------
  survey$participants[, weight := 1]

  if (weigh_dayofweek) {
    if ("dayofweek" %in% colnames(survey$participants)) {
      survey <- weigh(
        survey,
        "dayofweek",
        target = c(5, 2),
        groups = list(1:5, c(0, 6))
      )
      # Add is.weekday for return_part_weights compatibility
      # Use fifelse to preserve NA (NA %in% 1:5 would return FALSE)
      survey$participants[,
        is.weekday := fifelse(is.na(dayofweek), NA, dayofweek %in% 1:5)
      ]
    } else {
      cli::cli_warn(
        c(
          "{.code weigh_dayofweek} is {.val TRUE}, but no {.col dayofweek} \\
            column in the data.",
          i = "Will ignore."
        )
      )
      weigh_dayofweek <- FALSE
    }
  }

  if (weigh_age) {
    survey <- weigh(survey, "part_age", target = survey_pop.full)
  }

  if (length(weights) > 0) {
    for (w in weights) {
      survey <- weigh(survey, w)
    }
  }

  # Post-stratification normalisation (with optional threshold)
  normalise_weights(
    survey$participants,
    by = "age.group",
    threshold = weight_threshold
  )

  ## merge participants and contacts into a single data table ------------------
  survey$contacts <- merge_participants_contacts(
    participants = survey$participants,
    contacts = survey$contacts
  )

  ## sample contacts randomly (if requested) -----------------------------------
  no_contact_ages <- nrow(survey$contacts[is.na(cnt_age)]) > 0
  if (missing_contact_age == "sample" && no_contact_ages) {
    survey$contacts <- impute_age_by_sample(survey$contacts)
  }

  max.age <- max_participant_age(survey$participants)

  ## add contact age groups
  survey$contacts <- add_contact_age_groups(
    contacts = survey$contacts,
    age_breaks = create_age_breaks(age_limits, max.age),
    age_groups = age_group_labels(survey$participants)
  )

  ## calculate weighted contact matrix -----------------------------------------
  sampled_contacts_participants <- sample_contacts_participants(
    sample_participants = sample_participants,
    participants = survey$participants,
    contacts = survey$contacts,
    age_limits = age_limits,
    sample_all_age_groups = sample_all_age_groups,
    max.tries = sample_participants_max_tries
  )

  weighted.matrix <- weighted_matrix_array(
    contacts = sampled_contacts_participants$sampled_contacts
  )

  if (!counts) {
    ## normalise to give mean number of contacts
    weighted.matrix <- normalise_weights_to_counts(
      sampled_participants = sampled_contacts_participants$sampled_participants,
      weighted_matrix = weighted.matrix
    )
  }

  warn_symmetric_counts_na(symmetric, counts, weighted.matrix)
  matrix_not_scalar <- prod(dim(as.matrix(weighted.matrix))) > 1
  na_in_weighted_mtx <- na_in_weighted_matrix(weighted.matrix)
  if (symmetric && matrix_not_scalar && !na_in_weighted_mtx) {
    weighted.matrix <- normalise_weighted_matrix(
      survey_pop = survey_pop,
      weighted_matrix = weighted.matrix,
      symmetric_norm_threshold = symmetric_norm_threshold
    )
  }

  ## Split contact matrix ------------------------------------------------------
  # do not return matrix with mean/norm/contacts if counts and split elected
  warn_if_counts_and_split(counts = counts, split = split)
  check_na_in_weighted_matrix(weighted_matrix = weighted.matrix, split = split)

  # make sure the dim.names are retained after symmetric or split procedure
  retained_dimnames <- dimnames(weighted.matrix)

  ret <- list()
  if (split && !counts && !na_in_weighted_matrix(weighted.matrix)) {
    splitted <- split_mean_norm_contacts(
      weighted_matrix = weighted.matrix,
      population = survey_pop$population
    )

    weighted.matrix <- splitted$weighted_matrix
    ret[["mean.contacts"]] <- splitted$mean_contacts
    ret[["normalisation"]] <- splitted$normalisation
    ret[["contacts"]] <- splitted$contacts
  }
  # make sure the dim.names are retained after symmetric or split procedure
  dimnames(weighted.matrix) <- retained_dimnames

  ret[["matrix"]] <- weighted.matrix

  ## Option to add matrix per capita -------------------------------------------
  # i.e., contact rate of age i with one individual of age j in the population.
  warn_counts_split_per_capita(
    counts = counts,
    split = split,
    per_capita = per_capita
  )
  if (per_capita && !counts && !split) {
    ret[["matrix.per.capita"]] <- matrix_per_capita(
      weighted_matrix = weighted.matrix,
      survey_pop = survey_pop
    )
  }

  if (need_survey_pop && is.null(survey_pop$survey.year)) {
    survey_pop[, year := survey.year]
    survey_pop <- merge(
      x = survey_pop,
      y = unique(survey$participants[, list(lower.age.limit, age.group)])
    )
    survey_pop <- survey_pop[, list(
      age.group,
      population,
      proportion = population / sum(population),
      year
    )]
  }

  ## get number of participants in each age group
  part.pop <- n_participants_per_age_group(survey$participants)

  if (need_survey_pop && (is.na(return_demography) || return_demography)) {
    # change survey_pop$age.group factors into characters (cfr. part.pop)
    survey_pop[, age.group := as.character(age.group)]
    ret[["demography"]] <- survey_pop[]
  }
  ret[["participants"]] <- part.pop[]

  # option to return participant weights ---------------------------------------
  if (return_part_weights) {
    # default
    part_weights <- survey$participants[, .N, by = list(age.group, weight)]
    part_weights <- part_weights[order(age.group, weight), ]

    # add age and/or dayofweek info
    if (weigh_age && weigh_dayofweek) {
      part_weights <- survey$participants[,
        .N,
        by = list(age.group, participant.age = part_age, is.weekday, weight)
      ]
    }

    if (weigh_age && !weigh_dayofweek) {
      part_weights <- survey$participants[,
        .N,
        by = list(age.group, participant.age = part_age, weight)
      ]
    }

    if (weigh_dayofweek && !weigh_age) {
      part_weights <- survey$participants[,
        .N,
        by = list(age.group, is.weekday, weight)
      ]
    }

    # order (from left to right)
    part_weights <- part_weights[order(part_weights), ] # nolint

    # set name of last column
    names(part_weights)[ncol(part_weights)] <- "participants"

    part_weights[, proportion := participants / sum(participants)]
    ret[["participants.weights"]] <- part_weights[]
  }

  ret
}
# nolint end

test_that("[.contact_survey filters by participant column", {
  uk <- polymod[country == "United Kingdom"]
  expect_s3_class(uk, "contact_survey")
  expect_true(all(uk$participants$country == "United Kingdom"))
  expect_gt(nrow(uk$participants), 0)
})

test_that("[.contact_survey prunes contacts when participants filtered", {
  uk <- polymod[country == "United Kingdom"]
  expect_true(all(
    uk$contacts$part_id %in% uk$participants$part_id
  ))
})

test_that("[.contact_survey filters by contact column", {
  young_contacts <- polymod[cnt_age_exact < 10]
  expect_s3_class(young_contacts, "contact_survey")
  expect_true(all(
    young_contacts$contacts$cnt_age_exact < 10,
    na.rm = TRUE
  ))
  # participants are kept when only contacts are filtered
  expect_identical(
    nrow(young_contacts$participants),
    nrow(polymod$participants)
  )
})

test_that("[.contact_survey keeps participants when only contacts filtered", {
  filtered <- polymod[cnt_age_exact < 5]
  expect_identical(
    nrow(filtered$participants),
    nrow(polymod$participants)
  )
})

test_that("[.contact_survey supports environment variables", {
  target_country <- "United Kingdom"
  uk <- polymod[country == target_country]
  expect_true(all(uk$participants$country == "United Kingdom"))
})

test_that("[.contact_survey returns contact_survey class", {
  result <- polymod[country == "United Kingdom"]
  expect_s3_class(result, "contact_survey")
  expect_named(result, c("participants", "contacts", "reference"))
})

test_that("[.contact_survey errors for cross-table expressions", {
  expect_error(
    polymod[country == "United Kingdom" & cnt_age_exact < 10],
    "both participants.*contacts"
  )
})

test_that("[.contact_survey allows filtering by part_id", {
  ids <- polymod$participants$part_id[1:5]
  result <- polymod[part_id %in% ids]
  expect_identical(nrow(result$participants), 5L)
  expect_true(all(result$contacts$part_id %in% ids))
})

test_that("[.contact_survey errors for numeric indexing", {
  expect_error(
    polymod[1:10],
    "Column-based expressions"
  )
})

test_that("[.contact_survey warns for unknown columns", {
  expect_warning(
    polymod[nonexistent_col == "foo"],
    "not found"
  )
})

test_that("[.contact_survey does not modify original", {
  original_nrow <- nrow(polymod$participants)
  uk <- polymod[country == "United Kingdom"]
  expect_identical(nrow(polymod$participants), original_nrow)
})

test_that("[.contact_survey with no filter returns copy", {
  result <- polymod[]
  expect_s3_class(result, "contact_survey")
  expect_identical(nrow(result$participants), nrow(polymod$participants))
  expect_false(identical(result$participants, polymod$participants))
})

test_that("[.contact_survey preserves extra fields", {
  survey <- polymod
  survey$observation_key <- c("wave", "studyDay")
  survey$custom_field <- "test_value"
  filtered <- survey[country == "United Kingdom"]
  expect_identical(filtered$observation_key, c("wave", "studyDay"))
  expect_identical(filtered$custom_field, "test_value")
})

---
title: "Introduction to socialmixr"
author: "Sebastian Funk"
date: "`r Sys.Date()`"
bibliography: socialmixr.bib
nocite: '@*'
link-citations: true
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Introduction to socialmixr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
data.table::setDTthreads(1)
```

[socialmixr](https://github.com/epiforecasts/socialmixr) is an `R` package to derive social mixing matrices from survey data. These are particularly useful for age-structured [infectious disease models](https://en.wikipedia.org/wiki/Mathematical_modelling_of_infectious_disease). For background on age-specific mixing matrices and what data inform them, see, for example, the paper on POLYMOD by [@mossong_social_2008].

# Setup

This vignette uses the POLYMOD survey data which is included in socialmixr, and ggplot2 for plotting. To download other surveys from the [Social contact data](https://zenodo.org/communities/social_contact_data) community on Zenodo, use the [contactsurveys](https://cran.r-project.org/package=contactsurveys) package.

```{r}
library(socialmixr)
library(ggplot2)
data(polymod)
```

# The pipeline workflow

`socialmixr` provides a small set of composable functions that each perform one step in turning a survey into a contact matrix:

1. `survey[expr]` -- filter participants or contacts with an expression.
2. `assign_age_groups()` -- impute missing ages and assign participants and contacts to age groups.
3. `weigh()` -- optionally attach participant weights (day of week, age, user-defined).
4. `compute_matrix()` -- compute the mean contact matrix.
5. `symmetrise()`, `split_matrix()`, `per_capita()` -- optional post-processing.

A minimal example extracting a matrix for the UK part of POLYMOD:

```{r}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix()
```

This produces a contact matrix with age groups 0-1, 1-5, 5-15 and 15+ years. It contains the mean number of contacts that each member of an age group (row) has reported with members of the same or another age group (column).

## Assigning age groups

`assign_age_groups()` prepares the survey for matrix computation. It imputes participant and contact ages from any available ranges, drops or keeps rows with missing ages (configurable via `missing_participant_age` and `missing_contact_age`), and adds `age.group` and `contact.age.group` columns using the `age_limits` you supply:

```{r}
uk_grouped <- polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15))

head(uk_grouped$participants[, c("part_id", "part_age", "age.group")])
head(uk_grouped$contacts[, c("part_id", "cnt_age", "contact.age.group")])
```

The resulting survey object can be inspected, subset, or passed through any number of `weigh()` calls before `compute_matrix()`. If no `age_limits` are supplied, age groups are inferred from participant and contact ages.

# Surveys

Some surveys contain data from multiple countries. The POLYMOD survey, for example, contains data from:

```{r}
unique(polymod$participants$country)
```

Use the subset method `[` on a survey to restrict to one or more countries or any other column in the participant or contact data:

```{r}
polymod[country %in% c("United Kingdom", "Germany")]
```

When participants are filtered, contacts are automatically pruned to matching participants. If this subsetting is not done, the different sub-surveys contained in a dataset are combined as if they were a single sample (i.e., not applying any population-weighting by country or other correction).

# Bootstrapping

To get an idea of the uncertainty in the contact matrices, participants can be resampled with replacement. A short helper replicates participant (and matching contact) rows for each occurrence of a resampled ID, so that duplicates are preserved:

```{r}
bootstrap <- function(survey) {
  sampled_ids <- sample(
    unique(survey$participants$part_id),
    replace = TRUE
  )
  survey$participants <- survey$participants[
    list(sampled_ids), on = "part_id"
  ]
  survey$contacts <- survey$contacts[
    list(sampled_ids),
    on = "part_id",
    nomatch = NULL,
    allow.cartesian = TRUE
  ]
  survey
}

uk <- polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15))

m <- suppressWarnings(
  replicate(n = 5, uk |> bootstrap() |> compute_matrix())
)
mr <- Reduce("+", lapply(m["matrix", ], function(x) x / ncol(m)))
mr
```

From these matrices, derived quantities can be obtained, for example the mean across samples as shown above.

# Demography

Obtaining symmetric contact matrices, splitting out their components (see below) and population-based participant weights require information about the underlying demographic composition of the survey population. This is represented as a `data.frame` with columns `lower.age.limit` (the lower end of each age group) and `population` (the number of people in that age group).

For recent UN World Population Prospects data, the `wpp2024` package is available from GitHub (`remotes::install_github("PPgp/wpp2024")`):

```{r eval=requireNamespace("wpp2024", quietly = TRUE), message=FALSE, warning=FALSE}
data("popAge1dt", package = "wpp2024")
uk_pop <- popAge1dt[name == "United Kingdom" & year == 2020,
  .(lower.age.limit = age, population = pop * 1000)
]
head(uk_pop)
```

Any comparable data frame will work, e.g. constructed by hand:

```{r}
custom_pop <- data.frame(
  lower.age.limit = c(0, 18, 60),
  population = c(12000000, 35000000, 20000000)
)
```

If the survey has a `country` column, `survey_country_population()` looks up country- and year-specific population data:

```{r}
survey_country_population(polymod, countries = "United Kingdom")
```

This uses the older [`wpp2017`](https://cran.r-project.org/package=wpp2017) package, an optional (Suggests) dependency that needs to be installed separately. It is kept for backwards compatibility; use more recent population data (as shown above) where possible.

# Symmetric contact matrices

Conceivably, contact matrices should be symmetric: the total number of contacts made by members of one age group with those of another should be the same as vice versa. Mathematically, if $m_{ij}$ is the mean number of contacts made by members of age group $i$ with members of age group $j$, and the total number of people in age group $i$ is $N_i$, then

$$m_{ij} N_i = m_{ji}N_j$$

Because of variation in the sample from which the contact matrix is obtained, this relationship is usually not fulfilled exactly. In order to obtain a symmetric contact matrix that fulfills it, one can use

$$m'_{ij} = \frac{1}{2N_i} (m_{ij} N_i + m_{ji} N_j)$$

To get this version of the contact matrix, pipe the matrix through `symmetrise()`, passing the population data:

```{r message=FALSE, warning=FALSE}
uk_pop <- survey_country_population(polymod, countries = "United Kingdom")

polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix() |>
  symmetrise(survey_pop = uk_pop)
```

# Contact rates per capita

The contact matrix per capita $c_{ij}$ contains the social contact rates of one individual of age $i$ with one individual of age $j$, given the population details. For example, $c_{ij}$ is used in infectious disease modelling to calculate the force of infection, which is based on the likelihood that one susceptible individual of age $i$ will be in contact with one infectious individual of age $j$. The contact rates per capita are calculated as follows:

$$c_{ij} =  \tfrac{m_{ij}}{N_{j}}$$

Pipe the matrix through `per_capita()` to convert to per-capita rates. If combined with `symmetrise()`, the contact matrix $m_{ij}$ can show asymmetry if the sub-population sizes are different, but the contact matrix per capita will be fully symmetric:

$$c'_{ij} = \frac{m_{ij} N_i + m_{ji} N_j}{2N_iN_j} = c'_{ji}$$

```{r message=FALSE, warning=FALSE}
de_pop <- survey_country_population(polymod, countries = "Germany")

polymod[country == "Germany"] |>
  assign_age_groups(age_limits = c(0, 60)) |>
  compute_matrix() |>
  symmetrise(survey_pop = de_pop) |>
  per_capita(survey_pop = de_pop)
```

# Splitting contact matrices

`split_matrix()` decomposes the contact matrix into a _global_ component as well as components representing _contacts_, _assortativity_ and _demography_. The elements $m_{ij}$ of the contact matrix are modelled as

$$ m_{ij} = c q d_i a_{ij} n_j $$

where $c$ is the mean number of contacts across the whole population, $c q d_i$ is the number of contacts that a member of group $i$ makes across age groups, and $n_j$ is the proportion of the surveyed population in age group $j$. The constant $q$ is set so that $c q$ is equal to the value of the largest eigenvalue of $m_{ij}$; if used in an infectious disease model and assumed that every contact leads to infection, $c q$ can be replaced by the basic reproduction number $R_0$.

`split_matrix()` returns the assortativity matrix $a_{ij}$ in `$matrix`, with additional components `$mean.contacts` ($c$), `$normalisation` ($q$) and `$contacts` ($d_i$).

```{r}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix() |>
  split_matrix(survey_pop = uk_pop)
```

# Filtering

The `[` method can be used to select particular participants or contacts. For example, in the `polymod` dataset, the indicators `cnt_home`, `cnt_work`, `cnt_school`, `cnt_transport`, `cnt_leisure` and `cnt_otherplace` take value 0 or 1 depending on where a contact occurred. The filter is evaluated against whichever table contains the referenced columns (participants, contacts, or both). Multiple filters can be chained:

```{r, warning=FALSE, message=FALSE}
# contact matrix for school-related contacts
polymod[cnt_school == 1] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

# contact matrix for work-related contacts involving physical contact
polymod[cnt_work == 1][phys_contact == 1] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

# contact matrix for daily contacts at home with males
polymod[cnt_home == 1][cnt_gender == "M"][duration_multi == 5] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()
```

# Participant weights

## Temporal aspects and demography

Participant weights are commonly used to align sample and population characteristics in terms of temporal aspects and the age distribution. For example, the day of the week has been reported as a driving factor for social contact behaviour, hence to obtain a weekly average, the survey data should represent the weekly 2/5 distribution of weekend/week days. To align the survey data to this distribution, one can obtain participant weights in the form of:
$$w_{\textrm{day.of.week}} = \tfrac{5/7}{N_{\textrm{weekday}}/N} \text{  OR   } \tfrac{2/7}{N_{\textrm{weekend}}/N}$$
with sample size $N$, and $N_{weekday}$ and $N_{weekend}$ the number of participants that were surveyed during weekdays and weekend days, respectively.

Another driver of social contact patterns is age. To improve the representativeness of survey data, age-specific weights can be calculated as:
$$w_{age} = \tfrac{P_{a}\ /\ P}{N_{a}\ /\ N}$$
with $P$ the population size, $P_a$ the population fraction of age $a$, $N$ the survey sample size and $N_a$ the survey fraction of age $a$. The combination of age-specific and temporal weights for participant $i$ of age $a$ can be constructed as:
$$w_{i} = w_{\textrm{age}} * w_{\textrm{day.of.week}} $$

If the social contact analysis is based on stratification by splitting the population into non-overlapping groups, it requires the weights to be standardised so that the weighted totals within mutually exclusive cells equal the known population totals [@kolenikov_post-stratification_2016]. The post-stratification cells need to be mutually exclusive and cover the whole population. `compute_matrix()` applies this post-stratification normalisation within age groups.

`weigh()` is composable: each call multiplies new weights into the participants' `weight` column. In POLYMOD, the `dayofweek` column uses 0 for Sunday and 6 for Saturday, so weekdays are 1-5 and weekend days are 0 and 6:

```{r message=FALSE, warning=FALSE}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
  weigh("part_age", target = uk_pop) |>
  compute_matrix()
```

The first `weigh()` call assigns weekday participants a total weight of 5 and weekend participants a total weight of 2 (the weekly 5/2 split). The second call post-stratifies against the population structure in `uk_pop` (passed as a data frame).

## User-defined participant weights

`weigh()` with no `target` multiplies an existing participant column directly into the weight. For instance, to give more importance to participants from large households:

```{r message=FALSE, warning=FALSE}
polymod |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("hh_size") |>
  compute_matrix()
```

## Weight threshold

If the survey population differs extensively from the demography, some participants can end up with relatively high weights and as such, an excessive contribution to the population average. This warrants the limitation of single participant influences by a truncation of the weights. `compute_matrix()` accepts a numeric `weight_threshold` which caps the standardised weights and re-normalises so that the weight sum equals the group size. Weights close to the threshold may slightly exceed it after re-normalisation.

```{r message=FALSE, warning=FALSE}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
  weigh("part_age", target = uk_pop) |>
  compute_matrix(weight_threshold = 3)
```


## Numerical example

With these numeric examples, we show the importance of post-stratification weights in contrast to using the crude weights directly within age-groups. We will apply the weights by age and day of week separately in these examples, though the combination is straightforward via multiplication.

### Get survey data
We start from a survey including 6 participants of 1, 2 and 3 years of age. The ages are not equally represented in the sample, though we assume they are equally present in the reference population. We will calculate the weighted average number of contacts by age and by age group, using {1,2} and {3} years of age. The following table shows the reported number of contacts per participant $i$, represented by $m_i$:

```{r, echo=FALSE}
survey_data <- data.frame(
  age = c(1, 1, 2, 2, 2, 3),
  day.of.week = as.factor(c(
    "weekend", "weekend", "weekend",
    "week", "week", "week"
  )),
  age.group = NA,
  m_i = c(3, 2, 9, 10, 8, 15)
)

# age groups 1-2 and 3
survey_data$age.group <- 1 - (survey_data$age < 3) + 1
survey_data$age.group <- as.factor(c("A", "B"))[survey_data$age.group]

knitr::kable(survey_data)
```

The summary statistics for the sample (N) and reference population (P) are as follows

```{r}
N <- 6
N_age <- c(2, 3, 1)
N_age.group <- c(5, 1)
N_day.of.week <- c(3, 3)

P <- 3000
P_age <- c(1000, 1000, 1000)
P_age.group <- c(2000, 1000)

P_day.of.week <- c(5 / 7, 2 / 7) * 3000
```

This survey data results in an unweighted average number of contacts:

```{r, echo=FALSE}
cat(paste(
  "unweighted average number of contacts:",
  round(mean(survey_data$m_i), digits = 2)
))
```

and age-specific unweighted averages on the number of contacts:
```{r, echo=FALSE}
knitr::kable(aggregate(
  m_i ~ age + age.group, data = survey_data, mean
))
```

### Weight by day of week

The following table contains the participants weights based on the survey day with and without the population and sample size constants ($w$ and $w'$, respectively). Note that the standardised weights $\tilde{w}$ and $\tilde{w'}$ are the same:

```{r, echo=FALSE}
# including population constants
survey_data$w <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w[i] <- (P_day.of.week[day_i] / P) / (N_day.of.week[day_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# without population constants
survey_data$w_dot <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w_dot[i] <- (P_day.of.week[day_i]) / (N_day.of.week[day_i])
}
survey_data$w_dot_tilde <- survey_data$w_dot / sum(survey_data$w_dot) * N

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)

# print
knitr::kable(survey_data)

# remove the 'dot' weights
survey_data$w_dot <- NULL
survey_data$w_dot_tilde <- NULL
```

Note the different scale of $w$ and $w'$, and the more straightforward interpretation of the numerical value of $w$ in terms of relative differences to apply truncation.
Using the standardised weights, we are able to calculate the weighted number of contacts:

```{r, echo=FALSE}
# add weighted number of contacts
survey_data["m_i * w_tilde"] <- survey_data$m_i * survey_data$w_tilde

# show table
knitr::kable(survey_data)

# remove the weighted number of contacts
survey_data$`m_i * w_tilde` <- NULL

cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

If the population-based weights are directly used in age-specific groups, the contact behaviour of the 3 year-old participant, which participated during week day, is inflated due to the under-representation of week days in the survey sample. In addition, the number of contacts for 1 year-old participants is decreased because of the over-representation of weekend days in the survey. Using the population-weights within the two aggregated age groups, we obtain a more intuitive weighting for age group A, but it is still skewed for individuals in age group B. As such, this weighted average for age group B has no meaning in terms of social contact behaviour:

```{r, echo=FALSE}
knitr::kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))
```

If we subdivide the population, we need to use post-stratification weights ("w_PS") in which the weighted totals within mutually exclusive cells equal the sample size. For the age groups, this goes as follows:

```{r, echo=FALSE}
survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] /
    sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)

knitr::kable(survey_data)
```

The weighted means equal:

```{r, echo=FALSE}
knitr::kable(aggregate(
  m_i * w_PS ~ age.group,
  data = survey_data, mean
))
```

### Weight by age

We repeated the example by calculating age-specific participant weights on the population and age-group level:

```{r, echo=FALSE}
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] /
    sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)

# print
knitr::kable(survey_data)

cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

If the age-specific weights are directly used within the age groups, the contact behaviour for age group B is inflated to unrealistic levels and the number of contacts for age group A is artificially low:

```{r, echo=FALSE}
knitr::kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))
```

Using the post-stratification weights, we end up with:

```{r, echo=FALSE}
knitr::kable(aggregate(m_i * w_PS ~ age.group, data = survey_data, mean))
```


### Apply threshold

We start with survey data of 14 participants of 1, 2 and 3 years of age, sampled from a population in which all ages are equally present. Given the high representation of participants aged 1 year, the age-specific proportions are skewed in comparison with the reference population. If we calculate the age-specific weights and (un)weighted average number of contacts, we end up with:

```{r, echo=FALSE}
survey_data <- survey_data[c(rep(1:2, 5), 3:6), ]
survey_data$m_i[survey_data$age == 3] <- 30
rownames(survey_data) <- NULL

survey_data <- survey_data[order(survey_data$age), ]
survey_data$w <- NULL
survey_data$w_tilde <- NULL
survey_data$w_PS <- NULL
```


```{r, echo=FALSE}
N <- nrow(survey_data)
N_age <- table(survey_data$age)
N_age.group <- table(survey_data$age.group)
N_day.of.week <- table(survey_data$day.of.week)
```

```{r, echo=FALSE}
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)
knitr::kable(survey_data)

cat(paste(
  "unweighted average number of contacts:",
  round(mean(survey_data$m_i), digits = 2)
))
cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

The single participant of 3 years of age has a very large influence on the weighted population average. As such, we propose to truncate the relative age-specific weights $w$ at 3. As such, the weighted population average equals:

```{r, echo=FALSE}
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3
```

```{r, echo=FALSE}
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3

cat(paste(
  "weighted average number of contacts after truncation:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

# Plotting

## Using ggplot2

The contact matrices can be plotted by using the `geom_tile()` function of the `ggplot2` package.

```{r fig.width=4, fig.height=4}
df <- reshape2::melt(
  mr,
  varnames = c("age.group", "age.group.contact"),
  value.name = "contacts"
)
ggplot(df, aes(x = age.group, y = age.group.contact, fill = contacts)) +
  theme(legend.position = "bottom") +
  geom_tile()
```

## Using R base

The contact matrices can also be plotted with the `matrix_plot()` function as a grid of coloured rectangles with the numeric values in the cells. Heat colours are used by default, though this can be changed.

```{r fig.width=4, fig.height=4}
matrix_plot(mr)
matrix_plot(mr, color.palette = gray.colors)
```

# References

---
title: "The socialmixr package"
author: "Sebastian Funk"
institute: https://epiforecasts.io
date: "`r Sys.Date()`"
---

```{r setup, include = FALSE}
library(knitr)
library(socialmixr)
library(ggplot2)
library(data.table)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
data.table::setDTthreads(1)
```

[socialmixr](https://github.com/epiforecasts/socialmixr) is an `R` package to derive social mixing matrices from survey data. These are particularly useful for age-structured [infectious disease models](https://en.wikipedia.org/wiki/Mathematical_modelling_of_infectious_disease). For background on age-specific mixing matrices and what data inform them, see, for example, the paper on POLYMOD by [@mossong_social_2008].

# The pipeline workflow

Contact matrices are computed through a small pipeline of composable functions:

1. `survey[expr]` -- filter participants or contacts.
2. `assign_age_groups()` -- impute missing ages and assign age groups.
3. `weigh()` -- optionally apply participant weights.
4. `compute_matrix()` -- compute the mean contact matrix.
5. `symmetrise()`, `split_matrix()`, `per_capita()` -- optional post-processing.

Each step is a separate function.

The POLYMOD data are included with the package and can be loaded using

```{r}
data(polymod)
```

An example use would be

```{r}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix()
```

This generates a contact matrix from the UK part of the POLYMOD study, with age groups 0-1, 1-5, 5-15 and 15+ years. It contains the mean number of contacts that each member of an age group (row) has reported with members of the same or another age group (column).

## Assigning age groups

`assign_age_groups()` prepares the survey for matrix computation. It imputes participant and contact ages from any available ranges, drops or keeps rows with missing ages (configurable via `missing_participant_age` and `missing_contact_age`), and adds `age.group` and `contact.age.group` columns using the `age_limits` you supply:

```{r}
uk_grouped <- polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15))

head(uk_grouped$participants[, c("part_id", "part_age", "age.group")])
head(uk_grouped$contacts[, c("part_id", "cnt_age", "contact.age.group")])
```

The resulting survey object can be inspected, subset, or passed through any number of `weigh()` calls before `compute_matrix()`.

# Surveys

The key argument to the pipeline functions is the `survey` they operate on. The `socialmixr` package includes the POLYMOD survey. It also provides access to all surveys in the [Social contact data](https://zenodo.org/communities/social_contact_data) community on [Zenodo](https://zenodo.org). The available surveys can be listed (if an internet connection is available) with

```{r eval=FALSE}
list_surveys()
```

A survey can be downloaded using the `get_survey()` command. This will get the relevant data of a survey given its Zenodo DOI (as returned by `list_surveys()`). If a survey is to be used repeatedly it is worth downloading it and storing it locally to avoid the need for a network connection and speed up processing.

```{r eval=FALSE}
peru_survey <- get_survey("https://doi.org/10.5281/zenodo.1095664")
saveRDS(peru_survey, "peru.rds")
```

This way, the `peru` data set can be loaded in the future without the need for an internet connection using

```{r eval=FALSE}
peru_survey <- readRDS("peru.rds")
```

Some surveys may contain data from multiple countries. To check this, use the `survey_countries` function

```{r}
survey_countries(polymod)
```

Use the subset method `[` to restrict to one or more countries:

```{r eval=FALSE}
polymod[country %in% c("United Kingdom", "Germany")]
```

If this is not done, the different sub-surveys contained in a dataset are combined as a single sample (without country-specific weighting).


A reference for any given survey can be obtained using `get_citation()`, e.g.

```{r}
get_citation(polymod)
```

# Bootstrapping

To get an idea of the uncertainty in the contact matrices, participants can be resampled with replacement. A short helper replicates participant (and matching contact) rows for each occurrence of a resampled ID, so that duplicates are preserved:

```{r}
bootstrap <- function(survey) {
  sampled_ids <- sample(
    unique(survey$participants$part_id),
    replace = TRUE
  )
  survey$participants <- survey$participants[
    list(sampled_ids), on = "part_id"
  ]
  survey$contacts <- survey$contacts[
    list(sampled_ids),
    on = "part_id",
    nomatch = NULL,
    allow.cartesian = TRUE
  ]
  survey
}

uk <- polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15))

m <- suppressWarnings(
  replicate(n = 5, uk |> bootstrap() |> compute_matrix())
)
mr <- Reduce("+", lapply(m["matrix", ], function(x) x / ncol(m)))
mr
```

# Demography

Obtaining symmetric contact matrices, splitting out their components (see below) and population-based participant weights require information about the underlying demographic composition of the survey population. This is represented as a `data.frame` with columns `lower.age.limit` (the lower end of each age group) and `population` (the number of people in that age group).

## Using custom population data

You can construct any comparable data frame and pass it to the relevant post-processing step:

```{r message=FALSE, warning=FALSE}
custom_pop <- data.frame(
  lower.age.limit = c(0, 18, 60),
  population = c(12000000, 35000000, 20000000)
)
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  compute_matrix() |>
  symmetrise(survey_pop = custom_pop)
```

For recent UN World Population Prospects data, the `wpp2024` package is available from GitHub (`remotes::install_github("PPgp/wpp2024")`):

```{r eval=requireNamespace("wpp2024", quietly = TRUE), message=FALSE, warning=FALSE}
data("popAge1dt", package = "wpp2024")
uk_pop <- popAge1dt[name == "United Kingdom" & year == 2020,
  .(lower.age.limit = age, population = pop * 1000)
]
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  compute_matrix() |>
  symmetrise(survey_pop = uk_pop)
```

`survey_country_population()` looks up country-specific population data from the [`wpp2017`](https://cran.r-project.org/package=wpp2017) package, which is an optional (Suggests) dependency of socialmixr; install it separately to use this function. Passing population data directly (as above) is preferred where possible:

```{r}
uk_pop_default <- survey_country_population(
  polymod, countries = "United Kingdom"
)
head(uk_pop_default)
```


# Symmetric contact matrices

Conceivably, contact matrices should be symmetric: the total number of contacts made by members of one age group with those of another should be the same as vice versa. Mathematically, if $m_{ij}$ is the mean number of contacts made by members of age group $i$ with members of age group $j$, and the total number of people in age group $i$ is $N_i$, then

$$m_{ij} N_i = m_{ji}N_j$$

Because of variation in the sample from which the contact matrix is obtained, this relationship is usually not fulfilled exactly. In order to obtain a symmetric contact matrix that fulfills it, one can use

$$m'_{ij} = \frac{1}{2N_i} (m_{ij} N_i + m_{ji} N_j)$$

To get this version of the contact matrix, pipe the matrix through `symmetrise()`, passing the population data:

```{r}
uk_pop <- survey_country_population(polymod, countries = "United Kingdom")

polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix() |>
  symmetrise(survey_pop = uk_pop)
```

# Contact rates per capita

The contact matrix per capita $c_{ij}$ contains the social contact rates of one individual of age $i$ with one individual of age $j$, given the population details. For example, $c_{ij}$ is used in infectious disease modelling to calculate the force of infection, which is based on the likelihood that one susceptible individual of age $i$ will be in contact with one infectious individual of age $j$. The contact rates per capita are calculated as follows:

$$c_{ij} =  \tfrac{m_{ij}}{N_{j}}$$

Pipe the matrix through `per_capita()`. If combined with `symmetrise()`, the contact matrix $m_{ij}$ can show asymmetry if the sub-population sizes are different, but the contact matrix per capita will be fully symmetric:

$$c'_{ij} = \frac{m_{ij} N_i + m_{ji} N_j}{2N_iN_j} = c'_{ji}$$


```{r message=FALSE, warning=FALSE}
de_pop <- survey_country_population(polymod, countries = "Germany")

polymod[country == "Germany"] |>
  assign_age_groups(age_limits = c(0, 60)) |>
  compute_matrix() |>
  symmetrise(survey_pop = de_pop) |>
  per_capita(survey_pop = de_pop)
```

# Splitting contact matrices

`split_matrix()` decomposes the contact matrix into a _global_ component as well as three components representing _contacts_, _assortativity_ and _demography_. In other words, the elements $m_{ij}$ of the contact matrix are modelled as

$$ m_{ij} = c q d_i a_{ij} n_j $$

where $c$ is the mean number of contacts across the whole population, $c q d_i$ is the number of contacts that a member of group $i$ makes across age groups, $n_j$ is the proportion of the surveyed population in age group $j$. The constant $q$ is set so that $c q$ is equal to the value of the largest eigenvalue of $m_{ij}$; if used in an infectious disease model and assumed that every contact leads to infection, $c q$ can be replaced by the basic reproduction number $R_0$.

`split_matrix()` returns the assortativity matrix $a_{ij}$ in `$matrix`, with additional components `$mean.contacts` ($c$), `$normalisation` ($q$) and `$contacts` ($d_i$).

```{r}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 1, 5, 15)) |>
  compute_matrix() |>
  split_matrix(survey_pop = uk_pop)
```

# Filtering

The `[` method can be used to select particular participants or contacts. For example, in the `polymod` dataset, the indicators `cnt_home`, `cnt_work`, `cnt_school`, `cnt_transport`, `cnt_leisure` and `cnt_otherplace` take value 0 or 1 depending on where a contact occurred. The filter is evaluated against whichever table contains the referenced columns (participants, contacts, or both). Multiple filters can be chained:

```{r, warning=FALSE, message=FALSE}
# contact matrix for school-related contacts
polymod[cnt_school == 1] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

# contact matrix for work-related contacts involving physical contact
polymod[cnt_work == 1][phys_contact == 1] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()

# contact matrix for daily contacts at home with males
polymod[cnt_home == 1][cnt_gender == "M"][duration_multi == 5] |>
  assign_age_groups(age_limits = c(0, 20, 60)) |>
  compute_matrix()
```

# Participant weights

## Temporal aspects and demography

Participant weights are commonly used to align sample and population characteristics in terms of temporal aspects and the age distribution. For example, the day of the week has been reported as a driving factor for social contact behaviour, hence to obtain a weekly average, the survey data should represent the weekly 2/5 distribution of weekend/week days. To align the survey data to this distribution, one can obtain participant weights in the form of:
$$w_{\textrm{day.of.week}} = \tfrac{5/7}{N_{\textrm{weekday}}/N} \text{  OR   } \tfrac{2/7}{N_{\textrm{weekend}}/N}$$
with sample size $N$, and $N_{weekday}$ and $N_{weekend}$ the number of participants that were surveyed during weekdays and weekend days, respectively.

Another driver of social contact patterns is age. To improve the representativeness of survey data, age-specific weights can be calculated as:
$$w_{age} = \tfrac{P_{a}\ /\ P}{N_{a}\ /\ N}$$
with $P$ the population size, $P_a$ the population fraction of age $a$, $N$ the survey sample size and $N_a$ the survey fraction of age $a$. The combination of age-specific and temporal weights for participant $i$ of age $a$ can be constructed as:
$$w_{i} = w_{\textrm{age}} * w_{\textrm{day.of.week}} $$

If the social contact analysis is based on stratification by splitting the population into non-overlapping groups, it requires the weights to be standardised so that the weighted totals within mutually exclusive cells equal the known population totals [@kolenikov_post-stratification_2016]. `compute_matrix()` applies this post-stratification normalisation within age groups.

`weigh()` is composable: each call multiplies new weights into the participants' `weight` column. In POLYMOD, the `dayofweek` column uses 0 for Sunday and 6 for Saturday, so weekdays are 1-5 and weekend days are 0 and 6:

```{r message=FALSE, warning=FALSE}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
  weigh("part_age", target = uk_pop) |>
  compute_matrix()
```

The first `weigh()` call assigns weekday participants a total weight of 5 and weekend participants a total weight of 2 (the weekly 5/2 split). The second call post-stratifies against the population structure in `uk_pop`.

## User-defined participant weights

`weigh()` with no `target` multiplies an existing participant column directly into the weight. For instance, to give more importance to participants from large households:

```{r message=FALSE, warning=FALSE}
polymod |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("hh_size") |>
  compute_matrix()
```


## Weight threshold

If the survey population differs extensively from the demography, some participants can end up with relatively high weights and as such, an excessive contribution to the population average. This warrants the limitation of single participant influences by a truncation of the weights. `compute_matrix()` accepts a numeric `weight_threshold` which caps the standardised weights and re-normalises so that the weight sum equals the group size. Weights close to the threshold may slightly exceed it after re-normalisation.

```{r message=FALSE, warning=FALSE}
polymod[country == "United Kingdom"] |>
  assign_age_groups(age_limits = c(0, 18, 60)) |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6))) |>
  weigh("part_age", target = uk_pop) |>
  compute_matrix(weight_threshold = 3)
```


## Numerical example

With these numeric examples, we show the importance of post-stratification weights in contrast to using the crude weights directly within age-groups. We will apply the weights by age and day of week separately in these examples, though the combination is straightforward via multiplication.

### Get survey data
We start from a survey including 6 participants of 1, 2 and 3 years of age. The ages are not equally represented in the sample, though we assume they are equally present in the reference population. We will calculate the weighted average number of contacts by age and by age group, using {1,2} and {3} years of age. The following table shows the reported number of contacts per participant $i$, represented by $m_i$:

```{r, echo=FALSE}
survey_data <- data.frame(
  age = c(1, 1, 2, 2, 2, 3),
  day.of.week = as.factor(c(
    "weekend", "weekend", "weekend",
    "week", "week", "week"
  )),
  age.group = NA,
  m_i = c(3, 2, 9, 10, 8, 15)
)

# age groups 1-2 and 3
survey_data$age.group <- 1 - (survey_data$age < 3) + 1
survey_data$age.group <- as.factor(c("A", "B"))[survey_data$age.group]

kable(survey_data)
```

The summary statistics for the sample (N) and reference population (P) are as follows

```{r}
N <- 6
N_age <- c(2, 3, 1)
N_age.group <- c(5, 1)
N_day.of.week <- c(3, 3)

P <- 3000
P_age <- c(1000, 1000, 1000)
P_age.group <- c(2000, 1000)

P_day.of.week <- c(5 / 7, 2 / 7) * 3000
```

This survey data results in an unweighted average number of contacts:

```{r, echo=FALSE}
cat(paste(
  "unweighted average number of contacts:",
  round(mean(survey_data$m_i), digits = 2)
))
```

and age-specific unweighted averages on the number of contacts:
```{r, echo=FALSE}
kable(aggregate(
  m_i ~ age + age.group, data = survey_data, mean
))
```

### Weight by day of week

The following table contains the participants weights based on the survey day with and without the population and sample size constants ($w$ and $w'$, respectively). Note that the standardised weights $\tilde{w}$ and $\tilde{w'}$ are the same:

```{r, echo=FALSE}
# including population constants
survey_data$w <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w[i] <- (P_day.of.week[day_i] / P) / (N_day.of.week[day_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# without population constants
survey_data$w_dot <- NA
for (i in seq_len(nrow(survey_data))) {
  day_i <- survey_data$day.of.week[i]
  survey_data$w_dot[i] <- (P_day.of.week[day_i]) / (N_day.of.week[day_i])
}
survey_data$w_dot_tilde <- survey_data$w_dot / sum(survey_data$w_dot) * N

# round
survey_data[, -(1:4)] <- round(survey_data[, -(1:4)], digits = 2)

# print
kable(survey_data)

# remove the 'dot' weights
survey_data$w_dot <- NULL
survey_data$w_dot_tilde <- NULL
```

Note the different scale of $w$ and $w'$, and the more straightforward interpretation of the numerical value of $w$ in terms of relative differences to apply truncation.
Using the standardised weights, we are able to calculate the weighted number of contacts:

```{r, echo=FALSE}
# add weighted number of contacts
survey_data["m_i * w_tilde"] <- survey_data$m_i * survey_data$w_tilde

# show table
kable(survey_data)

# remove the weighted number of contacts
survey_data$`m_i * w_tilde` <- NULL

cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

If the population-based weights are directly used in age-specific groups, the contact behaviour of the 3 year-old participant, which participated during week day, is inflated due to the under-representation of week days in the survey sample. In addition, the number of contacts for 1 year-old participants is decreased because of the over-representation of weekend days in the survey. Using the population-weights within the two aggregated age groups, we obtain a more intuitive weighting for age group A, but it is still skewed for individuals in age group B. As such, this weighted average for age group B has no meaning in terms of social contact behaviour:

```{r, echo=FALSE}
kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))
```

If we subdivide the population, we need to use post-stratification weights ("w_PS") in which the weighted totals within mutually exclusive cells equal the sample size. For the age groups, this goes as follows:

```{r, echo=FALSE}
survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] /
    sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)

kable(survey_data)
```

The weighted means equal:

```{r, echo=FALSE}
kable(aggregate(m_i * w_PS ~ age.group, data = survey_data, mean))
```

### Weight by age

We repeated the example by calculating age-specific participant weights on the population and age-group level:

```{r, echo=FALSE}
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

survey_data$w_PS <- NA
for (i in seq_len(nrow(survey_data))) {
  k_i <- survey_data$age.group[i]
  flag_k <- survey_data$age.group == k_i
  survey_data$w_PS[i] <- survey_data$w[i] /
    sum(survey_data$w[flag_k]) * N_age.group[k_i]
}

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)

# print
kable(survey_data)

cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

If the age-specific weights are directly used within the age groups, the contact behaviour for age group B is inflated to unrealistic levels and the number of contacts for age group A is artificially low:

```{r, echo=FALSE}
kable(list(
  aggregate(m_i * w_tilde ~ age, data = survey_data, mean),
  aggregate(m_i * w_tilde ~ age.group, data = survey_data, mean)
))
```

Using the post-stratification weights, we end up with:

```{r, echo=FALSE}
kable(aggregate(
  m_i * w_PS ~ age.group,
  data = survey_data, mean
))
```


### Apply threshold

We start with survey data of 14 participants of 1, 2 and 3 years of age, sampled from a population in which all ages are equally present. Given the high representation of participants aged 1 year, the age-specific proportions are skewed in comparison with the reference population. If we calculate the age-specific weights and (un)weighted average number of contacts, we end up with:

```{r, echo=FALSE}
survey_data <- survey_data[c(rep(1:2, 5), 3:6), ]
survey_data$m_i[survey_data$age == 3] <- 30
rownames(survey_data) <- NULL

survey_data <- survey_data[order(survey_data$age), ]
survey_data$w <- NULL
survey_data$w_tilde <- NULL
survey_data$w_PS <- NULL
```


```{r, echo=FALSE}
N <- nrow(survey_data)
N_age <- table(survey_data$age)
N_age.group <- table(survey_data$age.group)
N_day.of.week <- table(survey_data$day.of.week)
```

```{r, echo=FALSE}
survey_data$w <- NA
survey_data$w_tilde <- NA
survey_data$w_PS <- NULL
for (i in seq_len(nrow(survey_data))) {
  age_i <- survey_data$age[i]
  survey_data$w[i] <- (P_age[age_i] / P) / (N_age[age_i] / N)
}
survey_data$w_tilde <- survey_data$w / sum(survey_data$w) * N

# round
survey_data[, -(1:4)] <- round(
  survey_data[, -(1:4)], digits = 2
)
kable(survey_data)

cat(paste(
  "unweighted average number of contacts:",
  round(mean(survey_data$m_i), digits = 2)
))
cat(paste(
  "weighted average number of contacts:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

The single participant of 3 years of age has a very large influence on the weighted population average. As such, we propose to truncate the relative age-specific weights $w$ at 3. As such, the weighted population average equals:

```{r, echo=FALSE}
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3
```

```{r, echo=FALSE}
survey_data$w_tilde[survey_data$w_tilde > 3] <- 3

cat(paste(
  "weighted average number of contacts after truncation:",
  round(
    mean(survey_data$m_i * survey_data$w_tilde),
    digits = 2
  )
))
```

# Plotting

## Using ggplot2

The contact matrices can be plotted by using the `geom_tile()` function of the `ggplot2` package.

```{r fig.width=4, fig.height=4}
df <- reshape2::melt(
  mr,
  varnames = c("age.group", "age.group.contact"),
  value.name = "contacts"
)
ggplot(df, aes(x = age.group, y = age.group.contact, fill = contacts)) +
  theme(legend.position = "bottom") +
  geom_tile()
```

## Using R base

The contact matrices can also be plotted with the `matrix_plot()` function as a grid of coloured rectangles with the numeric values in the cells. Heat colours are used by default, though this can be changed.

```{r fig.width=4, fig.height=4}
matrix_plot(mr)
matrix_plot(mr, color.palette = gray.colors)
```

# References

# list_survey() gives deprecation warning

    `list_surveys()` was deprecated in socialmixr 0.5.0.
    i Please use `contactsurveys::list_surveys()` instead.

# get_citation() gives deprecation warning

    `get_citation()` was deprecated in socialmixr 0.5.0.
    i Please use `contactsurveys::get_citation()` instead.

# missing surveys can't be cited

    Code
      get_citation("bogus")
    Condition
      Error in `download_survey()`:
      ! `survey` is not a DOI or URL.

# multiple DOI's cannot be loaded

    Code
      suppressMessages(suppressWarnings(get_survey(c("10.5281/zenodo.1095664",
        "10.5281/zenodo.1127693"))))
    Condition
      Error in `download_survey()`:
      ! `survey` must be a single, non-NA character string.

# survey_countries() sends deprecation warning

    `survey_countries()` was deprecated in socialmixr 0.5.0.
    i Please use `contactsurveys::download_survey()` instead.
    i We recommend using contactsurveys::download_survey() to download your surveys, and then you can load them with socialmixr::load_survey() and explore which countries are in the data.

# contact matrices look as expected

    Code
      contacts
    Output
      $test1
      $test1$matrix
                contact.age.group
      age.group       [0,5)     [5,10)    [10,15)    [15,20)     [20,25)    [25,30)
        [0,5)    178.966087  88.880184  45.354727  30.972284  42.6051270  74.733934
        [5,10)    66.232016 668.044656 134.274185  35.203096  31.4355346  66.434003
        [10,15)   42.023723 111.436616 688.396514 103.714503  22.1280347  44.956109
        [15,20)   24.673006  78.346794 158.510591 700.280260  97.5761783  77.930858
        [20,25)   26.973497  35.707872  15.991597  93.248381 152.7117404  76.040717
        [25,30)   43.200207  43.480995  18.063760  42.873620  88.9892562 107.950930
        [30,35)   40.425052  56.416277  28.910736  24.861596  45.4570091  57.681248
        [35,40)   57.413396  96.817442  53.334831  39.057969  44.1828453  49.687374
        [40,45)   14.841394  55.860697  61.956371  53.157270  47.6076053  45.907692
        [45,50)   12.023640   8.993476  37.232448  63.569833  48.1737784  46.208130
        [50,55)   24.897017  19.732155  21.398703  46.303262  58.2690366  58.172490
        [55,60)   11.463500  12.454714  24.118950  16.234386  33.0913231  46.499652
        [60,65)   13.095699  32.832242  17.740504  13.131377  35.0665384  43.906001
        [65,70)    7.009383  13.174960  11.319417  13.157194  10.0526235  19.717793
        [70,75)    3.046779   2.949233   8.945245  15.924334   9.6280647   9.140336
        [75,80)    0.000000   2.137842   1.068921   3.110274   0.9724316   1.068921
        [80,Inf)   0.000000   0.000000   0.000000   0.000000   0.0000000   0.000000
                contact.age.group
      age.group     [30,35)    [35,40)    [40,45)     [45,50)   [50,55)   [55,60)
        [0,5)     91.708976  95.882802  51.956159  27.5221001 30.779454 29.361522
        [5,10)   108.803439 100.314642 102.291590  54.9530408 38.811919 20.808874
        [10,15)   63.258912 127.846712 115.751398  58.3753992 40.509099 25.514605
        [15,20)   52.068678 113.778963  98.589173  81.3480282 42.672234 34.694509
        [20,25)   51.094350  44.650806  43.261966  57.6368055 25.722105 23.020902
        [25,30)   69.506069  55.360694  51.309724  54.4364839 49.972141 30.754586
        [30,35)   99.809224  91.659424  48.732356  34.1893655 35.866531 40.608316
        [35,40)   61.866181 104.774182  86.462595  55.7946573 42.695162 37.111034
        [40,45)   61.971992  81.658609  84.170188  82.6206655 43.740523 33.866735
        [45,50)   49.432321  59.367926  69.586529 102.0096136 51.387754 27.667104
        [50,55)   61.170361  55.261460  58.258871  39.5785652 48.739562 54.806600
        [55,60)   33.424551  36.774400  36.345039  42.6153774 56.419796 62.962663
        [60,65)   49.981599  66.335051  61.743763  40.0468592 38.810582 55.694066
        [65,70)   17.141104  26.737375  22.039887  15.8129795 14.819010 22.646402
        [70,75)    4.029856   2.949233  20.149282  15.9243343 10.025868  7.962167
        [75,80)    3.206763   3.110274   4.082705   0.9724316  7.096489  2.041353
        [80,Inf)   0.000000   0.000000   0.000000   0.0000000  0.000000  0.000000
                contact.age.group
      age.group    [60,65)   [65,70)   [70,75)   [75,80)  [80,Inf)
        [0,5)    24.677783  8.727480  9.909679  2.993785  0.000000
        [5,10)   25.803873 11.048926  6.071756  4.959343  4.094807
        [10,15)  18.924121 11.851619 10.258733  7.282992  3.982674
        [15,20)  24.902552 21.186852  4.887084  4.966279  1.074401
        [20,25)  11.156829 11.025208  4.056875  8.113749  2.037269
        [25,30)  20.230725 13.096621  5.962517  0.000000  4.007345
        [30,35)  23.553124  7.798386  5.067440  3.117843  1.071522
        [35,40)  27.499796 20.825517 15.028551  8.346657  6.186883
        [40,45)  29.207953 14.039942 12.903832  7.134775 10.833673
        [45,50)  29.816565  6.941030 12.314681 15.053803 12.314681
        [50,55)  26.754035 13.911797 17.707294  5.995743 10.975055
        [55,60)  41.949384 15.278607 10.217936  7.007663 23.503946
        [60,65)  43.217595 23.798264 13.034119  6.958521 10.976500
        [65,70)  22.812626 19.057977  9.023120  3.025473  8.003353
        [70,75)  11.876586 17.988035 12.780010 13.958179  4.915388
        [75,80)   5.151626  0.000000  1.068921  3.986216  1.944863
        [80,Inf)  0.000000  0.000000  0.000000  0.000000  0.000000
      
      $test1$participants
          age.group participants  proportion
             <char>        <int>       <num>
       1:     [0,5)           95 0.093966370
       2:    [5,10)          102 0.100890208
       3:   [10,15)          102 0.100890208
       4:   [15,20)          105 0.103857567
       5:   [20,25)           59 0.058358061
       6:   [25,30)           59 0.058358061
       7:   [30,35)           60 0.059347181
       8:   [35,40)           70 0.069238378
       9:   [40,45)           62 0.061325420
      10:   [45,50)           55 0.054401583
      11:   [50,55)           66 0.065281899
      12:   [55,60)           54 0.053412463
      13:   [60,65)           66 0.065281899
      14:   [65,70)           27 0.026706231
      15:   [70,75)           22 0.021760633
      16:   [75,80)            7 0.006923838
      17:  [80,Inf)            0 0.000000000
      
      
      $test2
      $test2$matrix
               contact.age.group
      age.group     [0,5)   [5,Inf)
        [0,5)   1.5429448  7.027931
        [5,Inf) 0.3673316 13.474102
      
      $test2$demography
         age.group population proportion  year
            <char>      <num>      <num> <int>
      1:     [0,5)   13498647 0.04967121  2005
      2:   [5,Inf)  258261321 0.95032879  2005
      
      $test2$participants
         age.group participants proportion
            <char>        <int>      <num>
      1:     [0,5)          654  0.0908712
      2:   [5,Inf)         6543  0.9091288
      
      
      $test3
      $test3$mean.contacts
      [1] 3.887154
      
      $test3$normalisation
      [1] 1.021167
      
      $test3$contacts
      [1] 1.0973166 1.4632281 0.9342813
      
      $test3$matrix
                contact.age.group
      age.group     [0,5)   [5,10)  [10,Inf)
        [0,5)    1.769367 1.836114 0.8815070
        [5,10)   1.376954 2.441012 0.8642004
        [10,Inf) 1.035333 1.353471 0.9708081
      
      $test3$demography
         age.group population proportion  year
            <char>      <num>      <num> <int>
      1:     [0,5)    1271578 0.06282841  2005
      2:    [5,10)    1329536 0.06569210  2005
      3:  [10,Inf)   17637787 0.87147948  2005
      
      $test3$participants
         age.group participants proportion
            <char>        <int>      <num>
      1:     [0,5)           95 0.09396637
      2:    [5,10)          102 0.10089021
      3:  [10,Inf)          814 0.80514342
      
      
      $test4
      $test4$matrix
                contact.age.group
      age.group      [0,5)   [5,15)  [15,Inf)
        [0,5)    0.0000000 2.019697  3.087121
        [5,15)   0.9217259 7.579330 27.392285
        [15,Inf) 0.1822259 3.542984  7.130996
      
      $test4$demography
         age.group population proportion  year
            <char>      <num>      <num> <int>
      1:     [0,5)   13498647 0.04967121  2005
      2:    [5,15)   29578398 0.10884016  2005
      3:  [15,Inf)  228682923 0.84148863  2005
      
      $test4$participants
         age.group participants proportion
            <char>        <int>      <num>
      1:     [0,5)          660 0.09134948
      2:    [5,15)         1374 0.19017301
      3:  [15,Inf)         5191 0.71847751
      
      

# survey argument is validated

    Code
      contact_matrix(survey = "bogus")
    Condition
      Error in `contact_matrix()`:
      ! `survey` must be a survey object (created using `survey()` or `get_survey()`).

# error is thrown if age limits are non-numeric

    Code
      contact_matrix(survey = polymod, age_limits = c(0, 5, "fifteen"))
    Condition
      Warning in `check_age_limits_increasing()`:
      NAs introduced by coercion
      Error in `contact_matrix()`:
      ! `<int>` must be an increasing integer vector of lower age limits.
      i We see: 0, 5, and NA

# error is thrown if country is not found

    Code
      contact_matrix(survey = polymod, countries = c("Italy", "Zamonia"))
    Condition
      Error in `flexible_countrycode()`:
      ! Survey data not found for: "Zamonia".

# warning is thrown if filter column is not found

    Filter column: "test" not found.

# warning is thrown if missing data exist

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

# error is thrown if an unknown argument is passed

    Code
      contact_matrix(dummy = "test")
    Condition
      Error in `check_arg_dots_in()`:
      ! Unknown argument: "dummy".

# error is thrown if invalid age limits are passed

    Code
      contact_matrix(survey = polymod, age_limits = c(13, 11))
    Condition
      Error in `contact_matrix()`:
      ! `<int>` must be an increasing integer vector of lower age limits.
      i We see: 13 and 11

# error if no participants after selecting the country

    Code
      contact_matrix(survey = polymod, countries = "Romania")
    Condition
      Error in `filter_countries()`:
      ! No participants left after selecting countries: "Romania"

# warning if population needed but no 'year' column

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

# warning if day of week weigh requested but not present

    `weigh_dayofweek` is "TRUE", but no `dayofweek` column in the data.
    i Will ignore.

# warning is thrown if country has no survey population

    Code
      contact_matrix(survey = polymod5, symmetric = TRUE)
    Condition
      Warning:
      Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
      When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
      i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
      Error in `survey_pop_from_countries()`:
      ! Could not find population data for: "Zamonia".
      i Pass population data directly via the `survey_pop` argument.

# warning is thrown if contact survey has no age information

    Contact age column `cnt_age` or columns to estimate contact age (`cnt_age_exact` or `cnt_age_est_min` and `cnt_age_est_max`) do not exist in the contact data frame.

# warning is thrown if participant data has no country

    Country column `country` does not exist in the participant data frame.

# check result is reported back

    Code
      check(x = polymod2)
    Message
      > Check OK.

# good suggestions are made

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

---

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

---

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

# nonsensical operations are warned about

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

---

    Code
      contact_matrix(survey = polymod, counts = TRUE, symmetric = TRUE, age_limits = c(
        0, 5))
    Condition
      Warning:
      Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
      When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
      i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
      Warning in `contact_matrix()`:
      `symmetric = TRUE` does not make sense with `counts = TRUE`; will not make matrix symmetric.
      Warning in `normalise_weighted_matrix()`:
      Large differences in the size of the sub-populations with the current age breaks are likely to result in artefacts after making the matrix symmetric.
      ! Please reconsider the age breaks to obtain more equally sized sub-populations.
      i Normalization factors: [0.2 and 5.1]
    Output
      $matrix
               contact.age.group
      age.group    [0,5)  [5,Inf)
        [0,5)   1463.000 26710.24
        [5,Inf) 1396.075 87824.00
      
      $demography
         age.group population proportion  year
            <char>      <num>      <num> <int>
      1:     [0,5)   13498647 0.04967121  2005
      2:   [5,Inf)  258261321 0.95032879  2005
      
      $participants
         age.group participants proportion
            <char>        <int>      <num>
      1:     [0,5)          654 0.09085857
      2:   [5,Inf)         6544 0.90914143
      

---

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

# warning if survey is assumed to be representative

    No `survey.pop` or `countries` given, and no `country` column found in the data.
    i I don't know which population this is from (assuming the survey is representative).

# user-defined reference populations with open age groups

    Code
      contact_matrix(polymod_nocountry, age_limits = c(0, 18, 60), symmetric = TRUE,
      survey_pop = "dummy")
    Condition
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
      Error in `survey_pop_from_countries()`:
      ! Could not find population data for: "dummy".
      i Pass population data directly via the `survey_pop` argument.

# symmetric matrices with large norm weights warn

    Automatic country population lookup in `contact_matrix()` was deprecated in socialmixr 0.6.0.
    When `countries` is given (or a `country` column is present) without `survey_pop`, contact_matrix() currently calls the soft-deprecated `wpp_age()` to look up population data. This automatic lookup will be removed in a future release: callers will then have to supply `survey_pop` whenever `symmetric`, `split`, `per_capita`, `weigh_age`, or `return_demography` is TRUE.
    i Pass `survey_pop` explicitly to silence this warning, e.g. `survey_pop = survey_country_population(survey, countries)` or a data frame from the wpp2024 package.

# pop_age doesn't change total population size

    Not all age groups represented in population data (5-year age band).
    i Linearly estimating age group sizes from the 5-year bands.

# pop_age throws warnings or errors

    Code
      pop_age(3)
    Condition
      Error in `pop_age()`:
      ! Expecting `pop` to be a data.frame with columns `lower.age.limit` and `population`.

---

    `wpp_age()` was deprecated in socialmixr 0.6.0.
    Pass population data directly via the {.arg survey_pop} argument instead.
    i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.

# error is thrown if survey contains false data types

    Code
      check(erroneous_type1)
    Condition
      Error in `check()`:
      ! The participants and contacts elements of `x` must be data.frames.

---

    Code
      check(erroneous_type2)
    Condition
      Error in `check()`:
      ! The participants and contacts elements of `x` must be data.frames.

# incorrect structure of data frames is correctly identified

    `id.column` "part_id" does not exist in both the participants and contacts data frames.

---

    Participant age column `part_age` or columns to estimate participant age (`part_age_exact` or `part_age_est_min` and `part_age_est_max`) do not exist in the participant data frame.

---

    Contact age column `cnt_age` or columns to estimate contact age (`cnt_age_exact` or `cnt_age_est_min` and `cnt_age_est_max`) do not exist in the contact data frame.

# error is thrown if survey contains false data types

    Code
      as_contact_survey(erroneous_type1)
    Condition
      Error in `as_contact_survey()`:
      ! Assertion on 'x$participants' failed: Must be of type 'data.frame', not 'character'.

---

    Code
      as_contact_survey(erroneous_type2)
    Condition
      Error in `as_contact_survey()`:
      ! Assertion on 'x$participants' failed: Must be of type 'data.frame', not 'double'.

# incorrect structure of data frames is correctly identified

    Code
      as_contact_survey(erroneous_structure1)
    Condition
      Error in `as_contact_survey()`:
      ! Assertion on 'colnames(x$participants)' failed: Names must include the elements {'part_id'}, but is missing elements {'part_id'}.

# explicitly specified missing columns still error

    Code
      as_contact_survey(no_country_survey, country.column = "country")
    Condition
      Warning:
      The `country.column` argument of `as_contact_survey()` is deprecated as of socialmixr 0.5.0.
      i Please use the `country_column` argument instead.
      Error in `map()`:
      i In index: 1.
      Caused by error in `.f()`:
      ! `country` column "country" does not exist in the participant data frame.

---

    Code
      as_contact_survey(no_year_survey, year.column = "year")
    Condition
      Warning:
      The `year.column` argument of `as_contact_survey()` is deprecated as of socialmixr 0.5.0.
      i Please use the `year_column` argument instead.
      Error in `map()`:
      i In index: 2.
      Caused by error in `.f()`:
      ! `year` column "year" does not exist in the participant data frame.

# survey_country_population() works

    Code
      as.data.frame(survey_country_population(polymod))
    Condition
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
    Output
         lower.age.limit population
      1                0   13498647
      2                5   14193543
      3               10   15384855
      4               15   16464604
      5               20   17158735
      6               25   17416147
      7               30   18726407
      8               35   21118833
      9               40   21529102
      10              45   19927357
      11              50   18134701
      12              55   17122402
      13              60   14454988
      14              65   14280416
      15              70   11565417
      16              75    9469420
      17              80    6736784
      18              85    2832127
      19              90    1417066
      20              95     300205
      21             100      28212

---

    Code
      as.data.frame(survey_country_population(polymod, countries = "Belgium"))
    Condition
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
    Output
         lower.age.limit population
      1                0     583492
      2                5     593148
      3               10     632157
      4               15     626921
      5               20     649588
      6               25     663176
      7               30     705878
      8               35     773177
      9               40     823305
      10              45     779436
      11              50     710061
      12              55     674177
      13              60     508701
      14              65     496720
      15              70     470384
      16              75     392469
      17              80     291166
      18              85     108599
      19              90      52117
      20              95      11126
      21             100       1087

---

    Code
      as.data.frame(survey_country_population(polymod, countries = c("Belgium",
        "Italy")))
    Condition
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
    Output
         lower.age.limit population
      1                0    3342241
      2                5    3305938
      3               10    3447403
      4               15    3544397
      5               20    3818506
      6               25    4449760
      7               30    5258993
      8               35    5570182
      9               40    5525281
      10              45    4915874
      11              50    4537519
      12              55    4551877
      13              60    3770551
      14              65    3819156
      15              70    3329171
      16              75    2771930
      17              80    2031237
      18              85     818477
      19              90     445093
      20              95      93168
      21             100       8612

---

    Code
      as.data.frame(survey_country_population(polymod, countries = "Australia"))
    Condition
      Warning:
      `wpp_age()` was deprecated in socialmixr 0.6.0.
      Pass population data directly via the {.arg survey_pop} argument instead.
      i The underlying {.pkg wpp2017} data is also outdated; use {.pkg wpp2024} from GitHub for more recent data.
    Output
         lower.age.limit population
      1                0    1271578
      2                5    1329536
      3               10    1405655
      4               15    1393171
      5               20    1415491
      6               25    1330279
      7               30    1515614
      8               35    1456957
      9               40    1560931
      10              45    1455358
      11              50    1326630
      12              55    1247551
      13              60     920492
      14              65     750231
      15              70     619847
      16              75     548509
      17              80     401271
      18              85     191366
      19              90      79997
      20              95      16649
      21             100       1788

# survey_country_population() errors appropriately

    Code
      survey_country_population(polymod_copy)
    Condition
      Error in `survey_country_population()`:
      ! Country name must be provided in `survey` or `countries`
      i `survey`: `survey$participants$country` is: "NULL"
      i `countries` is:

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check}
\alias{check}
\alias{check.contact_survey}
\title{Check contact survey data}
\usage{
\method{check}{contact_survey}(
  x,
  id.column = "part_id",
  participant.age.column = "part_age",
  country.column = "country",
  year.column = "year",
  contact.age.column = "cnt_age",
  ...
)
}
\arguments{
\item{x}{A \code{\link[=survey]{survey()}} object}

\item{id.column}{the column in both the \code{participants} and
\code{contacts} data frames that links contacts to participants}

\item{participant.age.column}{the column in the \code{participants}
data frame containing participants' age; if this does not
exist, at least columns "..._exact", "..._est_min" and
"..._est_max" must exist (see the
\code{estimated.participant.age} option in \code{\link[=contact_matrix]{contact_matrix()}})}

\item{country.column}{the column in the \code{participants} data
frame containing the country in which the participant was
queried}

\item{year.column}{the column in the \code{participants} data frame
containing the year in which the participant was queried}

\item{contact.age.column}{the column in the \code{contacts} data
frame containing the age of contacts; if this does not exist,
at least columns "..._exact", "..._est_min" and
"..._est_max" must exist (see the
\code{estimated.contact.age} option in \code{\link[=contact_matrix]{contact_matrix()}})}

\item{...}{ignored}
}
\value{
invisibly returns a character vector of the relevant columns
}
\description{
Checks that a survey fulfills all the requirements
to work with the 'contact_matrix' function
}
\examples{
data(polymod)
check(polymod)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact_matrix.R
\name{contact_matrix}
\alias{contact_matrix}
\title{Generate a contact matrix from diary survey data}
\usage{
contact_matrix(
  survey,
  countries = NULL,
  survey_pop = NULL,
  age_limits = NULL,
  filter = NULL,
  counts = FALSE,
  symmetric = FALSE,
  split = FALSE,
  sample_participants = FALSE,
  estimated_participant_age = c("mean", "sample", "missing"),
  estimated_contact_age = c("mean", "sample", "missing"),
  missing_participant_age = c("remove", "keep"),
  missing_contact_age = c("remove", "sample", "keep", "ignore"),
  weights = NULL,
  weigh_dayofweek = FALSE,
  weigh_age = FALSE,
  weight_threshold = NA,
  symmetric_norm_threshold = 2,
  sample_all_age_groups = FALSE,
  sample_participants_max_tries = 1000,
  return_part_weights = FALSE,
  return_demography = NA,
  per_capita = FALSE,
  ...,
  survey.pop = deprecated(),
  age.limits = deprecated(),
  sample.participants = deprecated(),
  estimated.participant.age = deprecated(),
  estimated.contact.age = deprecated(),
  missing.participant.age = deprecated(),
  missing.contact.age = deprecated(),
  weigh.dayofweek = deprecated(),
  weigh.age = deprecated(),
  weight.threshold = deprecated(),
  symmetric.norm.threshold = deprecated(),
  sample.all.age.groups = deprecated(),
  sample.participants.max.tries = deprecated(),
  return.part.weights = deprecated(),
  return.demography = deprecated(),
  per.capita = deprecated()
)
}
\arguments{
\item{survey}{a \code{\link[=survey]{survey()}} object.}

\item{countries}{limit to one or more countries; if NULL
(default), will use all countries in the survey; these can be
given as country names or 2-letter (ISO Alpha-2) country
codes.}

\item{survey_pop}{survey population -- either a data frame with
columns 'lower.age.limit' and 'population', or a character
vector giving the name(s) of a country or countries from the
list that can be obtained via \code{wpp_countries}; if NULL
(default), will use the country populations from the chosen
countries, or all countries in the survey if \code{countries} is
NULL.}

\item{age_limits}{lower limits of the age groups over which to
construct the matrix. If NULL (default), age limits are
inferred from participant and contact ages.}

\item{filter}{any filters to apply to the data, given as list
of the form (column=filter_value) - only contacts that have
'filter_value' in 'column' will be considered. If multiple
filters are given, they are all applied independently and in
the sequence given. Default value is NULL; no filtering
performed.}

\item{counts}{whether to return counts (instead of means).}

\item{symmetric}{whether to make matrix symmetric, such that
\eqn{c_{ij}N_i = c_{ji}N_j}.}

\item{split}{whether to split the contact matrix into the mean
number of contacts, in each age group (split further into the
product of the mean number of contacts across the whole
population (\code{mean.contacts}), a normalisation constant
(\code{normalisation}) and age-specific variation in contacts
(\code{contacts})), multiplied with an assortativity matrix
(\code{assortativity}) and a population multiplier (\code{demography}).
For more detail on this, see the "Getting Started" vignette.}

\item{sample_participants}{whether to sample participants
randomly (with replacement); done multiple times this can be
used to assess uncertainty in the generated contact matrices.
See the "Bootstrapping" section in the vignette for how to
do this.}

\item{estimated_participant_age}{if set to "mean" (default),
people whose ages are given as a range (in columns named
"..._est_min" and "..._est_max") but not exactly (in a
column named "..._exact") will have their age set to the
mid-point of the range; if set to "sample", the age will be
sampled from the range; if set to "missing", age ranges will
be treated as missing}

\item{estimated_contact_age}{if set to "mean" (default),
contacts whose ages are given as a range (in columns named
"..._est_min" and "..._est_max") but not exactly (in a
column named "..._exact") will have their age set to the
mid-point of the range; if set to "sample", the age will be
sampled from the range; if set to "missing", age ranges will
be treated as missing.}

\item{missing_participant_age}{if set to "remove" (default),
participants without age information are removed; if set to
"keep", participants with missing age are kept and will
appear in the contact matrix in a row labelled "NA".}

\item{missing_contact_age}{if set to "remove" (default),
participants that have contacts without age information are
removed; if set to "sample", contacts without age information
are sampled from all the contacts of participants of the same
age group; if set to "keep", contacts with missing age are
kept and will appear in the contact matrix in a column
labelled "NA"; if set to "ignore", contacts without age
information are removed from the analysis (but the
participants that made them are kept).}

\item{weights}{column name(s) of the participant data of the
\code{\link[=survey]{survey()}} object with user-specified weights (default =
empty vector).}

\item{weigh_dayofweek}{whether to weigh social contacts data
by the day of the week (weight (5/7 / N_week / N) for
weekdays and (2/7 / N_weekend / N) for weekends).}

\item{weigh_age}{whether to weigh social contacts data by the
age of the participants (vs. the populations' age
distribution).}

\item{weight_threshold}{threshold value for the standardized
weights before running an additional standardisation (default
'NA' = no cutoff).}

\item{symmetric_norm_threshold}{threshold value for the
normalization weights when \code{symmetric = TRUE} before showing
a warning that that large differences in the size of the
sub-populations are likely to result in artefacts when making
the matrix symmetric (default 2).}

\item{sample_all_age_groups}{what to do if sampling
participants (with \code{sample_participants = TRUE}) fails to
sample participants from one or more age groups; if FALSE
(default), corresponding rows will be set to NA, if TRUE the
sample will be discarded and a new one taken instead.}

\item{sample_participants_max_tries}{maximum number of attempts
when \code{sample_all_age_groups = TRUE}; defaults to 1000.}

\item{return_part_weights}{boolean to return the participant
weights.}

\item{return_demography}{boolean to explicitly return
demography data that corresponds to the survey data (default
'NA' = if demography data is requested by other function
parameters).}

\item{per_capita}{whether to return a matrix with contact rates
per capita (default is FALSE and not possible if 'counts=TRUE'
or 'split=TRUE').}

\item{...}{further arguments to pass to \code{\link[=get_survey]{get_survey()}},
\code{\link[=check]{check()}} and \code{\link[=pop_age]{pop_age()}} (especially column names).}

\item{survey.pop, age.limits, sample.participants, estimated.participant.age, estimated.contact.age, missing.participant.age, missing.contact.age, weigh.dayofweek, weigh.age, weight.threshold, symmetric.norm.threshold, sample.all.age.groups, sample.participants.max.tries, return.part.weights, return.demography, per.capita}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use the underscore-separated versions of these arguments instead.}
}
\value{
a contact matrix, and the underlying demography of the
surveyed population
}
\description{
Samples a contact survey
}
\examples{
data(polymod)
contact_matrix(
  survey = polymod,
  countries = "United Kingdom",
  age_limits = c(0, 1, 5, 15)
)
}
\author{
Sebastian Funk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reduce_agegroups.R
\name{reduce_agegroups}
\alias{reduce_agegroups}
\title{Reduce the number of age groups given a broader set of limits}
\usage{
reduce_agegroups(x, limits)
}
\arguments{
\item{x}{vector of limits}

\item{limits}{new limits}
}
\value{
vector with the new age groups
}
\description{
Operates on lower limits
}
\examples{
reduce_agegroups(seq_len(20), c(0, 5, 10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weigh.R
\name{weigh}
\alias{weigh}
\title{Weigh survey participants}
\usage{
weigh(survey, by, target = NULL, groups = NULL, ...)
}
\arguments{
\item{survey}{a \code{\link[=survey]{survey()}} object (must have been processed by
\code{\link[=assign_age_groups]{assign_age_groups()}} if using data frame target)}

\item{by}{column name in the participant data to weigh by}

\item{target}{target weights: \code{NULL} for direct numeric weighting, an
unnamed numeric vector (with \code{groups}), a named numeric vector, or a
data frame with columns \code{lower.age.limit} and \code{population}}

\item{groups}{a list of value sets mapping column values to groups (used
with unnamed \code{target} vector); must be the same length as \code{target}}

\item{...}{further arguments passed to \code{\link[=pop_age]{pop_age()}} when \code{target} is a data
frame}
}
\value{
the survey object with updated participant weights
}
\description{
Applies weights to participants in a \code{contact_survey} object. Weights are
always multiplied into an existing \code{weight} column (or one is created with
value 1), making multiple calls composable.

The behaviour depends on the combination of arguments:
\describe{
\item{\code{target = NULL}}{Numeric column: multiply \code{weight} by column values
directly.}
\item{Unnamed \code{target} + \code{groups}}{Map column values to groups, assign
\code{target[g] / n_in_group} per participant.}
\item{Named \code{target}}{Names match column values, assign
\code{target[val] / n_with_val} per participant.}
\item{Data frame \code{target}}{Post-stratify against population data (expanded
to single-year ages via \code{\link[=pop_age]{pop_age()}}).}
}
}
\examples{
data(polymod)
# Direct numeric weighting
if ("survey_weight" \%in\% names(polymod$participants)) {
  polymod |> weigh("survey_weight")
}

# Dayofweek weighting with groups (POLYMOD uses 0 = Sunday, 6 = Saturday)
polymod |>
  weigh("dayofweek", target = c(5, 2), groups = list(1:5, c(0, 6)))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_survey.R
\name{load_survey}
\alias{load_survey}
\title{Load a survey from local files}
\usage{
load_survey(files, participant_key = NULL, ...)
}
\arguments{
\item{files}{a vector of file names as returned by \code{\link[=download_survey]{download_survey()}}}

\item{participant_key}{character vector specifying columns that uniquely
identify participant observations. For cross-sectional surveys this is
typically just \code{"part_id"} (the default). For longitudinal surveys with
multiple observations per participant, specify additional columns like
\code{c("part_id", "wave")}. When \code{NULL} (the default), the function will
auto-detect if additional columns are needed and inform you.}

\item{...}{options for \code{\link[=clean]{clean()}}, which is called at the end of this}
}
\value{
a survey in the correct format. For longitudinal surveys with
multiple observations per participant, the returned object includes an
\code{observation_key} field containing the column names (excluding \code{part_id})
that distinguish observations for the same participant.
}
\description{
Loads a survey from a local file system. Tables are expected
as csv files, and a reference (if present) as JSON.
}
\examples{
\dontrun{
list_surveys()
peru_files <- download_survey("https://doi.org/10.5281/zenodo.1095664")
peru_survey <- load_survey(peru_files)

# For longitudinal surveys, specify the unique key explicitly:
france_files <- download_survey("https://doi.org/10.5281/zenodo.1157918")
france_survey <- load_survey(france_files,
  participant_key = c("part_id", "wave", "studyDay")
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limits_to_agegroups.R
\name{agegroups_to_limits}
\alias{agegroups_to_limits}
\title{Convert age groups to lower age limits}
\usage{
agegroups_to_limits(x)
}
\arguments{
\item{x}{age groups (a factor, as produced by \code{\link[=limits_to_agegroups]{limits_to_agegroups()}} or
\code{\link[=assign_age_groups]{assign_age_groups()}})}
}
\value{
a numeric vector of lower age limits
}
\description{
Inverse of \code{\link[=limits_to_agegroups]{limits_to_agegroups()}}. Extracts lower age limits from age group
labels.
}
\examples{
agegroups_to_limits(limits_to_agegroups(c(0, 5, 10), notation = "brackets"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{warn_multiple_observations}
\alias{warn_multiple_observations}
\title{Warn if survey has multiple observations per participant}
\usage{
warn_multiple_observations(
  participants,
  observation_key = NULL,
  filter_hint = c("pipeline", "legacy")
)
}
\arguments{
\item{participants}{participant data.table}

\item{observation_key}{optional column name(s) identifying observations}

\item{filter_hint}{character; "pipeline" for pipeline-style hint or
"legacy" for contact_matrix-style hint}
}
\value{
NULL invisibly
}
\description{
Issues a warning when a survey contains multiple observations per
participant (more rows than unique part_id values).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{impute_participant_ages}
\alias{impute_participant_ages}
\title{Impute participant ages}
\usage{
impute_participant_ages(
  participants,
  estimate = c("mean", "sample", "missing")
)
}
\arguments{
\item{participants}{A survey data set of participants}

\item{estimate}{if set to "mean" (default), people whose ages are given as a
range (in columns named "..._est_min" and "..._est_max") but not exactly
(in a column named "..._exact") will have their age set to the mid-point of
the range; if set to "sample", the age will be sampled from the range; if
set to "missing", age ranges will be treated as missing}
}
\value{
The participant data, potentially with participant ages imputed depending on
the \code{estimate} method and whether age columns are present in the data.
}
\description{
Imputes participant survey data, where variables are named:
"part_age_est_min" and "part_age_est_max". Uses mean imputation, sampling
(hot deck), or leaves them as missing. These are controlled by the
\code{estimate} argument.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{is_doi}
\alias{is_doi}
\title{Checks if a character string is a DOI}
\usage{
is_doi(x)
}
\arguments{
\item{x}{Character vector; the string or strings to check}
}
\value{
Logical; \code{TRUE} if \code{x} is a DOI, \code{FALSE} otherwise
}
\description{
Checks if a character string is a DOI
}
\author{
Sebastian Funk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-contact-survey.R
\name{assemble_survey}
\alias{assemble_survey}
\title{Assemble a contact survey with new participant/contact data}
\usage{
assemble_survey(x, participants, contacts)
}
\arguments{
\item{x}{a \code{contact_survey} object}

\item{participants}{new participants data.table}

\item{contacts}{new contacts data.table}
}
\value{
a \code{contact_survey} object with all fields from \code{x} preserved
}
\description{
Creates a new survey object preserving all fields from the original,
replacing only \code{participants} and \code{contacts} with the supplied data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{get_citation}
\alias{get_citation}
\title{Citation for a survey}
\usage{
get_citation(x)
}
\arguments{
\item{x}{a character vector of surveys to cite}
}
\value{
citation as bibentry
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{get_citation()} has been deprecated in favour of
\code{contactsurveys::get_citation()}.

Gets a full citation for a \code{\link[=survey]{survey()}}.
}
\examples{
# we recommend using the contactsurveys package for get_citation()
\dontrun{
data(polymod)
citation <- contactsurveys::get_citation(polymod)
print(citation)
print(citation, style = "bibtex")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{download_survey}
\alias{download_survey}
\title{Download a survey from its Zenodo repository}
\usage{
download_survey(survey, dir = NULL, sleep = 1)
}
\arguments{
\item{survey}{a URL (see \code{contactsurveys::list_surveys()})}

\item{dir}{a directory to save the files to; if not given, will save to a
temporary directory}

\item{sleep}{time to sleep between requests to avoid overloading the server
(passed on to \code{\link[base]{Sys.sleep}})}
}
\value{
a vector of filenames that can be used with \link{load_survey}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{download_survey()} has been deprecated in favour of
\code{contactsurveys::download_survey()}.

\code{download_survey()} downloads survey data from Zenodo.
}
\examples{
# we recommend using the contactsurveys package for download_survey()
\dontrun{
# if needed, discover surveys with:
contactsurveys::list_surveys()
peru_survey <- download_survey("https://doi.org/10.5281/zenodo.1095664")
# -->
peru_survey <- contactsurveys::download_survey(
  "https://doi.org/10.5281/zenodo.1095664"
)
}
}
\seealso{
load_survey
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postprocess-matrix.R
\name{resolve_survey_pop}
\alias{resolve_survey_pop}
\title{Resolve survey population to match matrix age groups}
\usage{
resolve_survey_pop(survey_pop, age_limits, ...)
}
\arguments{
\item{survey_pop}{a data frame with columns \code{lower.age.limit} and
\code{population} (e.g. from \code{\link[=wpp_age]{wpp_age()}})}

\item{age_limits}{numeric vector of age group lower limits from the matrix}

\item{...}{passed to \code{\link[=pop_age]{pop_age()}} for interpolation}
}
\value{
a data.table with \code{lower.age.limit}, \code{population}, and
\code{upper.age.limit} aligned to the matrix age groups
}
\description{
Resolve survey population to match matrix age groups
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{polymod}
\alias{polymod}
\title{Social contact data from 8 European countries}
\format{
A list of two data frames:
\describe{
\item{participants}{the study participant, with age, country, year and day
of the week (starting with 1 = Monday)}
\item{contacts}{reported contacts of the study participants. The variable
phys_contact has two levels (1 denotes physical contact while 2 denotes
non-physical contact), duration_multi has five levels (1 is less than 5
minutes while 5 is more than 4 hours, increasing in the order found in
Figure 1 in Mossong et al.), and frequency_multi has five levels (1 is
daily, 2 is weekly, 3 is monthly, 4 is less often, and 5 is first time)}
All other variables are described on the Zenodo repository of the data,
available at \doi{10.5281/zenodo.1043437}
}
}
\source{
\doi{10.1371/journal.pmed.0050074}
}
\usage{
polymod
}
\description{
A dataset containing social mixing diary data from 8 European countries:
Belgium, Germany, Finland, Great Britain, Italy, Luxembourg,
The Netherlands and Poland.
The Data are fully described in Mossong J, Hens N, Jit M, Beutels P, Auranen
K, Mikolajczyk R, et al. (2008) Social Contacts and Mixing Patterns Relevant
to the Spread of Infectious Diseases. PLoS Med 5(3): e74.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-age-distribution.R
\name{contact_age_distribution}
\alias{contact_age_distribution}
\title{Extract the empirical age distribution of contacts from a survey}
\usage{
contact_age_distribution(survey)
}
\arguments{
\item{survey}{a \code{\link[=survey]{survey()}} object}
}
\value{
a data.frame with columns \code{age} (integer) and \code{proportion} (numeric,
summing to 1)
}
\description{
Returns a data.frame of (age, proportion) pairs representing how
contact ages are distributed in the survey. This can be passed to
\code{\link[=assign_age_groups]{assign_age_groups()}} as \code{estimated_contact_age} to impute ages
from ranges using this distribution rather than uniform sampling.
}
\examples{
data(polymod)
dist <- contact_age_distribution(polymod)
head(dist)
plot(dist$age, dist$proportion, type = "h",
     xlab = "Age", ylab = "Proportion")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{get_survey}
\alias{get_survey}
\title{Get a survey, either from its Zenodo repository, a set of
files, or a survey variable}
\usage{
get_survey(survey, clear_cache = FALSE, ...)
}
\arguments{
\item{survey}{a DOI or url to get the survey from, or a \code{\link[=survey]{survey()}} object.}

\item{clear_cache}{logical, whether to clear the cache before downloading
the survey; by default, the cache is not cleared and so multiple calls of
this function to access the same survey will not result in repeated
downloads.}

\item{...}{currently unused}
}
\value{
a survey in the correct format
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{get_survey()} has been deprecated in favour of using
\code{contactsurveys::download_survey()} and then \code{\link[=load_survey]{load_survey()}}.

Downloads survey data, or extracts them from files, and returns a clean data
set. If a survey URL is accessed multiple times, the data will be cached
(unless \code{clear_cache} is set to \code{TRUE}) to avoid repeated downloads.

If survey objects are used repeatedly the downloaded files can be saved and
reloaded between sessions then survey objects can be saved/loaded using
\code{\link[base:readRDS]{base::saveRDS()}} and \code{\link[base:readRDS]{base::readRDS()}}, or via the individual survey files
that can be downloaded using \code{\link[=download_survey]{download_survey()}} and subsequently loaded
using \code{\link[=load_survey]{load_survey()}}.
}
\examples{
\dontrun{
list_surveys()
peru_doi <- "https://doi.org/10.5281/zenodo.1095664"
peru_survey <- get_survey(peru_doi)
## --> We now recommend:
peru_survey <- contactsurveys::download_survey(peru_doi)
peru_data <- load_survey(peru_survey)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecate-args.R
\name{deprecate_arg}
\alias{deprecate_arg}
\title{Handle deprecated argument}
\usage{
deprecate_arg(old_arg, new_arg, old_name, new_name, fn_name, version = "0.5.0")
}
\arguments{
\item{old_arg}{the deprecated argument value}

\item{new_arg}{the new argument value}

\item{old_name}{the old argument name (with dot)}

\item{new_name}{the new argument name (with underscore)}

\item{fn_name}{the function name}

\item{version}{the version when deprecated}
}
\value{
the value to use (new_arg if provided, otherwise old_arg)
}
\description{
Handle deprecated argument
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{sample_from_age_distribution}
\alias{sample_from_age_distribution}
\title{Sample ages from a distribution within \verb{[min, max]} bands}
\usage{
sample_from_age_distribution(mins, maxs, distribution)
}
\arguments{
\item{mins}{integer vector of lower bounds}

\item{maxs}{integer vector of upper bounds}

\item{distribution}{data.frame with \code{age} and \code{proportion} columns}
}
\value{
integer vector of sampled ages
}
\description{
Sample ages from a distribution within \verb{[min, max]} bands
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{.get_survey}
\alias{.get_survey}
\title{Internal function to get survey data}
\usage{
.get_survey(survey, ...)
}
\arguments{
\item{survey}{a DOI or url to get the survey from, or a \code{\link[=survey]{survey()}} object.}

\item{...}{currently unused}
}
\description{
Internal function to get survey data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey.R
\name{survey}
\alias{survey}
\title{Contact survey}
\usage{
survey(participants, contacts, reference = NULL)
}
\arguments{
\item{participants}{a \code{data.frame} containing information on participants}

\item{contacts}{a \code{data.frame} containing information on contacts}

\item{reference}{a \code{list} containing information needed to
reference the survey, in particular it can contain a "title",
"bibtype", "author", "doi", "publisher", "note", "year"}
}
\value{
a new survey object
}
\description{
Deprecated. Use \code{as_survey} instead.
}
\author{
Sebastian Funk
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{survey_countries}
\alias{survey_countries}
\title{List all countries contained in a survey}
\usage{
survey_countries(survey, country.column = "country", ...)
}
\arguments{
\item{survey}{a DOI or url to get the survey from, or a \code{\link[=survey]{survey()}} object.}

\item{country.column}{column in the survey indicating the country}

\item{...}{further arguments for \code{\link[=get_survey]{get_survey()}}}
}
\value{
list of countries
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\details{
\code{survey_countries()} has been deprecated in favour of using
\code{contactsurveys::download_survey()}, and \code{\link[=load_survey]{load_survey()}}, and then
exploring the country column yourself.
}
\examples{
data(polymod)
survey_countries(polymod)
## --> we now recommend
\dontrun{
doi_peru <- "10.5281/zenodo.1095664" # nolint
# download the data with the contactsurveys package
peru_survey <- contactsurveys::download_survey(doi_peru)
# load the survey with socialmixr
peru_data <- socialmixr::load_survey(peru_survey)
# find the unique country - assuming your data has a "country" column:
unique(peru_data$participants$country)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign-age-groups.R
\name{assign_age_groups}
\alias{assign_age_groups}
\title{Assign age groups in survey data}
\usage{
assign_age_groups(
  survey,
  age_limits = NULL,
  estimated_participant_age = c("mean", "sample", "missing"),
  estimated_contact_age = c("mean", "sample", "missing"),
  missing_participant_age = c("remove", "keep"),
  missing_contact_age = c("remove", "sample", "keep", "ignore")
)
}
\arguments{
\item{survey}{a \code{\link[=survey]{survey()}} object}

\item{age_limits}{lower limits of the age groups over which to construct
the matrix. Defaults to NULL. If NULL, age limits are inferred from
participant and contact ages.}

\item{estimated_participant_age}{if set to "mean" (default), people whose
ages are given as a range (in columns named "..._est_min" and
"..._est_max") but not exactly (in a column named "..._exact") will have
their age set to the mid-point of the range; if set to "sample", the age
will be sampled from the range; if set to "missing", age ranges will be
treated as missing}

\item{estimated_contact_age}{if set to "mean" (default), contacts whose ages
are given as a range (in columns named "..._est_min" and "..._est_max") but
not exactly (in a column named "..._exact") will have their age set to the
mid-point of the range; if set to "sample", the age will be sampled from
the range; if set to "missing", age ranges will be treated as missing}

\item{missing_participant_age}{if set to "remove" (default), participants
without age information are removed; if set to "keep", participants with
missing age are kept and treated as a separate age group}

\item{missing_contact_age}{if set to "remove" (default), participants that
have contacts without age information are removed; if set to "sample",
contacts without age information are sampled from all the contacts of
participants of the same age group; if set to "keep", contacts with missing
age are kept and treated as a separate age group; if set to "ignore",
contact with missing age are ignored in the contact analysis}
}
\value{
The survey object with processed age data.
}
\description{
This function processes age data in a survey object. It imputes ages from
ranges, handles missing values, and assigns age groups.
}
\examples{
polymod_grouped <- assign_age_groups(polymod)
polymod_grouped
polymod_custom <- assign_age_groups(polymod, age_limits = c(0, 5, 10, 15))
polymod_custom
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{add_age}
\alias{add_age}
\title{Add age column from exact age (generic helper)}
\usage{
add_age(data, prefix)
}
\arguments{
\item{data}{A data.table containing age data}

\item{prefix}{Column name prefix: "part_age" for participants, "cnt_age" for
contacts}
}
\value{
The data with the age column set from exact ages or
initialised to NA
}
\description{
Generic function to add an age column from an exact age column. Works for
both participant and contact data by specifying the column prefix.
If \verb{<prefix>_exact} exists, it overwrites \verb{<prefix>} with its values.
Otherwise, it creates \verb{<prefix>} with NA values if it doesn't exist.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey-country-population.R
\name{survey_country_population}
\alias{survey_country_population}
\title{Get survey country population data}
\usage{
survey_country_population(survey, countries = NULL)
}
\arguments{
\item{survey}{A \code{\link[=survey]{survey()}} object, with column "country" in "participants".}

\item{countries}{Optional. A character vector of country names. If specified,
this will be used instead of the potential "country" column in
"participants".}
}
\value{
A data table with population data by age group for the survey
countries, aggregated by lower age limit. The function will error if no
country information is available from either the survey or countries
argument.
}
\description{
Looks up the country and year inside a survey, or a provided
"countries" value, and determines the corresponding demographics in the world
population prospects data using \code{\link[=wpp_age]{wpp_age()}}.
}
\examples{
survey_country_population(polymod)
survey_country_population(polymod, countries = "Belgium")
survey_country_population(polymod, countries = c("Belgium", "Italy"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{impute_contact_ages}
\alias{impute_contact_ages}
\title{Impute contact ages}
\usage{
impute_contact_ages(contacts, estimate = c("mean", "sample", "missing"))
}
\arguments{
\item{contacts}{a survey data set of contacts}

\item{estimate}{if set to "mean" (default), contacts whose ages are given as
a range (in columns named "..._est_min" and "..._est_max") but not exactly
(in a column named "..._exact") will have their age set to the mid-point of
the range; if set to "sample", the age will be sampled from the range; if
set to "missing", age ranges will be treated as missing}
}
\value{
The contact data, potentially with contact ages imputed depending on the
\code{estimate} method and whether age columns are present in the data.
}
\description{
Imputes contact survey data, where variables are named:
"cnt_age_est_min" and "cnt_age_est_max". Uses mean imputation, sampling
(hot deck), or leaves them as missing. These are controlled by the
\code{estimate} argument.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-result.R
\name{is_contact_matrix}
\alias{is_contact_matrix}
\title{Test whether an object is a contact_matrix}
\usage{
is_contact_matrix(x)
}
\arguments{
\item{x}{object to test}
}
\value{
logical
}
\description{
Test whether an object is a contact_matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pop_age.R
\name{pop_age}
\alias{pop_age}
\title{Change age groups in population data}
\usage{
pop_age(
  pop,
  age_limits = NULL,
  pop_age_column = "lower.age.limit",
  pop_column = "population",
  ...,
  age.limits = deprecated(),
  pop.age.column = deprecated(),
  pop.column = deprecated()
)
}
\arguments{
\item{pop}{a data frame with columns indicating lower age
limits and population sizes (see 'pop_age_column' and
'pop_column')}

\item{age_limits}{lower age limits of age groups to extract; if NULL
(default), the population data is returned unchanged}

\item{pop_age_column}{column in the 'pop' data frame indicating
the lower age group limit}

\item{pop_column}{column in the 'pop' data frame indicating
the population size}

\item{...}{ignored}

\item{age.limits, pop.age.column, pop.column}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use the underscore
versions (e.g., \code{age_limits}) instead.}
}
\value{
data frame of age-specific population data
}
\description{
This changes population data to have age groups with the given
age_limits, extrapolating linearly between age groups (if more
are requested than available) and summing populations (if fewer
are requested than available)
}
\examples{
ages_it_2015 <- wpp_age("Italy", 2015)

# Modify the age data.frame to get age groups of 10 years instead of 5
pop_age(ages_it_2015, age_limits = seq(0, 100, by = 10))

# The function will also automatically interpolate if necessary
pop_age(ages_it_2015, age_limits = c(0, 18, 40, 65))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{impute_ages}
\alias{impute_ages}
\title{Impute ages from ranges (generic helper)}
\usage{
impute_ages(data, prefix, estimate = c("mean", "sample", "missing"))
}
\arguments{
\item{data}{A data.table containing age data}

\item{prefix}{Column name prefix: "part_age" for participants, "cnt_age" for
contacts}

\item{estimate}{Imputation method: "mean", "sample", or "missing"}
}
\value{
The data with ages imputed according to the specified method
}
\description{
Generic function to impute ages from min/max ranges. Works for both
participant and contact data by specifying the column prefix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load-survey-utils.R
\name{find_unique_key}
\alias{find_unique_key}
\title{Find the minimal unique key for a data.table}
\usage{
find_unique_key(data, base_id = "part_id")
}
\arguments{
\item{data}{A data.table}

\item{base_id}{The base identifier column name (default: "part_id")}
}
\value{
A character vector of column names that form the unique key
}
\description{
Given a data.table and a base identifier column, finds the minimal set of
additional columns needed to uniquely identify each row.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lists.R
\name{wpp_countries}
\alias{wpp_countries}
\title{List all countries and regions for which socialmixr has population data}
\usage{
wpp_countries()
}
\value{
list of countries
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This function is deprecated in favour of passing population data directly
to \code{\link[=contact_matrix]{contact_matrix()}} via the \code{survey_pop} argument, which removes the need
for a country list. Additionally, the underlying \code{wpp2017} data is outdated.
For countries available in more recent WPP editions, use the \code{wpp2024}
package from GitHub.
}
\details{
Uses the World Population Prospects data from the \code{wpp2017} package.
}
\examples{
if (requireNamespace("wpp2017", quietly = TRUE)) {
  wpp_countries()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-contact-survey.R
\name{[.contact_survey}
\alias{[.contact_survey}
\title{Subset a contact survey}
\usage{
\method{[}{contact_survey}(x, i, ...)
}
\arguments{
\item{x}{a \code{contact_survey} object}

\item{i}{an expression to evaluate as a row filter (e.g.
\code{country == "United Kingdom"})}

\item{...}{ignored}
}
\value{
a filtered \code{contact_survey} object
}
\description{
Filters a \code{contact_survey} object using an expression. The expression is
evaluated against whichever table(s) contain the referenced columns
(participants, contacts, or both). When participants are filtered, contacts
are automatically pruned to matching \code{part_id}s.
}
\examples{
data(polymod)
polymod[country == "United Kingdom"]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{list_surveys}
\alias{list_surveys}
\title{List all surveys available for download}
\usage{
list_surveys(clear_cache = FALSE)
}
\arguments{
\item{clear_cache}{logical, whether to clear the cache before downloading
the survey; by default, the cache is not cleared and so multiple calls of
this function to access the same survey will not result in repeated
downloads.}
}
\value{
character vector of surveys
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{list_surveys()} has been deprecated in favour of
\code{contactsurveys::list_surveys()}.
}
\examples{
# we recommend using the contactsurveys package now for listing surveys.
\dontrun{
contactsurveys::list_surveys()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_contact_survey.R
\name{as_contact_survey}
\alias{as_contact_survey}
\title{Check contact survey data}
\usage{
as_contact_survey(
  x,
  id_column = "part_id",
  country_column = NULL,
  year_column = NULL,
  ...,
  id.column = deprecated(),
  country.column = deprecated(),
  year.column = deprecated()
)
}
\arguments{
\item{x}{list containing
\itemize{
\item an element named 'participants', a data frame containing participant
information
\item an element named 'contacts', a data frame containing contact information
\item (optionally) an element named 'reference, a list containing information
information needed to reference the survey, in particular it can contain$a
"title", "bibtype", "author", "doi", "publisher", "note", "year"
}}

\item{id_column}{the column in both the \code{participants} and
\code{contacts} data frames that links contacts to participants}

\item{country_column}{the column in the \code{participants} data frame
containing the country in which the participant was queried; if
NULL (default), will use "country" column if present}

\item{year_column}{the column in the \code{participants} data frame
containing the year in which the participant was queried; if
NULL (default), will use "year" column if present}

\item{...}{additional arguments (currently ignored)}

\item{id.column, country.column, year.column}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use the underscore versions
(e.g., \code{id_column}) instead.}
}
\value{
invisibly returns a character vector of the relevant columns
}
\description{
Checks that a survey fulfills all the requirements
to work with the 'contact_matrix' function
}
\examples{
data(polymod)
check(polymod)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wpp_age.R
\name{wpp_age}
\alias{wpp_age}
\title{Get age-specific population data according to the World
Population Prospects 2017 edition}
\usage{
wpp_age(countries, years)
}
\arguments{
\item{countries}{countries, will return all if not given}

\item{years}{years, will return all if not given}
}
\value{
data frame of age-specific population data
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This function is deprecated in favour of passing population data directly
to \code{\link[=contact_matrix]{contact_matrix()}} via the \code{survey_pop} argument. Additionally, the
underlying \code{wpp2017} data is outdated. For more recent population data,
use the \code{wpp2024} package from GitHub.
}
\details{
This uses data from the \code{wpp2017} package but combines male and female,
and converts age groups to lower age limits. If the requested
year is not present in the historical data, WPP projections
are used.
}
\examples{
wpp_age("Italy", c(1990, 2000))

# For more recent data, use wpp2024 from GitHub:
# remotes::install_github("PPgp/wpp2024")
# library(wpp2024)
# data(popAge1dt)
# uk_pop <- popAge1dt[name == "United Kingdom" & year == 2020,
#                     .(lower.age.limit = age, population = pop * 1000)]
# contact_matrix(polymod, countries = "United Kingdom", survey_pop = uk_pop)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{normalise_country_names}
\alias{normalise_country_names}
\title{Normalise country names}
\usage{
normalise_country_names(countries)
}
\arguments{
\item{countries}{A vector of country names or codes}
}
\value{
A character vector of normalised country names
}
\description{
Uses the countrycode package to standardise country names. This handles
2-letter ISO codes, 3-letter ISO codes, and full country names, converting
them all to standardised country names.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-age-distribution.R
\name{validate_age_distribution}
\alias{validate_age_distribution}
\title{Validate an age distribution data.frame}
\usage{
validate_age_distribution(x)
}
\arguments{
\item{x}{object to validate}
}
\value{
\code{x} invisibly, or errors
}
\description{
Validate an age distribution data.frame
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-utils.R
\name{normalise_weights}
\alias{normalise_weights}
\title{Post-stratification weight normalisation}
\usage{
normalise_weights(participants, by = "age.group", threshold = NULL)
}
\arguments{
\item{participants}{participant data.table with a \code{weight} column}

\item{by}{character; column name(s) to group by (default "age.group")}

\item{threshold}{numeric; if provided, weights above this value are capped
and the weights are re-normalised (default NULL)}
}
\value{
the participants data.table (modified by reference)
}
\description{
Normalises participant weights within groups so that they sum to the number
of participants in each group. Optionally truncates extreme weights to a
threshold and re-normalises.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute-matrix.R
\name{compute_matrix}
\alias{compute_matrix}
\title{Compute contact matrix from prepared survey data}
\usage{
compute_matrix(survey, counts = FALSE, weight_threshold = NULL)
}
\arguments{
\item{survey}{a \code{\link[=survey]{survey()}} object with age groups assigned (via
\code{\link[=assign_age_groups]{assign_age_groups()}})}

\item{counts}{whether to return counts instead of means}

\item{weight_threshold}{numeric; if provided, weights above this threshold
are capped to the threshold value and then re-normalised (default NULL)}
}
\value{
a list with elements \code{matrix} and \code{participants}
}
\description{
Computes a contact matrix from a \code{contact_survey} that has been processed
by \code{\link[=assign_age_groups]{assign_age_groups()}} and optionally \code{\link[=weigh]{weigh()}}. This is the final step
in the pipeline workflow.

For post-processing, pipe the result into \code{\link[=symmetrise]{symmetrise()}},
\code{\link[=split_matrix]{split_matrix()}}, or \code{\link[=per_capita]{per_capita()}}.
}
\examples{
data(polymod)
polymod |>
  assign_age_groups(age_limits = c(0, 5, 15)) |>
  compute_matrix()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-contact-survey.R
\name{copy_survey}
\alias{copy_survey}
\title{Deep copy a contact survey}
\usage{
copy_survey(survey)
}
\arguments{
\item{survey}{a \code{contact_survey} object}
}
\value{
a deep copy of the survey
}
\description{
Creates a deep copy of a \code{contact_survey} object, including its
\code{participants} and \code{contacts} data.tables.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postprocess-matrix.R
\name{per_capita}
\alias{per_capita}
\title{Convert a contact matrix to per-capita rates}
\usage{
per_capita(x, survey_pop, ...)
}
\arguments{
\item{x}{a list as returned by \code{\link[=compute_matrix]{compute_matrix()}}, with elements \code{matrix}
and \code{participants}}

\item{survey_pop}{a data frame with columns \code{lower.age.limit} and
\code{population} (e.g. from \code{\link[=wpp_age]{wpp_age()}})}

\item{...}{passed to \code{\link[=pop_age]{pop_age()}} for interpolation}
}
\value{
\code{x} with \verb{$matrix} replaced by the per-capita version
}
\description{
Divides each column of the contact matrix by the population of the
corresponding age group, giving the contact rate of age group i with
one individual of age group j.
}
\examples{
data(polymod)
pop <- wpp_age("United Kingdom", 2005)
polymod |>
  (\(s) s[country == "United Kingdom"])() |>
  assign_age_groups(age_limits = c(0, 5, 15)) |>
  compute_matrix() |>
  per_capita(survey_pop = pop)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contact-matrix-result.R
\name{new_contact_matrix}
\alias{new_contact_matrix}
\title{Create a contact_matrix object}
\usage{
new_contact_matrix(matrix, participants, ...)
}
\arguments{
\item{matrix}{a numeric matrix with age group dimnames}

\item{participants}{a data.frame with columns \code{age.group}, \code{participants},
\code{proportion}}

\item{...}{additional named elements (e.g. \code{mean.contacts}, \code{normalisation},
\code{contacts} from \code{\link[=split_matrix]{split_matrix()}})}
}
\value{
a \code{contact_matrix} object (an S3 class inheriting from \code{list})
}
\description{
Create a contact_matrix object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limits_to_agegroups.R
\name{limits_to_agegroups}
\alias{limits_to_agegroups}
\title{Convert lower age limits to age groups.}
\usage{
limits_to_agegroups(
  x,
  limits = sort(unique(x)),
  notation = c("dashes", "brackets")
)
}
\arguments{
\item{x}{age limits to transform}

\item{limits}{lower age limits; if not given, will use all limits in \code{x}}

\item{notation}{whether to use bracket notation, e.g. [0,4) or dash
notation, e.g. 0-4)}
}
\value{
Age groups as specified in \code{notation}
}
\description{
Mostly used for plot labelling
}
\examples{
limits_to_agegroups(c(0, 5, 10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix_plot.R
\name{matrix_plot}
\alias{matrix_plot}
\title{Draws an image plot of a contact matrix with a legend strip
and the numeric values in the cells.}
\usage{
matrix_plot(
  mij,
  min.legend = 0,
  max.legend = NA,
  num.digits = 2,
  num.colors = 50,
  main,
  xlab,
  ylab,
  legend.width,
  legend.mar,
  legend.shrink,
  cex.lab,
  cex.axis,
  cex.text,
  color.palette = heat.colors
)
}
\arguments{
\item{mij}{a contact matrix containing contact rates between
participants of age i (rows) with contacts of age j
(columns). This is the default matrix format of
\code{\link[=contact_matrix]{contact_matrix()}}.}

\item{min.legend}{the color scale minimum (default = 0). Set
to NA to use the minimum value of \code{mij}.}

\item{max.legend}{the color scale maximum (default = NA). Set
to NA to use the maximum value of \code{mij}.}

\item{num.digits}{the number of digits when rounding the
contact rates (default = 2). Use NA to disable this.}

\item{num.colors}{the number of color breaks (default = 50)}

\item{main}{the figure title}

\item{xlab}{a title for the x axis (default: "Age group
(year)")}

\item{ylab}{a title for the y axis (default: "Contact age
group (year)")}

\item{legend.width}{width of the legend strip in characters.
Default is 1.}

\item{legend.mar}{width in characters of legend margin.
Default is 5.1.}

\item{legend.shrink}{amount to shrink the size of legend
relative to the full height or width of the plot. Default
is 0.9.}

\item{cex.lab}{size of the x and y labels (default: 1.2)}

\item{cex.axis}{size of the axis labels (default: 0.8)}

\item{cex.text}{size of the numeric values in the matrix
(default: 1)}

\item{color.palette}{the color palette to use (default:
\code{\link[=heat.colors]{heat.colors()}}). Other examples are \code{\link[=topo.colors]{topo.colors()}},
\code{\link[=terrain.colors]{terrain.colors()}} and \code{\link[=hcl.colors]{hcl.colors()}}. User-defined
functions are also possible if they take the number of
colors to be in the palette as function argument.}
}
\description{
This function combines the R image.plot function with numeric
contact rates in the matrix cells.
}
\details{
This is a function using basic R graphics to
visualise a social contact matrix.
}
\examples{
\dontrun{
data(polymod)
mij <- contact_matrix(
  polymod,
  countries = "United Kingdom",
  age_limits = c(0, 18, 65)
)$matrix
matrix_plot(mij)
}
}
\author{
Lander Willem
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean}
\alias{clean}
\alias{clean.contact_survey}
\title{Clean contact survey data}
\usage{
\method{clean}{contact_survey}(
  x,
  participant_age_column = "part_age",
  ...,
  participant.age.column = deprecated()
)
}
\arguments{
\item{x}{A \code{\link[=survey]{survey()}} object}

\item{participant_age_column}{the column in \code{x$participants}
containing participants' age}

\item{...}{ignored}

\item{participant.age.column}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
Use \code{participant_age_column} instead.}
}
\value{
a cleaned survey in the correct format
}
\description{
Cleans survey data to work with the 'contact_matrix' function
}
\examples{
data(polymod)
cleaned <- clean(polymod) # not really necessary, polymod is clean
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postprocess-matrix.R
\name{symmetrise}
\alias{symmetrise}
\title{Symmetrise a contact matrix}
\usage{
symmetrise(x, survey_pop, symmetric_norm_threshold = 2, ...)
}
\arguments{
\item{x}{a list as returned by \code{\link[=compute_matrix]{compute_matrix()}}, with elements \code{matrix}
and \code{participants}}

\item{survey_pop}{a data frame with columns \code{lower.age.limit} and
\code{population} (e.g. from \code{\link[=wpp_age]{wpp_age()}})}

\item{symmetric_norm_threshold}{threshold for the normalisation factor
before issuing a warning (default 2)}

\item{...}{passed to \code{\link[=pop_age]{pop_age()}} for interpolation}
}
\value{
\code{x} with \verb{$matrix} replaced by the symmetrised version
}
\description{
Makes a contact matrix symmetric so that \eqn{c_{ij} N_i = c_{ji} N_j},
where \eqn{c_{ij}} is the (i, j) entry and \eqn{N_i} is the population
of age group i. This is done by replacing each pair with half their sum,
weighted by population size.
}
\examples{
data(polymod)
pop <- wpp_age("United Kingdom", 2005)
polymod |>
  (\(s) s[country == "United Kingdom"])() |>
  assign_age_groups(age_limits = c(0, 5, 15)) |>
  compute_matrix() |>
  symmetrise(survey_pop = pop)

}
